@@ -13,14 +13,16 @@
 
 /*
  * This file specified the sheepdog-internal protocol, which is spoken between
- * sheepdog daemons, as well as between collie and sheepdog daemon for internal
+ * sheepdog daemons, as well as between dog and sheepdog daemon for internal
  * operations.
  */
 
 #include <stdint.h>
 #include <netinet/in.h>
 
-#define SD_SHEEP_PROTO_VER 0x07
+#include "sheepdog_proto.h"
+
+#define SD_SHEEP_PROTO_VER 0x08
 
 #define SD_DEFAULT_COPIES 3
 #define SD_MAX_COPIES 8
@@ -31,7 +33,7 @@
 
 /*
  * Operations with opcodes above 0x80 are considered part of the inter-sheep
- * include sheep-collie protocol and are versioned using SD_SHEEP_PROTO_VER
+ * include sheep-dog protocol and are versioned using SD_SHEEP_PROTO_VER
  * instead of SD_PROTO_VER.
  *
  * These same applies for the above 0x80 flags and error values below.
@@ -48,12 +50,14 @@
 #define SD_OP_RESTORE        0x92
 #define SD_OP_GET_SNAP_FILE  0x93
 #define SD_OP_CLEANUP        0x94
-#define SD_OP_TRACE          0x95
+#define SD_OP_TRACE_STATUS   0x95
 #define SD_OP_TRACE_READ_BUF 0x96
 #define SD_OP_STAT_RECOVERY  0x97
 #define SD_OP_FLUSH_DEL_CACHE  0x98
 #define SD_OP_NOTIFY_VDI_DEL 0x99
 #define SD_OP_KILL_NODE      0x9A
+#define SD_OP_TRACE_ENABLE   0x9B
+#define SD_OP_TRACE_DISABLE  0x9C
 #define SD_OP_GET_OBJ_LIST   0xA1
 #define SD_OP_GET_EPOCH      0xA2
 #define SD_OP_CREATE_AND_WRITE_PEER 0xA3
@@ -74,7 +78,7 @@
 #define SD_OP_MD_UNPLUG 0xB3
 #define SD_OP_GET_HASH       0xB4
 #define SD_OP_REWEIGHT       0xB5
-#define SD_OP_UPDATE_SIZE    0xB6
+#define SD_OP_GET_CACHE_INFO 0xB6
 
 /* internal flags for hdr.flags, must be above 0x80 */
 #define SD_FLAG_CMD_RECOVERY 0x0080
@@ -85,8 +89,8 @@
 #define SD_FLAG_CMD_DEL      0x0400
 
 /* internal error return values, must be above 0x80 */
-#define SD_RES_OLD_NODE_VER  0x81 /* Remote node has an old epoch */
-#define SD_RES_NEW_NODE_VER  0x82 /* Remote node has a new epoch */
+#define SD_RES_OLD_NODE_VER  0x81 /* Request has an old epoch */
+#define SD_RES_NEW_NODE_VER  0x82 /* Request has a new epoch */
 #define SD_RES_NOT_FORMATTED 0x83 /* Sheepdog is not formatted yet */
 #define SD_RES_INVALID_CTIME 0x84 /* Creation time of sheepdog is different */
 #define SD_RES_INVALID_EPOCH 0x85 /* Invalid epoch */
@@ -101,16 +105,14 @@
 #define SD_RES_OID_EXIST        0x8E /* Object ID exists already */
 #define SD_RES_AGAIN            0x8F /* Ask to try again */
 #define SD_RES_STALE_OBJ        0x90 /* Object may be stale */
+#define SD_RES_CLUSTER_ERROR    0x91 /* Cluster driver error */
 
-#define SD_FLAG_NOHALT       0x0004 /* Serve the IO rquest even lack of nodes */
-#define SD_FLAG_QUORUM       0x0008 /* Serve the IO rquest as long we are quorate */
-
-#define SD_STATUS_OK                0x00000001
-#define SD_STATUS_WAIT_FOR_FORMAT   0x00000002
-#define SD_STATUS_WAIT_FOR_JOIN     0x00000004
-#define SD_STATUS_SHUTDOWN          0x00000008
-#define SD_STATUS_HALT              0x00000020
-#define SD_STATUS_KILLED            0x00000040
+enum sd_status {
+	SD_STATUS_OK = 1,
+	SD_STATUS_WAIT,
+	SD_STATUS_SHUTDOWN,
+	SD_STATUS_KILLED,
+};
 
 struct node_id {
 	uint8_t addr[16];
@@ -129,6 +131,26 @@ struct sd_node {
 	uint64_t        space;
 };
 
+/*
+ * A joining sheep multicasts the local cluster info.  Then, the existing nodes
+ * reply the latest cluster info which is unique among all of the nodes.
+ */
+struct cluster_info {
+	uint8_t proto_ver; /* the version number of the internal protocol */
+	uint8_t disable_recovery;
+	int16_t nr_nodes;
+	uint32_t epoch;
+	uint64_t ctime;
+	uint16_t flags;
+	uint8_t nr_copies;
+	enum sd_status status : 8;
+	uint32_t __pad;
+	uint8_t store[STORE_LEN];
+
+	/* node list at cluster_info->epoch */
+	struct sd_node nodes[SD_MAX_NODES];
+};
+
 struct epoch_log {
 	uint64_t ctime;
 	uint64_t time;		/* treated as time_t */
@@ -139,31 +161,6 @@ struct epoch_log {
 	struct sd_node nodes[SD_MAX_NODES];
 };
 
-struct join_message {
-	uint8_t proto_ver;
-	uint8_t nr_copies;
-	int16_t nr_nodes;
-	uint16_t nr_failed_nodes;
-	uint16_t nr_delayed_nodes;
-	uint32_t cluster_status;
-	uint32_t epoch;
-	uint64_t ctime;
-	uint8_t inc_epoch; /* set non-zero when we increment epoch of all nodes */
-	uint8_t disable_recovery;
-	uint16_t cluster_flags;
-	uint32_t __pad;
-	uint8_t store[STORE_LEN];
-
-	/*
-	 * A joining sheep puts the local node list here, which is nr_nodes
-	 * entries long.  After the master replies it will contain the list of
-	 * nodes that attempted to join but failed the join process.  The
-	 * number of entries in that case is nr_failed_nodes, which by
-	 * defintion must be smaller than nr_nodes.
-	 */
-	struct sd_node nodes[];
-};
-
 struct vdi_op_message {
 	struct sd_req req;
 	struct sd_rsp rsp;
@@ -183,30 +180,37 @@ struct sd_md_info {
 	int nr;
 };
 
-enum cluster_join_result {
-	/* Success */
-	CJ_RES_SUCCESS,
-
-	/* Fail to join. The joining node has an invalid epoch. */
-	CJ_RES_FAIL,
-
-	/*
-	 * Fail to join. The joining node should be added after the cluster
-	 * start working.
-	 */
-	CJ_RES_JOIN_LATER,
-
-	/*
-	 * Transfer mastership.  The joining node has a newer epoch, so this
-	 * node will leave the cluster (restart later).
-	 */
-	CJ_RES_MASTER_TRANSFER,
-};
-
 static inline __attribute__((used)) void __sd_epoch_format_build_bug_ons(void)
 {
 	/* never called, only for checking BUILD_BUG_ON()s */
 	BUILD_BUG_ON(sizeof(struct sd_node) != SD_NODE_SIZE);
 }
 
+enum rw_state {
+	RW_PREPARE_LIST, /* the recovery thread is preparing object list */
+	RW_RECOVER_OBJ, /* the thread is recoering objects */
+	RW_NOTIFY_COMPLETION, /* the thread is notifying recovery completion */
+};
+
+struct recovery_state {
+	uint8_t in_recovery;
+	enum rw_state state;
+	uint64_t nr_finished;
+	uint64_t nr_total;
+};
+
+#define CACHE_MAX	1024
+struct cache_info {
+	uint32_t vid;
+	uint32_t dirty;
+	uint32_t total;
+};
+
+struct object_cache_info {
+	uint64_t size;
+	uint64_t used;
+	struct cache_info caches[CACHE_MAX];
+	int count;
+};
+
 #endif /* __INTERNAL_PROTO_H__ */
