@@ -49,20 +49,20 @@ int conn_rx_off(struct connection *conn);
 int conn_rx_on(struct connection *conn);
 bool is_conn_dead(const struct connection *conn);
 int do_read(int sockfd, void *buf, int len,
-	    bool (*need_retry)(uint32_t), uint32_t);
+	    bool (*need_retry)(uint32_t), uint32_t, uint32_t);
 int rx(struct connection *conn, enum conn_state next_state);
 int tx(struct connection *conn, enum conn_state next_state);
 int connect_to(const char *name, int port);
 int send_req(int sockfd, struct sd_req *hdr, void *data, unsigned int wlen,
-	     bool (*need_retry)(uint32_t), uint32_t);
+	     bool (*need_retry)(uint32_t), uint32_t, uint32_t);
 int exec_req(int sockfd, struct sd_req *hdr, void *,
-	     bool (*need_retry)(uint32_t), uint32_t);
+	     bool (*need_retry)(uint32_t), uint32_t, uint32_t);
 int create_listen_ports(const char *bindaddr, int port,
 			int (*callback)(int fd, void *), void *data);
 int create_unix_domain_socket(const char *unix_path,
 			      int (*callback)(int, void *), void *data);
 
-char *addr_to_str(char *str, int size, const uint8_t *addr, uint16_t port);
+const char *addr_to_str(const uint8_t *addr, uint16_t port);
 uint8_t *str_to_addr(const char *ipstr, uint8_t *addr);
 char *sockaddr_in_to_str(struct sockaddr_in *sockaddr);
 int set_nonblocking(int fd);
@@ -80,10 +80,7 @@ int do_writev2(int fd, void *hdr, size_t hdr_len, void *body, size_t body_len);
 
 static inline int connect_to_addr(const uint8_t *addr, int port)
 {
-	char name[INET6_ADDRSTRLEN];
-
-	addr_to_str(name, sizeof(name), addr, 0);
-	return connect_to(name, port);
+	return connect_to(addr_to_str(addr, 0), port);
 }
 
 #endif
