@@ -14,15 +14,19 @@
 #include <string.h>
 #include <inttypes.h>
 
+#define SHA1_DIGEST_SIZE        20
+#define SHA1_BLOCK_SIZE         64
+
 struct sha1_ctx {
 	uint64_t count;
-	uint32_t state[5];
-	uint8_t buffer[64];
+	uint32_t state[SHA1_DIGEST_SIZE / 4];
+	uint8_t buffer[SHA1_BLOCK_SIZE];
 };
 
 void sha1_init(void *ctx);
 void sha1_update(void *ctx, const uint8_t *data, unsigned int len);
 void sha1_final(void *ctx, uint8_t *out);
 const char *sha1_to_hex(const unsigned char *sha1);
+void sha1_from_buffer(const void *buf, size_t size, unsigned char *sha1);
 
 #endif
