@@ -14,9 +14,9 @@
 #include <stdint.h>
 #include "internal_proto.h"
 #include "util.h"
+#include "bitops.h"
 #include "list.h"
 #include "net.h"
-#include "logger.h"
 
 struct sd_vnode {
 	struct node_id  nid;
@@ -33,7 +33,7 @@ struct vnode_info {
 	int nr_nodes;
 
 	int nr_zones;
-	int refcnt;
+	refcnt_t refcnt;
 };
 
 #define TRACE_GRAPH_ENTRY  0x01
@@ -47,8 +47,8 @@ struct trace_graph_item {
 	int type;
 	char fname[TRACE_FNAME_LEN];
 	int depth;
-	unsigned long long entry_time;
-	unsigned long long return_time;
+	uint64_t entry_time;
+	uint64_t return_time;
 };
 
 static inline void sd_init_req(struct sd_req *req, uint8_t opcode)
@@ -70,6 +70,8 @@ static inline int get_vnode_first_idx(const struct sd_vnode *entries,
 	uint64_t id = fnv_64a_buf(&oid, sizeof(oid), FNV1A_64_INIT);
 	int start, end, pos;
 
+	assert(nr_entries > 0);
+
 	start = 0;
 	end = nr_entries - 1;
 
@@ -99,7 +101,7 @@ static inline int get_vnode_next_idx(const struct sd_vnode *entries,
 	idx = prev_idxs[nr_prev_idxs - 1];
 	for (;;) {
 		idx = (idx + 1) % nr_entries;
-		if (idx == first_idx)
+		if (unlikely(idx == first_idx))
 			panic("can't find next new idx");
 
 		for (found = false, i = 0; i < nr_prev_idxs; i++) {
@@ -143,12 +145,27 @@ static inline const struct sd_vnode *oid_to_vnode(const struct sd_vnode *entries
 	return &entries[idx];
 }
 
+static inline const struct sd_node *oid_to_node(const struct sd_vnode *entries,
+						int nr_entries, uint64_t oid,
+						int copy_idx,
+						const struct sd_node *all_nodes)
+{
+	const struct sd_vnode *vnode;
+
+	vnode = oid_to_vnode(entries, nr_entries, oid, copy_idx);
+
+	return &all_nodes[vnode->node_idx];
+}
+
 static inline void oid_to_vnodes(const struct sd_vnode *entries, int nr_entries,
 				 uint64_t oid, int nr_copies,
 				 const struct sd_vnode **vnodes)
 {
 	int idx, idxs[SD_MAX_COPIES], i;
 
+	if (nr_entries == 0)
+		return;
+
 	idx = get_vnode_first_idx(entries, nr_entries, oid);
 	idxs[0] = idx;
 	vnodes[0] = &entries[idx];
@@ -202,12 +219,12 @@ static inline const char *sd_strerror(int err)
 		[SD_RES_WAIT_FOR_FORMAT] = "Waiting for cluster to be formatted",
 		[SD_RES_WAIT_FOR_JOIN] = "Waiting for other nodes to join cluster",
 		[SD_RES_JOIN_FAILED] = "Node has failed to join cluster",
-		[SD_RES_HALT] = "IO has halted as there are too few living nodes",
+		[SD_RES_HALT] = "IO has halted as there are no living nodes",
 		[SD_RES_READONLY] = "Object is read-only",
 
 		/* from internal_proto.h */
-		[SD_RES_OLD_NODE_VER] = "Remote node has an old epoch",
-		[SD_RES_NEW_NODE_VER] = "Remote node has a new epoch",
+		[SD_RES_OLD_NODE_VER] = "Request has an old epoch",
+		[SD_RES_NEW_NODE_VER] = "Request has a new epoch",
 		[SD_RES_NOT_FORMATTED] = "Cluster has not been formatted",
 		[SD_RES_INVALID_CTIME] = "Creation times differ",
 		[SD_RES_INVALID_EPOCH] = "Invalid epoch",
@@ -222,6 +239,7 @@ static inline const char *sd_strerror(int err)
 		[SD_RES_OID_EXIST] = "Object ID exists already",
 		[SD_RES_AGAIN] = "Ask to try again",
 		[SD_RES_STALE_OBJ] = "Object may be stale",
+		[SD_RES_CLUSTER_ERROR] = "Cluster driver error",
 	};
 
 	if (descs[err] == NULL) {
@@ -240,11 +258,7 @@ static inline int node_id_cmp(const struct node_id *node1,
 	if (cmp != 0)
 		return cmp;
 
-	if (node1->port < node2->port)
-		return -1;
-	if (node1->port > node2->port)
-		return 1;
-	return 0;
+	return intcmp(node1->port, node2->port);
 }
 
 static inline int node_cmp(const struct sd_node *node1,
@@ -261,11 +275,7 @@ static inline bool node_eq(const struct sd_node *a, const struct sd_node *b)
 static inline int vnode_cmp(const struct sd_vnode *node1,
 			    const struct sd_vnode *node2)
 {
-	if (node1->id < node2->id)
-		return -1;
-	if (node1->id > node2->id)
-		return 1;
-	return 0;
+	return intcmp(node1->id, node2->id);
 }
 
 static inline int nodes_to_vnodes(struct sd_node *nodes, int nr,
@@ -305,10 +315,9 @@ static inline int nodes_to_vnodes(struct sd_node *nodes, int nr,
 
 #define MAX_NODE_STR_LEN 256
 
-static inline char *node_to_str(const struct sd_node *id)
+static inline const char *node_to_str(const struct sd_node *id)
 {
-	static char str[MAX_NODE_STR_LEN];
-	char name[MAX_NODE_STR_LEN];
+	static __thread char str[MAX_NODE_STR_LEN];
 	int af = AF_INET6;
 	const uint8_t *addr = id->nid.addr;
 
@@ -323,7 +332,7 @@ static inline char *node_to_str(const struct sd_node *id)
 
 	snprintf(str, sizeof(str), "%s ip:%s port:%d",
 		(af == AF_INET) ? "IPv4" : "IPv6",
-		addr_to_str(name, sizeof(name), id->nid.addr, 0), id->nid.port);
+		addr_to_str(id->nid.addr, 0), id->nid.port);
 
 	return str;
 }
