@@ -13,9 +13,10 @@
 
 #include <inttypes.h>
 #include <stdint.h>
+#include <stdbool.h>
 #include <linux/limits.h>
 
-#include "util.h"
+#include "compiler.h"
 
 #define SD_PROTO_VER 0x02
 
@@ -179,7 +180,7 @@ struct sd_rsp {
 			uint32_t	__pad;
 			uint32_t	nr_nodes;
 			uint32_t	local_idx;
-			uint32_t	master_idx;
+			uint32_t	__reserved;
 			uint64_t	store_size;
 			uint64_t	store_free;
 		} node;
@@ -221,15 +222,6 @@ struct sheepdog_vdi_attr {
 	char value[SD_MAX_VDI_ATTR_VALUE_LEN];
 };
 
-#define SHA1_LEN        20
-
-struct snap_log {
-	uint32_t idx;
-	char tag[SD_MAX_SNAPSHOT_TAG_LEN];
-	uint64_t time;
-	unsigned char sha1[SHA1_LEN];
-};
-
 /* 64 bit FNV-1a non-zero initial basis */
 #define FNV1A_64_INIT ((uint64_t) 0xcbf29ce484222325ULL)
 
@@ -316,6 +308,11 @@ static inline uint64_t vid_to_attr_oid(uint32_t vid, uint32_t attrid)
 	return ((uint64_t)vid << VDI_SPACE_SHIFT) | VDI_ATTR_BIT | attrid;
 }
 
+static inline uint64_t vid_to_vmstate_oid(uint32_t vid, uint32_t idx)
+{
+	return VMSTATE_BIT | ((uint64_t)vid << VDI_SPACE_SHIFT) | idx;
+}
+
 static inline bool vdi_is_snapshot(const struct sd_inode *inode)
 {
 	return !!inode->snap_ctime;
