@@ -7,7 +7,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "logger.h"
+#include "util.h"
 
 struct strbuf {
 	size_t alloc;
