@@ -19,11 +19,9 @@
  */
 #include <arpa/inet.h>
 #include "sha1.h"
+#include "util.h"
 
-#define SHA1_DIGEST_SIZE	20
-#define SHA1_HMAC_BLOCK_SIZE	64
-
-static inline uint32_t rol(uint32_t value, uint32_t bits)
+static __always_inline uint32_t rol(uint32_t value, uint32_t bits)
 {
 	return (value << bits) | (value >> (32 - bits));
 }
@@ -129,13 +127,13 @@ static void sha1_transform(uint32_t *state, const uint8_t *in)
 void sha1_init(void *ctx)
 {
 	struct sha1_ctx *sctx = ctx;
-	static const struct sha1_ctx initstate = {
+	static const struct sha1_ctx init_state = {
 	  0,
 	  { 0x67452301, 0xEFCDAB89, 0x98BADCFE, 0x10325476, 0xC3D2E1F0 },
 	  { 0, }
 	};
 
-	*sctx = initstate;
+	*sctx = init_state;
 }
 
 void sha1_update(void *ctx, const uint8_t *data, unsigned int len)
@@ -212,3 +210,32 @@ const char *sha1_to_hex(const unsigned char *sha1)
 	}
 	return buffer;
 }
+
+/*
+ * Calculate a sha1 message digest based on the content of 'buf'
+ *
+ * We can uniquely generate the original buffer from
+ * - the trimmed buffer
+ * - the orignal buffer length
+ * - the trimmed buffer length
+ * - the trimmed buffer offset
+ *
+ * This calculates a unique sha1 digest faster than the naive calculation when
+ * the content of 'buf' is sparse.  The result will be set in 'sha1'.
+ */
+void sha1_from_buffer(const void *buf, size_t size, unsigned char *sha1)
+{
+	struct sha1_ctx c;
+	uint64_t offset = 0;
+	uint32_t length = size;
+
+	sha1_init(&c);
+	sha1_update(&c, (uint8_t *)&length, sizeof(length));
+
+	find_zero_blocks(buf, &offset, &length);
+
+	sha1_update(&c, (uint8_t *)&length, sizeof(length));
+	sha1_update(&c, (uint8_t *)&offset, sizeof(offset));
+	sha1_update(&c, buf, length);
+	sha1_final(&c, sha1);
+}
