@@ -21,7 +21,7 @@
 
 #include "sheepdog_proto.h"
 #include "sheep.h"
-#include "logger.h"
+#include "config.h"
 
 /* maximum payload size sent in ->notify and ->unblock */
 #define SD_MAX_EVENT_BUF_SIZE (128 * 1024) /* 128k */
@@ -51,12 +51,12 @@ struct cluster_driver {
 	 *
 	 * This function is used to join the cluster, and notifies a join
 	 * event to all the nodes.  The copy of 'opaque' is passed to
-	 * sd_check_join_cb() and sd_join_handler().
+	 * sd_join_handler() and sd_accept_handler().
 	 *
-	 * sd_check_join_cb() is called on one of the nodes which already
-	 * paticipate in the cluster.  If the content of 'opaque' is
-	 * changed in sd_check_join_cb(), the updated 'opaque' must be
-	 * passed to sd_join_handler().
+	 * sd_join_handler() must be called on at least one node which already
+	 * paticipates in the cluster.  If the content of 'opaque' is changed in
+	 * sd_join_handler(), the updated 'opaque' must be passed to
+	 * sd_accept_handler().
 	 *
 	 * Returns zero on success, -1 on error
 	 */
@@ -82,7 +82,7 @@ struct cluster_driver {
 	 * can be read through sd_notify_handler() and totally ordered with
 	 * node change events.
 	 *
-	 * Returns zero on success, -1 on error
+	 * Returns SD_RES_XXX
 	 */
 	int (*notify)(void *msg, size_t msg_len);
 
@@ -92,23 +92,37 @@ struct cluster_driver {
 	 * Once the cluster driver has ensured that events are blocked on all
 	 * nodes it needs to call sd_block_handler() on the node where ->block
 	 * was called.
+	 *
+	 * Returns SD_RES_XXX
 	 */
-	void (*block)(void);
+	int (*block)(void);
 
 	/*
 	 * Unblock events on all nodes, and send a total order message
 	 * to all nodes.
+	 *
+	 * Returns SD_RES_XXX
 	 */
-	void (*unblock)(void *msg, size_t msg_len);
+	int (*unblock)(void *msg, size_t msg_len);
 
-	/* Update the specific node in the driver's private copy of nodes */
-	void (*update_node)(struct sd_node *);
+	/*
+	 * Update the specific node in the driver's private copy of nodes
+	 *
+	 * Returns SD_RES_XXX
+	 */
+	int (*update_node)(struct sd_node *);
 
 	struct list_head list;
 };
 
 extern struct list_head cluster_drivers;
 
+#ifdef HAVE_COROSYNC
+#define DEFAULT_CLUSTER_DRIVER "corosync"
+#else
+#define DEFAULT_CLUSTER_DRIVER "local"
+#endif /* HAVE_COROSYNC */
+
 #define cdrv_register(driver)						\
 static void __attribute__((constructor)) regist_ ## driver(void)	\
 {									\
@@ -148,16 +162,18 @@ static inline const char *get_cdrv_option(const struct cluster_driver *cdrv,
 }
 
 /* callbacks back into sheepdog from the cluster drivers */
-void sd_join_handler(const struct sd_node *joined,
-		     const struct sd_node *members,
-		     size_t nr_members, enum cluster_join_result result,
-		     const void *opaque);
+void sd_accept_handler(const struct sd_node *joined,
+		       const struct sd_node *members, size_t nr_members,
+		       const void *opaque);
 void sd_leave_handler(const struct sd_node *left, const struct sd_node *members,
 		      size_t nr_members);
 void sd_notify_handler(const struct sd_node *sender, void *msg, size_t msg_len);
 bool sd_block_handler(const struct sd_node *sender);
-enum cluster_join_result sd_check_join_cb(const struct sd_node *joining,
-					  void *opaque);
+int sd_reconnect_handler(void);
+void sd_update_node_handler(struct sd_node *);
+bool sd_join_handler(const struct sd_node *joining,
+		     const struct sd_node *nodes, size_t nr_nodes,
+		     void *opaque);
 void recalculate_vnodes(struct sd_node *nodes, int nr_nodes);
 
 #endif
