@@ -24,8 +24,7 @@
 struct cpg_node {
 	uint32_t nodeid;
 	uint32_t pid;
-	uint32_t gone;
-	struct sd_node ent;
+	struct sd_node node;
 };
 
 static cpg_handle_t cpg_handle;
@@ -46,21 +45,23 @@ static size_t nr_majority; /* used for network partition detection */
 
 /* event types which are dispatched in corosync_dispatch() */
 enum corosync_event_type {
-	COROSYNC_EVENT_TYPE_JOIN_REQUEST,
-	COROSYNC_EVENT_TYPE_JOIN_RESPONSE,
+	COROSYNC_EVENT_TYPE_JOIN,
+	COROSYNC_EVENT_TYPE_ACCEPT,
 	COROSYNC_EVENT_TYPE_LEAVE,
 	COROSYNC_EVENT_TYPE_BLOCK,
 	COROSYNC_EVENT_TYPE_NOTIFY,
+	COROSYNC_EVENT_TYPE_UPDATE_NODE,
 };
 
 /* multicast message type */
 enum corosync_message_type {
-	COROSYNC_MSG_TYPE_JOIN_REQUEST,
-	COROSYNC_MSG_TYPE_JOIN_RESPONSE,
+	COROSYNC_MSG_TYPE_JOIN,
+	COROSYNC_MSG_TYPE_ACCEPT,
 	COROSYNC_MSG_TYPE_LEAVE,
 	COROSYNC_MSG_TYPE_NOTIFY,
 	COROSYNC_MSG_TYPE_BLOCK,
 	COROSYNC_MSG_TYPE_UNBLOCK,
+	COROSYNC_MSG_TYPE_UPDATE_NODE,
 };
 
 struct corosync_event {
@@ -70,7 +71,6 @@ struct corosync_event {
 	void *msg;
 	size_t msg_len;
 
-	enum cluster_join_result result;
 	uint32_t nr_nodes;
 	struct cpg_node nodes[SD_MAX_NODES];
 
@@ -81,29 +81,24 @@ struct corosync_event {
 
 struct corosync_message {
 	struct cpg_node sender;
-	enum corosync_message_type type:4;
-	enum cluster_join_result result:4;
+	enum corosync_message_type type:16;
+	uint16_t nr_nodes;
 	uint32_t msg_len;
-	uint32_t nr_nodes;
 	struct cpg_node nodes[SD_MAX_NODES];
 	uint8_t msg[0];
 };
 
-static bool cpg_node_equal(struct cpg_node *a, struct cpg_node *b)
+static int cpg_node_cmp(struct cpg_node *a, struct cpg_node *b)
 {
-	return (a->nodeid == b->nodeid && a->pid == b->pid);
+	int cmp = intcmp(a->nodeid, b->nodeid);
+	if (cmp == 0)
+		cmp = intcmp(a->pid, b->pid);
+	return cmp;
 }
 
-static inline int find_cpg_node(struct cpg_node *nodes, size_t nr_nodes,
-				struct cpg_node *key)
+static bool cpg_node_equal(struct cpg_node *a, struct cpg_node *b)
 {
-	int i;
-
-	for (i = 0; i < nr_nodes; i++)
-		if (cpg_node_equal(nodes + i, key))
-			return i;
-
-	return -1;
+	return cpg_node_cmp(a, b) == 0;
 }
 
 static inline int find_sd_node(struct cpg_node *nodes, size_t nr_nodes,
@@ -112,7 +107,7 @@ static inline int find_sd_node(struct cpg_node *nodes, size_t nr_nodes,
 	int i;
 
 	for (i = 0; i < nr_nodes; i++)
-		if (node_eq(&nodes[i].ent, key))
+		if (node_eq(&nodes[i].node, key))
 			return i;
 
 	return -1;
@@ -127,16 +122,7 @@ static inline void add_cpg_node(struct cpg_node *nodes, size_t nr_nodes,
 static inline void del_cpg_node(struct cpg_node *nodes, size_t nr_nodes,
 				struct cpg_node *deled)
 {
-	int idx;
-
-	idx = find_cpg_node(nodes, nr_nodes, deled);
-	if (idx < 0) {
-		sd_dprintf("cannot find node");
-		return;
-	}
-
-	nr_nodes--;
-	memmove(nodes + idx, nodes + idx + 1, sizeof(*nodes) * (nr_nodes - idx));
+	xlremove(deled, nodes, &nr_nodes, cpg_node_cmp);
 }
 
 static int corosync_get_local_addr(uint8_t *addr)
@@ -151,12 +137,12 @@ static int corosync_get_local_addr(uint8_t *addr)
 	ret = corosync_cfg_get_node_addrs(cfg_handle, this_node.nodeid, 1,
 					  &nr, &caddr);
 	if (ret != CS_OK) {
-		sd_printf(SDOG_ERR, "failed to get node addresses (%d)", ret);
+		sd_err("failed to get node addresses (%d)", ret);
 		return -1;
 	}
 
 	if (!nr) {
-		sd_printf(SDOG_ERR, "no node addresses found");
+		sd_err("no node addresses found");
 		return -1;
 	}
 
@@ -168,7 +154,7 @@ static int corosync_get_local_addr(uint8_t *addr)
 		memset(addr, 0, 16);
 		memcpy(addr + 12, saddr, 4);
 	} else {
-		sd_printf(SDOG_ERR, "unknown protocol %d", ss->ss_family);
+		sd_err("unknown protocol %d", ss->ss_family);
 		return -1;
 	}
 
@@ -176,7 +162,6 @@ static int corosync_get_local_addr(uint8_t *addr)
 }
 
 static int send_message(enum corosync_message_type type,
-			enum cluster_join_result result,
 			struct cpg_node *sender, struct cpg_node *nodes,
 			size_t nr_nodes, void *msg, size_t msg_len)
 {
@@ -185,7 +170,6 @@ static int send_message(enum corosync_message_type type,
 	struct corosync_message cmsg = {
 		.type = type,
 		.msg_len = msg_len,
-		.result = result,
 		.sender = *sender,
 		.nr_nodes = nr_nodes,
 	};
@@ -206,14 +190,14 @@ retry:
 	case CS_OK:
 		break;
 	case CS_ERR_TRY_AGAIN:
-		sd_dprintf("failed to send message: retrying");
+		sd_debug("failed to send message: retrying");
 		sleep(1);
 		goto retry;
 	default:
-		sd_eprintf("failed to send message (%d)", ret);
-		return -1;
+		sd_err("failed to send message (%d)", ret);
+		return SD_RES_CLUSTER_ERROR;
 	}
-	return 0;
+	return SD_RES_SUCCESS;
 }
 
 static inline struct corosync_event *
@@ -253,35 +237,13 @@ find_event(enum corosync_event_type type, struct cpg_node *sender)
 		return find_nonblock_event(type, sender);
 }
 
-static int is_master(struct cpg_node *node)
-{
-	int i;
-	struct cpg_node *n = node;
-	if (!n)
-		n = &this_node;
-	if (nr_cpg_nodes == 0)
-		/* this node should be the first cpg node */
-		return 0;
-
-	for (i = 0; i < SD_MAX_NODES; i++) {
-		if (!cpg_nodes[i].gone)
-			goto eq_check;
-	}
-	return -1;
-
-eq_check:
-	if (cpg_node_equal(&cpg_nodes[i], n))
-		return i;
-	return -1;
-}
-
 static void build_node_list(const struct cpg_node *nodes, size_t nr_nodes,
 			    struct sd_node *entries)
 {
 	int i;
 
 	for (i = 0; i < nr_nodes; i++)
-		entries[i] = nodes[i].ent;
+		entries[i] = nodes[i].node;
 }
 
 /*
@@ -291,66 +253,49 @@ static void build_node_list(const struct cpg_node *nodes, size_t nr_nodes,
  */
 static bool __corosync_dispatch_one(struct corosync_event *cevent)
 {
-	enum cluster_join_result res;
-	struct sd_node entries[SD_MAX_NODES];
+	struct sd_node entries[SD_MAX_NODES], *node;
+	struct cpg_node *n;
 	int idx;
 
 	switch (cevent->type) {
-	case COROSYNC_EVENT_TYPE_JOIN_REQUEST:
-		if (is_master(&this_node) < 0)
-			return false;
-
+	case COROSYNC_EVENT_TYPE_JOIN:
 		if (!cevent->msg)
-			/* we haven't receive JOIN_REQUEST yet */
+			/* we haven't receive JOIN yet */
 			return false;
 
 		if (cevent->callbacked)
-			/* check_join() must be called only once */
+			/* sd_join_handler() must be called only once */
 			return false;
 
-		res = sd_check_join_cb(&cevent->sender.ent,
-						     cevent->msg);
-		if (res == CJ_RES_MASTER_TRANSFER)
-			nr_cpg_nodes = 0;
+		build_node_list(cpg_nodes, nr_cpg_nodes, entries);
+		if (sd_join_handler(&cevent->sender.node, entries,
+				    nr_cpg_nodes, cevent->msg)) {
+			send_message(COROSYNC_MSG_TYPE_ACCEPT, &cevent->sender,
+				     cpg_nodes, nr_cpg_nodes, cevent->msg,
+				     cevent->msg_len);
 
-		send_message(COROSYNC_MSG_TYPE_JOIN_RESPONSE, res,
-			     &cevent->sender, cpg_nodes, nr_cpg_nodes,
-			     cevent->msg, cevent->msg_len);
-
-		if (res == CJ_RES_MASTER_TRANSFER) {
-			sd_eprintf("failed to join sheepdog cluster:"
-				   " please retry when master is up");
-			exit(1);
+			cevent->callbacked = true;
 		}
-
-		cevent->callbacked = true;
 		return false;
-	case COROSYNC_EVENT_TYPE_JOIN_RESPONSE:
-		switch (cevent->result) {
-		case CJ_RES_SUCCESS:
-		case CJ_RES_MASTER_TRANSFER:
-		case CJ_RES_JOIN_LATER:
-			add_cpg_node(cpg_nodes, nr_cpg_nodes, &cevent->sender);
-			nr_cpg_nodes++;
-			/* fall through */
-		case CJ_RES_FAIL:
-			build_node_list(cpg_nodes, nr_cpg_nodes, entries);
-			sd_join_handler(&cevent->sender.ent, entries,
-					nr_cpg_nodes, cevent->result,
-					cevent->msg);
-			break;
-		}
+	case COROSYNC_EVENT_TYPE_ACCEPT:
+		add_cpg_node(cpg_nodes, nr_cpg_nodes, &cevent->sender);
+		nr_cpg_nodes++;
+
+		build_node_list(cpg_nodes, nr_cpg_nodes, entries);
+		sd_accept_handler(&cevent->sender.node, entries, nr_cpg_nodes,
+				  cevent->msg);
 		break;
 	case COROSYNC_EVENT_TYPE_LEAVE:
-		idx = find_cpg_node(cpg_nodes, nr_cpg_nodes, &cevent->sender);
-		if (idx < 0)
+		n = xlfind(&cevent->sender, cpg_nodes, nr_cpg_nodes,
+			   cpg_node_cmp);
+		if (n == NULL)
 			break;
-		cevent->sender.ent = cpg_nodes[idx].ent;
+		cevent->sender.node = n->node;
 
 		del_cpg_node(cpg_nodes, nr_cpg_nodes, &cevent->sender);
 		nr_cpg_nodes--;
 		build_node_list(cpg_nodes, nr_cpg_nodes, entries);
-		sd_leave_handler(&cevent->sender.ent, entries, nr_cpg_nodes);
+		sd_leave_handler(&cevent->sender.node, entries, nr_cpg_nodes);
 		break;
 	case COROSYNC_EVENT_TYPE_BLOCK:
 		if (cevent->callbacked)
@@ -359,17 +304,55 @@ static bool __corosync_dispatch_one(struct corosync_event *cevent)
 			 * removes this event
 			 */
 			return false;
-		cevent->callbacked = sd_block_handler(&cevent->sender.ent);
+		cevent->callbacked = sd_block_handler(&cevent->sender.node);
 		return false;
 	case COROSYNC_EVENT_TYPE_NOTIFY:
-		sd_notify_handler(&cevent->sender.ent, cevent->msg,
+		sd_notify_handler(&cevent->sender.node, cevent->msg,
 						 cevent->msg_len);
 		break;
+	case COROSYNC_EVENT_TYPE_UPDATE_NODE:
+		node = &cevent->sender.node;
+
+		if (cpg_node_equal(&cevent->sender, &this_node))
+			this_node = cevent->sender;
+
+		idx = find_sd_node(cpg_nodes, nr_cpg_nodes, node);
+		assert(idx >= 0);
+		cpg_nodes[idx].node = *node;
+		sd_update_node_handler(node);
+		break;
 	}
 
 	return true;
 }
 
+static bool update_join_status(struct corosync_event *cevent)
+{
+	if (join_finished)
+		return true;
+
+	switch (cevent->type) {
+	case COROSYNC_EVENT_TYPE_JOIN:
+		if (self_elect) {
+			nr_cpg_nodes = 0;
+			return true;
+		}
+		break;
+	case COROSYNC_EVENT_TYPE_ACCEPT:
+		if (cpg_node_equal(&cevent->sender, &this_node)) {
+			nr_cpg_nodes = cevent->nr_nodes;
+			memcpy(cpg_nodes, cevent->nodes,
+			       sizeof(*cevent->nodes) * cevent->nr_nodes);
+			return true;
+		}
+		break;
+	default:
+		break;
+	}
+
+	return false;
+}
+
 static void __corosync_dispatch(void)
 {
 	struct corosync_event *cevent;
@@ -385,7 +368,7 @@ static void __corosync_dispatch(void)
 		 * number of alive nodes correctly, we postpone
 		 * processsing events if there are incoming ones.
 		 */
-		sd_dprintf("wait for a next dispatch event");
+		sd_debug("wait for a next dispatch event");
 		return;
 	}
 
@@ -400,36 +383,14 @@ static void __corosync_dispatch(void)
 			cevent = list_first_entry(&corosync_block_event_list,
 						  typeof(*cevent), list);
 
-		/* update join status */
-		if (!join_finished) {
-			switch (cevent->type) {
-			case COROSYNC_EVENT_TYPE_JOIN_REQUEST:
-				if (self_elect) {
-					join_finished = true;
-					nr_cpg_nodes = 0;
-				}
-				break;
-			case COROSYNC_EVENT_TYPE_JOIN_RESPONSE:
-				if (cpg_node_equal(&cevent->sender,
-						   &this_node)) {
-					join_finished = true;
-					nr_cpg_nodes = cevent->nr_nodes;
-					memcpy(cpg_nodes, cevent->nodes,
-					       sizeof(*cevent->nodes) *
-					       cevent->nr_nodes);
-				}
-				break;
-			default:
-				break;
-			}
-		}
+		join_finished = update_join_status(cevent);
 
 		if (join_finished) {
 			if (!__corosync_dispatch_one(cevent))
 				return;
 		} else {
 			switch (cevent->type) {
-			case COROSYNC_MSG_TYPE_JOIN_REQUEST:
+			case COROSYNC_MSG_TYPE_JOIN:
 			case COROSYNC_MSG_TYPE_BLOCK:
 				return;
 			default:
@@ -483,14 +444,13 @@ static void cdrv_cpg_deliver(cpg_handle_t handle,
 {
 	struct corosync_event *cevent;
 	struct corosync_message *cmsg = msg;
-	int master;
 
-	sd_dprintf("%d", cmsg->type);
+	sd_debug("%d", cmsg->type);
 
 	switch (cmsg->type) {
-	case COROSYNC_MSG_TYPE_JOIN_REQUEST:
-		cevent = update_event(COROSYNC_EVENT_TYPE_JOIN_REQUEST,
-				      &cmsg->sender, cmsg->msg, cmsg->msg_len);
+	case COROSYNC_MSG_TYPE_JOIN:
+		cevent = update_event(COROSYNC_EVENT_TYPE_JOIN, &cmsg->sender,
+				      cmsg->msg, cmsg->msg_len);
 		if (!cevent)
 			break;
 
@@ -508,11 +468,19 @@ static void cdrv_cpg_deliver(cpg_handle_t handle,
 		/* fall through */
 	case COROSYNC_MSG_TYPE_BLOCK:
 	case COROSYNC_MSG_TYPE_NOTIFY:
+	case COROSYNC_MSG_TYPE_UPDATE_NODE:
 		cevent = xzalloc(sizeof(*cevent));
-		if (cmsg->type == COROSYNC_MSG_TYPE_BLOCK)
+		switch (cmsg->type) {
+		case COROSYNC_MSG_TYPE_BLOCK:
 			cevent->type = COROSYNC_EVENT_TYPE_BLOCK;
-		else
+			break;
+		case COROSYNC_MSG_TYPE_UPDATE_NODE:
+			cevent->type = COROSYNC_EVENT_TYPE_UPDATE_NODE;
+			break;
+		default:
 			cevent->type = COROSYNC_EVENT_TYPE_NOTIFY;
+			break;
+		}
 
 		cevent->sender = cmsg->sender;
 		cevent->msg_len = cmsg->msg_len;
@@ -527,16 +495,6 @@ static void cdrv_cpg_deliver(cpg_handle_t handle,
 	case COROSYNC_MSG_TYPE_LEAVE:
 		cevent = xzalloc(sizeof(*cevent));
 		cevent->type = COROSYNC_EVENT_TYPE_LEAVE;
-
-		master = is_master(&cmsg->sender);
-		if (master >= 0)
-			/*
-			 * Master is down before new nodes finish joining.
-			 * We have to revoke its mastership to avoid cluster
-			 * hanging
-			 */
-			cpg_nodes[master].gone = 1;
-
 		cevent->sender = cmsg->sender;
 		cevent->msg_len = cmsg->msg_len;
 		if (cmsg->msg_len) {
@@ -547,14 +505,13 @@ static void cdrv_cpg_deliver(cpg_handle_t handle,
 
 		queue_event(cevent);
 		break;
-	case COROSYNC_MSG_TYPE_JOIN_RESPONSE:
-		cevent = update_event(COROSYNC_EVENT_TYPE_JOIN_REQUEST,
-				      &cmsg->sender, cmsg->msg, cmsg->msg_len);
+	case COROSYNC_MSG_TYPE_ACCEPT:
+		cevent = update_event(COROSYNC_EVENT_TYPE_JOIN, &cmsg->sender,
+				      cmsg->msg, cmsg->msg_len);
 		if (!cevent)
 			break;
 
-		cevent->type = COROSYNC_EVENT_TYPE_JOIN_RESPONSE;
-		cevent->result = cmsg->result;
+		cevent->type = COROSYNC_EVENT_TYPE_ACCEPT;
 		cevent->nr_nodes = cmsg->nr_nodes;
 		memcpy(cevent->nodes, cmsg->nodes,
 		       sizeof(*cmsg->nodes) * cmsg->nr_nodes);
@@ -566,7 +523,7 @@ static void cdrv_cpg_deliver(cpg_handle_t handle,
 }
 
 static void build_cpg_node_list(struct cpg_node *nodes,
-		const struct cpg_address *list, size_t nr)
+				const struct cpg_address *list, size_t nr)
 {
 	int i;
 
@@ -592,8 +549,8 @@ static void cdrv_cpg_confchg(cpg_handle_t handle,
 	struct cpg_node left_sheep[SD_MAX_NODES];
 	bool promote = true;
 
-	sd_dprintf("mem:%zu, joined:%zu, left:%zu", member_list_entries,
-		   joined_list_entries, left_list_entries);
+	sd_debug("mem:%zu, joined:%zu, left:%zu", member_list_entries,
+		 joined_list_entries, left_list_entries);
 
 	/* check network partition */
 	if (left_list_entries) {
@@ -621,9 +578,7 @@ static void cdrv_cpg_confchg(cpg_handle_t handle,
 
 	/* dispatch leave_handler */
 	for (i = 0; i < left_list_entries; i++) {
-		int master;
-		cevent = find_event(COROSYNC_EVENT_TYPE_JOIN_REQUEST,
-				    left_sheep + i);
+		cevent = find_event(COROSYNC_EVENT_TYPE_JOIN, left_sheep + i);
 		if (cevent) {
 			/* the node left before joining */
 			list_del(&cevent->list);
@@ -641,15 +596,6 @@ static void cdrv_cpg_confchg(cpg_handle_t handle,
 		}
 
 		cevent = xzalloc(sizeof(*cevent));
-		master = is_master(&left_sheep[i]);
-		if (master >= 0)
-			/*
-			 * Master is down before new nodes finish joining.
-			 * We have to revoke its mastership to avoid cluster
-			 * hanging
-			 */
-			cpg_nodes[master].gone = 1;
-
 		cevent->type = COROSYNC_EVENT_TYPE_LEAVE;
 		cevent->sender = left_sheep[i];
 
@@ -659,7 +605,7 @@ static void cdrv_cpg_confchg(cpg_handle_t handle,
 	/* dispatch join_handler */
 	for (i = 0; i < joined_list_entries; i++) {
 		cevent = xzalloc(sizeof(*cevent));
-		cevent->type = COROSYNC_EVENT_TYPE_JOIN_REQUEST;
+		cevent->type = COROSYNC_EVENT_TYPE_JOIN;
 		cevent->sender = joined_sheep[i];
 		queue_event(cevent);
 	}
@@ -670,11 +616,11 @@ static void cdrv_cpg_confchg(cpg_handle_t handle,
 		 * all other members, because events are ordered.
 		 */
 		for (i = 0; i < member_list_entries; i++) {
-			cevent = find_event(COROSYNC_EVENT_TYPE_JOIN_REQUEST,
+			cevent = find_event(COROSYNC_EVENT_TYPE_JOIN,
 					    &member_sheep[i]);
 			if (!cevent) {
-				sd_dprintf("Not promoting because member is "
-					   "not in our event list.");
+				sd_debug("Not promoting because member is not "
+					 "in our event list.");
 				promote = false;
 				break;
 			}
@@ -701,47 +647,47 @@ retry:
 	case CS_OK:
 		break;
 	case CS_ERR_TRY_AGAIN:
-		sd_dprintf("failed to join the sheepdog group: retrying");
+		sd_debug("failed to join the sheepdog group: retrying");
 		sleep(1);
 		goto retry;
 	case CS_ERR_SECURITY:
-		sd_eprintf("permission denied to join the sheepdog group");
+		sd_err("permission denied to join the sheepdog group");
 		return -1;
 	default:
-		sd_eprintf("failed to join the sheepdog group (%d)", ret);
+		sd_err("failed to join the sheepdog group (%d)", ret);
 		return -1;
 	}
 
-	this_node.ent = *myself;
+	this_node.node = *myself;
 
-	ret = send_message(COROSYNC_MSG_TYPE_JOIN_REQUEST, 0, &this_node,
-			   NULL, 0, opaque, opaque_len);
+	ret = send_message(COROSYNC_MSG_TYPE_JOIN, &this_node, NULL, 0, opaque,
+			   opaque_len);
 
 	return ret;
 }
 
 static int corosync_leave(void)
 {
-	return send_message(COROSYNC_MSG_TYPE_LEAVE, 0, &this_node, NULL, 0,
-			    NULL, 0);
+	return send_message(COROSYNC_MSG_TYPE_LEAVE, &this_node, NULL, 0, NULL,
+			    0);
 }
 
-static void corosync_block(void)
+static int corosync_block(void)
 {
-	send_message(COROSYNC_MSG_TYPE_BLOCK, 0, &this_node, NULL, 0,
-			    NULL, 0);
+	return send_message(COROSYNC_MSG_TYPE_BLOCK, &this_node, NULL, 0, NULL,
+			    0);
 }
 
-static void corosync_unblock(void *msg, size_t msg_len)
+static int corosync_unblock(void *msg, size_t msg_len)
 {
-	send_message(COROSYNC_MSG_TYPE_UNBLOCK, 0, &this_node, NULL, 0,
-		     msg, msg_len);
+	return send_message(COROSYNC_MSG_TYPE_UNBLOCK, &this_node, NULL, 0, msg,
+			    msg_len);
 }
 
 static int corosync_notify(void *msg, size_t msg_len)
 {
-	return send_message(COROSYNC_MSG_TYPE_NOTIFY, 0, &this_node,
-			   NULL, 0, msg, msg_len);
+	return send_message(COROSYNC_MSG_TYPE_NOTIFY, &this_node, NULL, 0, msg,
+			    msg_len);
 }
 
 static void corosync_handler(int listen_fd, int events, void *data)
@@ -749,13 +695,13 @@ static void corosync_handler(int listen_fd, int events, void *data)
 	int ret;
 
 	if (events & EPOLLHUP) {
-		sd_eprintf("corosync driver received EPOLLHUP event, exiting.");
+		sd_err("corosync driver received EPOLLHUP event, exiting.");
 		goto out;
 	}
 
 	ret = cpg_dispatch(cpg_handle, CS_DISPATCH_ALL);
 	if (ret != CS_OK) {
-		sd_eprintf("cpg_dispatch returned %d", ret);
+		sd_err("cpg_dispatch returned %d", ret);
 		goto out;
 	}
 
@@ -782,28 +728,30 @@ again:
 		break;
 	case CS_ERR_TRY_AGAIN:
 		if (retry_cnt++ == CPG_INIT_RETRY_CNT) {
-			sd_eprintf("failed to initialize cpg (%d) - "
-				   "is corosync running?", ret);
+			sd_err("failed to initialize cpg (%d) - "
+			       "is corosync running?", ret);
 			return -1;
 		}
-		sd_dprintf("retry cpg_initialize");
+		sd_debug("retry cpg_initialize");
 		usleep(200000);
 		goto again;
+	case CS_ERR_SECURITY:
+		sd_err("failed to initialize cpg - permission denied");
+		return -1;
 	default:
-		sd_eprintf("failed to initialize cpg (%d) - "
-			   "is corosync running?", ret);
+		sd_err("failed to initialize cpg (%d)", ret);
 		return -1;
 	}
 
 	ret = corosync_cfg_initialize(&cfg_handle, NULL);
 	if (ret != CS_OK) {
-		sd_printf(SDOG_ERR, "failed to initialize cfg (%d)", ret);
+		sd_err("failed to initialize cfg (%d)", ret);
 		return -1;
 	}
 
 	ret = corosync_cfg_local_get(cfg_handle, &nodeid);
 	if (ret != CS_OK) {
-		sd_printf(SDOG_ERR, "failed to get node id (%d)", ret);
+		sd_err("failed to get node id (%d)", ret);
 		return -1;
 	}
 
@@ -812,28 +760,27 @@ again:
 
 	ret = cpg_fd_get(cpg_handle, &cpg_fd);
 	if (ret != CS_OK) {
-		sd_eprintf("failed to get cpg file descriptor (%d)", ret);
+		sd_err("failed to get cpg file descriptor (%d)", ret);
 		return -1;
 	}
 
 	ret = register_event(cpg_fd, corosync_handler, NULL);
 	if (ret) {
-		sd_eprintf("failed to register corosync event handler (%d)",
-			   ret);
+		sd_err("failed to register corosync event handler (%d)", ret);
 		return -1;
 	}
 
 	return 0;
 }
 
-static void corosync_update_node(struct sd_node *node)
+static int corosync_update_node(struct sd_node *node)
 {
-	int idx;
+	struct cpg_node cnode = this_node;
 
-	idx = find_sd_node(cpg_nodes, nr_cpg_nodes, node);
-	if (idx < 0)
-		return;
-	cpg_nodes[idx].ent = *node;
+	cnode.node = *node;
+
+	return send_message(COROSYNC_MSG_TYPE_UPDATE_NODE, &cnode, NULL, 0,
+			    NULL, 0);
 }
 
 static struct cluster_driver cdrv_corosync = {
