@@ -24,7 +24,7 @@
 #include "util.h"
 
 #define MAX_EVENTS 500
-#define PROCESS_CHECK_INTERVAL 200 /* ms */
+#define PROCESS_CHECK_INTERVAL 50 /* ms */
 
 static const char *shmfile = "/tmp/sheepdog_shm";
 static int shmfd;
@@ -40,24 +40,29 @@ struct local_node {
 	bool gateway;
 };
 
-static char *lnode_to_str(struct local_node *lnode)
+static const char *lnode_to_str(struct local_node *lnode)
 {
-	char *s = node_to_str(&lnode->node);
+	static __thread char s[MAX_NODE_STR_LEN + 32];
 
-	snprintf(s + strlen(s), MAX_NODE_STR_LEN - strlen(s), " pid:%d",
+	snprintf(s, sizeof(s), "%s pid:%d", node_to_str(&lnode->node),
 		 lnode->pid);
 
 	return s;
 }
 
+static int lnode_cmp(const struct local_node *a, const struct local_node *b)
+{
+	return node_cmp(&a->node, &b->node);
+}
+
 static bool lnode_eq(const struct local_node *a, const struct local_node *b)
 {
-	return node_eq(&a->node, &b->node);
+	return lnode_cmp(a, b) == 0;
 }
 
 enum local_event_type {
-	EVENT_JOIN_REQUEST = 1,
-	EVENT_JOIN_RESPONSE,
+	EVENT_JOIN = 1,
+	EVENT_ACCEPT,
 	EVENT_LEAVE,
 	EVENT_GATEWAY,
 	EVENT_BLOCK,
@@ -77,8 +82,6 @@ struct local_event {
 
 	size_t nr_lnodes; /* the number of sheep processes */
 	struct local_node lnodes[SD_MAX_NODES];
-
-	enum cluster_join_result join_result;
 };
 
 
@@ -180,7 +183,7 @@ static void shm_queue_notify(void)
 	nr = get_nodes(lnodes);
 
 	for (i = 0; i < nr; i++) {
-		sd_dprintf("send signal to %s", lnode_to_str(lnodes + i));
+		sd_debug("send signal to %s", lnode_to_str(lnodes + i));
 		kill(lnodes[i].pid, SIGUSR1);
 	}
 }
@@ -213,7 +216,7 @@ static void shm_queue_init(void)
 
 	shm_queue_lock();
 
-	ret = ftruncate(shmfd, sizeof(*shm_queue));
+	ret = xftruncate(shmfd, sizeof(*shm_queue));
 	if (ret != 0)
 		panic("failed to truncate shmfile, %m");
 
@@ -229,10 +232,10 @@ static void shm_queue_init(void)
 		/* initialize shared memory */
 		block_event_pos = 0;
 		nonblock_event_pos = 0;
-		ret = ftruncate(shmfd, 0);
+		ret = xftruncate(shmfd, 0);
 		if (ret != 0)
 			panic("failed to truncate shmfile, %m");
-		ret = ftruncate(shmfd, sizeof(*shm_queue));
+		ret = xftruncate(shmfd, sizeof(*shm_queue));
 		if (ret != 0)
 			panic("failed to truncate shmfile, %m");
 	}
@@ -240,22 +243,9 @@ static void shm_queue_init(void)
 	shm_queue_unlock();
 }
 
-static struct local_node *find_lnode(struct local_node *key, size_t nr_lnodes,
-				     struct local_node *lnodes)
+static int add_event(enum local_event_type type, struct local_node *lnode,
+		      void *buf, size_t buf_len)
 {
-	int i;
-
-	for (i = 0; i < nr_lnodes; i++)
-		if (lnode_eq(key, lnodes + i))
-			return lnodes + i;
-
-	panic("internal error");
-}
-
-static void add_event(enum local_event_type type, struct local_node *lnode,
-		void *buf, size_t buf_len)
-{
-	int idx, i;
 	struct local_node *n;
 	struct local_event ev = {
 		.type = type,
@@ -269,39 +259,48 @@ static void add_event(enum local_event_type type, struct local_node *lnode,
 	ev.nr_lnodes = get_nodes(ev.lnodes);
 
 	switch (type) {
-	case EVENT_JOIN_REQUEST:
+	case EVENT_JOIN:
 		ev.lnodes[ev.nr_lnodes] = *lnode;
 		ev.nr_lnodes++;
 		break;
 	case EVENT_LEAVE:
-		n = find_lnode(lnode, ev.nr_lnodes, ev.lnodes);
-		idx = n - ev.lnodes;
-
-		ev.nr_lnodes--;
-		memmove(n, n + 1, sizeof(*n) * (ev.nr_lnodes - idx));
+		xlremove(lnode, ev.lnodes, &ev.nr_lnodes, lnode_cmp);
 		break;
 	case EVENT_GATEWAY:
-		n = find_lnode(lnode, ev.nr_lnodes, ev.lnodes);
+		n = xlfind(lnode, ev.lnodes, ev.nr_lnodes, lnode_cmp);
 		n->gateway = true;
 		break;
 	case EVENT_NOTIFY:
 	case EVENT_BLOCK:
 		break;
 	case EVENT_UPDATE_NODE:
-		n = find_lnode(lnode, ev.nr_lnodes, ev.lnodes);
+		n = xlfind(lnode, ev.lnodes, ev.nr_lnodes, lnode_cmp);
 		n->node = lnode->node;
 		break;
-	case EVENT_JOIN_RESPONSE:
+	case EVENT_ACCEPT:
 		abort();
 	}
 
-	sd_dprintf("type = %d, sender = %s", ev.type, lnode_to_str(&ev.sender));
-	for (i = 0; i < ev.nr_lnodes; i++)
-		sd_dprintf("%d: %s", i, lnode_to_str(ev.lnodes + i));
+	sd_debug("type = %d, sender = %s", ev.type, lnode_to_str(&ev.sender));
+	for (int i = 0; i < ev.nr_lnodes; i++)
+		sd_debug("%d: %s", i, lnode_to_str(ev.lnodes + i));
 
 	shm_queue_push(&ev);
 
 	shm_queue_notify();
+
+	return SD_RES_SUCCESS;
+}
+
+static int add_event_lock(enum local_event_type type, struct local_node *lnode,
+			  void *buf, size_t buf_len)
+{
+	int ret;
+
+	shm_queue_lock();
+	ret = add_event(type, lnode, buf, buf_len);
+	shm_queue_unlock();
+	return ret;
 }
 
 static void check_pids(void *arg)
@@ -342,47 +341,26 @@ static int local_join(const struct sd_node *myself,
 	this_node.pid = getpid();
 	this_node.gateway = false;
 
-	shm_queue_lock();
-
-	add_event(EVENT_JOIN_REQUEST, &this_node, opaque, opaque_len);
-
-	shm_queue_unlock();
-
-	return 0;
+	return add_event_lock(EVENT_JOIN, &this_node, opaque, opaque_len);
 }
 
 static int local_leave(void)
 {
-	shm_queue_lock();
-
-	add_event(EVENT_GATEWAY, &this_node, NULL, 0);
-
-	shm_queue_unlock();
-
-	return 0;
+	return add_event_lock(EVENT_GATEWAY, &this_node, NULL, 0);
 }
 
 static int local_notify(void *msg, size_t msg_len)
 {
-	shm_queue_lock();
 
-	add_event(EVENT_NOTIFY, &this_node, msg, msg_len);
-
-	shm_queue_unlock();
-
-	return 0;
+	return add_event_lock(EVENT_NOTIFY, &this_node, msg, msg_len);
 }
 
-static void local_block(void)
+static int local_block(void)
 {
-	shm_queue_lock();
-
-	add_event(EVENT_BLOCK, &this_node, NULL, 0);
-
-	shm_queue_unlock();
+	return add_event_lock(EVENT_BLOCK, &this_node, NULL, 0);
 }
 
-static void local_unblock(void *msg, size_t msg_len)
+static int local_unblock(void *msg, size_t msg_len)
 {
 	struct local_event *ev;
 
@@ -396,13 +374,14 @@ static void local_unblock(void *msg, size_t msg_len)
 	add_event(EVENT_NOTIFY, &this_node, msg, msg_len);
 
 	shm_queue_unlock();
+
+	return SD_RES_SUCCESS;
 }
 
 /* Returns true if an event is processed */
 static bool local_process_event(void)
 {
 	struct local_event *ev;
-	enum cluster_join_result res;
 	int i;
 	struct sd_node nodes[SD_MAX_NODES];
 	size_t nr_nodes;
@@ -411,14 +390,12 @@ static bool local_process_event(void)
 	if (!ev)
 		return false;
 
-	sd_dprintf("type = %d, sender = %s", ev->type,
-		   lnode_to_str(&ev->sender));
-	sd_dprintf("callbacked = %d, removed = %d", ev->callbacked,
-		   ev->removed);
+	sd_debug("type = %d, sender = %s", ev->type, lnode_to_str(&ev->sender));
+	sd_debug("callbacked = %d, removed = %d", ev->callbacked, ev->removed);
 
 	nr_nodes = 0;
 	for (i = 0; i < ev->nr_lnodes; i++) {
-		sd_dprintf("%d: %s", i, lnode_to_str(ev->lnodes + i));
+		sd_debug("%d: %s", i, lnode_to_str(ev->lnodes + i));
 		if (!ev->lnodes[i].gateway)
 			nodes[nr_nodes++] = ev->lnodes[i].node;
 	}
@@ -429,62 +406,42 @@ static bool local_process_event(void)
 	if (ev->callbacked)
 		return false; /* wait for unblock event */
 
-	if (ev->type == EVENT_JOIN_RESPONSE &&
-	    lnode_eq(&this_node, &ev->sender)) {
-		sd_dprintf("join Sheepdog");
-		joined = true;
-	}
-
 	if (!joined) {
-		if (ev->type == EVENT_JOIN_REQUEST &&
-		    lnode_eq(&this_node, &ev->sender)) {
-			struct local_node lnodes[SD_MAX_NODES];
-
-			get_nodes(lnodes);
-
-			if (!lnode_eq(&this_node, &lnodes[0])) {
-				sd_dprintf("wait for another node"
-					   " to accept this node");
-				return false;
-			}
-		} else
+		if (!lnode_eq(&this_node, &ev->sender))
 			goto out;
+
+		switch (ev->type) {
+		case EVENT_JOIN:
+			break;
+		case EVENT_ACCEPT:
+			sd_debug("join Sheepdog");
+			joined = true;
+			break;
+		default:
+			goto out;
+		}
 	}
 
 	switch (ev->type) {
-	case EVENT_JOIN_REQUEST:
-		res = sd_check_join_cb(&ev->sender.node, ev->buf);
-		ev->join_result = res;
-		ev->type = EVENT_JOIN_RESPONSE;
-		msync(ev, sizeof(*ev), MS_SYNC);
-
-		shm_queue_notify();
-
-		if (res == CJ_RES_MASTER_TRANSFER) {
-			sd_eprintf("failed to join sheepdog cluster: "
-				   "please retry when master is up");
-			shm_queue_unlock();
-			exit(1);
-		}
-		return false;
-	case EVENT_JOIN_RESPONSE:
-		if (ev->join_result == CJ_RES_MASTER_TRANSFER) {
-			/* FIXME: This code is tricky, but Sheepdog assumes that */
-			/* nr_nodes = 1 when join_result = MASTER_TRANSFER... */
-			ev->nr_lnodes = 1;
-			ev->lnodes[0] = this_node;
-			nr_nodes = 1;
-			nodes[0] = this_node.node;
+	case EVENT_JOIN:
+		/* nodes[nr_nodes - 1] is a sender, so don't include it */
+		assert(node_eq(&ev->sender.node, &nodes[nr_nodes - 1]));
+		if (sd_join_handler(&ev->sender.node, nodes, nr_nodes - 1,
+				      ev->buf)) {
+			ev->type = EVENT_ACCEPT;
 			msync(ev, sizeof(*ev), MS_SYNC);
+
+			shm_queue_notify();
 		}
 
-		sd_join_handler(&ev->sender.node, nodes, nr_nodes,
-				ev->join_result, ev->buf);
+		return false;
+	case EVENT_ACCEPT:
+		sd_accept_handler(&ev->sender.node, nodes, nr_nodes, ev->buf);
 		break;
 	case EVENT_LEAVE:
 		if (ev->sender.gateway) {
-			sd_dprintf("gateway %s left sheepdog",
-				   lnode_to_str(&ev->sender));
+			sd_debug("gateway %s left sheepdog",
+				 lnode_to_str(&ev->sender));
 			break;
 		}
 		/* fall through */
@@ -499,6 +456,10 @@ static bool local_process_event(void)
 		sd_notify_handler(&ev->sender.node, ev->buf, ev->buf_len);
 		break;
 	case EVENT_UPDATE_NODE:
+		if (lnode_eq(&ev->sender, &this_node))
+			this_node = ev->sender;
+
+		sd_update_node_handler(&ev->sender.node);
 		break;
 	}
 out:
@@ -513,12 +474,12 @@ static void local_handler(int listen_fd, int events, void *data)
 	int ret;
 
 	if (events & EPOLLHUP) {
-		sd_eprintf("local driver received EPOLLHUP event, exiting.");
+		sd_err("local driver received EPOLLHUP event, exiting.");
 		log_close();
 		exit(1);
 	}
 
-	sd_dprintf("read siginfo");
+	sd_debug("read siginfo");
 
 	ret = read(sigfd, &siginfo, sizeof(siginfo));
 	if (ret != sizeof(siginfo))
@@ -561,7 +522,7 @@ static int local_init(const char *option)
 
 	sigfd = signalfd(-1, &mask, SFD_NONBLOCK);
 	if (sigfd < 0) {
-		sd_eprintf("failed to create a signal fd: %m");
+		sd_err("failed to create a signal fd: %m");
 		return -1;
 	}
 
@@ -569,25 +530,20 @@ static int local_init(const char *option)
 
 	ret = register_event(sigfd, local_handler, NULL);
 	if (ret) {
-		sd_eprintf("failed to register local event handler (%d)", ret);
+		sd_err("failed to register local event handler (%d)", ret);
 		return -1;
 	}
 
 	return 0;
 }
 
-/* FIXME: we have to call nr of nodes times to update nodes information */
-static void local_update_node(struct sd_node *node)
+static int local_update_node(struct sd_node *node)
 {
-	struct local_node n = {
-		.node = *node,
-	};
+	struct local_node lnode = this_node;
 
-	shm_queue_lock();
+	lnode.node = *node;
 
-	add_event(EVENT_UPDATE_NODE, &n, NULL, 0);
-
-	shm_queue_unlock();
+	return add_event_lock(EVENT_UPDATE_NODE, &lnode, NULL, 0);
 }
 
 static struct cluster_driver cdrv_local = {
