@@ -19,6 +19,7 @@
 #include <pthread.h>
 
 #include "cluster.h"
+#include "config.h"
 #include "event.h"
 #include "work.h"
 #include "util.h"
@@ -33,20 +34,20 @@
 
 /* iterate child znodes */
 #define FOR_EACH_ZNODE(parent, path, strs)			       \
-	for (zk_get_children(parent, strs),		               \
-		     (strs)->data += (strs)->count;		       \
+	for ((strs)->data += (strs)->count;			       \
 	     (strs)->count-- ?					       \
 		     snprintf(path, sizeof(path), "%s/%s", parent,     \
 			      *--(strs)->data) : (free((strs)->data), 0); \
 	     free(*(strs)->data))
 
 enum zk_event_type {
-	EVENT_JOIN_REQUEST = 1,
-	EVENT_JOIN_RESPONSE,
+	EVENT_JOIN = 1,
+	EVENT_ACCEPT,
 	EVENT_LEAVE,
 	EVENT_BLOCK,
 	EVENT_UNBLOCK,
 	EVENT_NOTIFY,
+	EVENT_UPDATE_NODE,
 };
 
 struct zk_node {
@@ -61,7 +62,6 @@ struct zk_event {
 	uint64_t id;
 	enum zk_event_type type;
 	struct zk_node sender;
-	enum cluster_join_result join_result;
 	size_t msg_len;
 	size_t nr_nodes;
 	size_t buf_len;
@@ -71,8 +71,15 @@ struct zk_event {
 static struct sd_node sd_nodes[SD_MAX_NODES];
 static size_t nr_sd_nodes;
 static struct rb_root zk_node_root = RB_ROOT;
-static pthread_rwlock_t zk_tree_lock = PTHREAD_RWLOCK_INITIALIZER;
+static struct sd_lock zk_tree_lock = SD_LOCK_INITIALIZER;
+static struct sd_lock zk_compete_master_lock = SD_LOCK_INITIALIZER;
 static LIST_HEAD(zk_block_list);
+static uatomic_bool is_master;
+static uatomic_bool stop;
+static bool joined;
+static bool first_push = true;
+
+static void zk_compete_master(void);
 
 static struct zk_node *zk_tree_insert(struct zk_node *new)
 {
@@ -125,9 +132,9 @@ static inline struct zk_node *zk_tree_search(const struct node_id *nid)
 {
 	struct zk_node *n;
 
-	pthread_rwlock_rdlock(&zk_tree_lock);
+	sd_read_lock(&zk_tree_lock);
 	n = zk_tree_search_nolock(nid);
-	pthread_rwlock_unlock(&zk_tree_lock);
+	sd_unlock(&zk_tree_lock);
 	return n;
 }
 
@@ -135,18 +142,60 @@ static inline struct zk_node *zk_tree_search(const struct node_id *nid)
 static zhandle_t *zhandle;
 static struct zk_node this_node;
 
+#define CHECK_ZK_RC(rc, path)						\
+	switch (rc) {							\
+	case ZNONODE:							\
+	case ZNODEEXISTS:						\
+		break;							\
+	case ZINVALIDSTATE:						\
+	case ZSESSIONEXPIRED:						\
+	case ZOPERATIONTIMEOUT:						\
+	case ZCONNECTIONLOSS:						\
+		sd_err("failed, path:%s, %s", path, zerror(rc));	\
+	case ZOK:							\
+		break;							\
+	case ZNOCHILDRENFOREPHEMERALS:					\
+		/*							\
+		 * Because code has guaranteed that parent nodes are	\
+		 * always non-ephemeral, this could happen only when	\
+		 * sheep joins a cluster in an incompatible version.	\
+		 */							\
+		sd_err("incompatible version of sheep %s",		\
+		       PACKAGE_VERSION);				\
+	default:							\
+		panic("failed, path:%s, %s", path, zerror(rc));		\
+	}
+#define RETURN_IF_ERROR(stmt, fmt, ...)					\
+	do {								\
+		int __rc = stmt;					\
+		if (__rc != ZOK) {					\
+			sd_err("failed, " fmt ", %s",			\
+			       ##__VA_ARGS__, zerror(__rc));		\
+			return __rc;					\
+		}							\
+	} while (0)
+#define RETURN_VOID_IF_ERROR(stmt, fmt, ...)				\
+	do {								\
+		int __rc = stmt;					\
+		if (__rc != ZOK) {					\
+			sd_err("failed, " fmt ", %s",			\
+			       ##__VA_ARGS__, zerror(__rc));		\
+			return;						\
+		}							\
+	} while (0)
+
 static inline ZOOAPI int zk_delete_node(const char *path, int version)
 {
 	int rc;
 	do {
 		rc = zoo_delete(zhandle, path, version);
 	} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
-	if (rc != ZOK)
-		sd_eprintf("failed, path:%s, %s", path, zerror(rc));
+	CHECK_ZK_RC(rc, path);
+
 	return rc;
 }
 
-static inline ZOOAPI void
+static inline ZOOAPI int
 zk_init_node(const char *path)
 {
 	int rc;
@@ -154,9 +203,11 @@ zk_init_node(const char *path)
 		rc = zoo_create(zhandle, path, "", 0, &ZOO_OPEN_ACL_UNSAFE, 0,
 				NULL, 0);
 	} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
+	CHECK_ZK_RC(rc, path);
 
-	if (rc != ZOK && rc != ZNODEEXISTS)
-		panic("failed, path:%s, %s", path, zerror(rc));
+	if (rc == ZNODEEXISTS)
+		rc = ZOK;
+	return rc;
 }
 
 static inline ZOOAPI int
@@ -168,9 +219,9 @@ zk_create_node(const char *path, const char *value, int valuelen,
 	do {
 		rc = zoo_create(zhandle, path, value, valuelen, acl,
 				flags, path_buffer, path_buffer_len);
-		if (rc != ZOK && rc != ZNODEEXISTS)
-			sd_eprintf("failed, path:%s, %s", path, zerror(rc));
 	} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
+	CHECK_ZK_RC(rc, path);
+
 	return rc;
 }
 
@@ -183,13 +234,15 @@ zk_create_node(const char *path, const char *value, int valuelen,
  */
 static inline ZOOAPI int
 zk_create_seq_node(const char *path, const char *value, int valuelen,
-		   char *path_buffer, int path_buffer_len)
+		   char *path_buffer, int path_buffer_len, bool ephemeral)
 {
 	int rc;
+	int flags = ZOO_SEQUENCE;
+	if (ephemeral)
+		flags = flags | ZOO_EPHEMERAL;
 	rc = zoo_create(zhandle, path, value, valuelen, &ZOO_OPEN_ACL_UNSAFE,
-			ZOO_SEQUENCE, path_buffer, path_buffer_len);
-	if (rc != ZOK)
-		sd_iprintf("failed, path:%s, %s", path, zerror(rc));
+			flags, path_buffer, path_buffer_len);
+	CHECK_ZK_RC(rc, path);
 
 	return rc;
 }
@@ -201,9 +254,9 @@ static inline ZOOAPI int zk_get_data(const char *path, void *buffer,
 	do {
 		rc = zoo_get(zhandle, path, 1, (char *)buffer,
 			     buffer_len, NULL);
-		if (rc != ZOK)
-			sd_eprintf("failed, path:%s, %s", path, zerror(rc));
 	} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
+	CHECK_ZK_RC(rc, path);
+
 	return rc;
 }
 
@@ -214,8 +267,8 @@ zk_set_data(const char *path, const char *buffer, int buflen, int version)
 	do {
 		rc = zoo_set(zhandle, path, buffer, buflen, version);
 	} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
-	if (rc != ZOK)
-		panic("failed, path:%s, %s", path, zerror(rc));
+	CHECK_ZK_RC(rc, path);
+
 	return rc;
 }
 
@@ -224,29 +277,29 @@ static inline ZOOAPI int zk_node_exists(const char *path)
 	int rc;
 	do {
 		rc = zoo_exists(zhandle, path, 1, NULL);
-		if (rc != ZOK && rc != ZNONODE)
-			sd_eprintf("failed, path:%s, %s", path, zerror(rc));
 	} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
+	CHECK_ZK_RC(rc, path);
 
 	return rc;
 }
 
-static inline ZOOAPI void zk_get_children(const char *path,
-					  struct String_vector *strings)
+static inline ZOOAPI int zk_get_children(const char *path,
+					 struct String_vector *strings)
 {
 	int rc;
 	do {
 		rc = zoo_get_children(zhandle, path, 1, strings);
 	} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
-	if (rc != ZOK)
-		panic("failed, path:%s, %s", path, zerror(rc));
+	CHECK_ZK_RC(rc, path);
+
+	return rc;
 }
 
 /* ZooKeeper-based queue give us an totally ordered events */
 static int efd;
 static int32_t queue_pos;
 
-static bool zk_queue_peek(void)
+static int zk_queue_peek(bool *peek)
 {
 	int rc;
 	char path[MAX_NODE_STR_LEN];
@@ -254,77 +307,91 @@ static bool zk_queue_peek(void)
 	snprintf(path, sizeof(path), QUEUE_ZNODE "/%010"PRId32, queue_pos);
 
 	rc = zk_node_exists(path);
-	if (rc == ZOK)
-		return true;
-
-	return false;
+	switch (rc) {
+	case ZOK:
+		*peek = true;
+		return ZOK;
+	case ZNONODE:
+		*peek = false;
+		return ZOK;
+	default:
+		sd_err("failed, %s", zerror(rc));
+		return rc;
+	}
 }
 
 /* return true if there is a node with 'id' in the queue. */
-static bool zk_find_seq_node(uint64_t id)
+static int zk_find_seq_node(uint64_t id, char *seq_path, int seq_path_len,
+			    bool *found)
 {
 	int rc, len;
 
 	for (int seq = queue_pos; ; seq++) {
-		char seq_path[MAX_NODE_STR_LEN];
 		struct zk_event ev;
 
-		snprintf(seq_path, sizeof(seq_path), QUEUE_ZNODE"/%010"PRId32,
-			 seq);
+		snprintf(seq_path, seq_path_len, QUEUE_ZNODE"/%010"PRId32, seq);
 		len = offsetof(typeof(ev), id) + sizeof(ev.id);
 		rc = zk_get_data(seq_path, &ev, &len);
 		switch (rc) {
 		case ZOK:
 			if (ev.id == id) {
-				sd_dprintf("id %"PRIx64" is found in %s", id,
-					   seq_path);
-				return true;
+				sd_debug("id %" PRIx64 " is found in %s", id,
+					 seq_path);
+				*found = true;
+				return ZOK;
 			}
 			break;
 		case ZNONODE:
-			sd_dprintf("id %"PRIx64" is not found", id);
-			return false;
+			sd_debug("id %"PRIx64" is not found", id);
+			*found = false;
+			return ZOK;
 		default:
-			panic("failed, %s", zerror(rc));
-			break;
+			sd_err("failed, %s", zerror(rc));
+			return rc;
 		}
 	}
 }
 
-static void zk_queue_push(struct zk_event *ev)
+static int zk_queue_push(struct zk_event *ev)
 {
-	static bool first_push = true;
 	int rc, len;
 	char path[MAX_NODE_STR_LEN], buf[MAX_NODE_STR_LEN];
+	bool found;
 
 	len = offsetof(typeof(*ev), buf) + ev->buf_len;
 	snprintf(path, sizeof(path), "%s/", QUEUE_ZNODE);
 again:
-	rc = zk_create_seq_node(path, (char *)ev, len, buf, sizeof(buf));
+	rc = zk_create_seq_node(path, (char *)ev, len, buf, sizeof(buf), false);
 	switch (rc) {
 	case ZOK:
 		/* Success */
 		break;
 	case ZOPERATIONTIMEOUT:
 	case ZCONNECTIONLOSS:
-		if (!zk_find_seq_node(ev->id))
-			/* retry if seq_node was not created */
-			goto again;
-		break;
+		if (zk_find_seq_node(ev->id, buf, sizeof(buf), &found) == ZOK) {
+			if (found)
+				break;
+			else
+				/* retry if seq_node was not created */
+				goto again;
+		}
+		/* fall through */
 	default:
-		panic("failed, path:%s, %s", path, zerror(rc));
+		sd_err("failed, path:%s, %s", path, zerror(rc));
+		return rc;
 	}
 	if (first_push) {
 		int32_t seq;
 
 		sscanf(buf, QUEUE_ZNODE "/%"PRId32, &seq);
 		queue_pos = seq;
-		eventfd_write(efd, 1);
+		eventfd_xwrite(efd, 1);
 		first_push = false;
 	}
 
-	sd_dprintf("create path:%s, queue_pos:%010"PRId32", len:%d",
-		   buf, queue_pos, len);
+	sd_debug("create path:%s, queue_pos:%010" PRId32 ", len:%d", buf,
+		 queue_pos, len);
+	return ZOK;
 }
 
 static inline void *zk_event_sd_nodes(struct zk_event *ev)
@@ -333,12 +400,12 @@ static inline void *zk_event_sd_nodes(struct zk_event *ev)
 }
 
 /* Change the join event in place and piggyback the nodes information. */
-static void push_join_response(struct zk_event *ev)
+static int push_join_response(struct zk_event *ev)
 {
 	char path[MAX_NODE_STR_LEN];
 	int len;
 
-	ev->type = EVENT_JOIN_RESPONSE;
+	ev->type = EVENT_ACCEPT;
 	ev->nr_nodes = nr_sd_nodes;
 	memcpy(zk_event_sd_nodes(ev), sd_nodes,
 	       nr_sd_nodes * sizeof(struct sd_node));
@@ -346,39 +413,33 @@ static void push_join_response(struct zk_event *ev)
 
 	len = offsetof(typeof(*ev), buf) + ev->buf_len;
 	snprintf(path, sizeof(path), QUEUE_ZNODE "/%010"PRId32, queue_pos);
-	zk_set_data(path, (char *)ev, len, -1);
-	sd_dprintf("update path:%s, queue_pos:%010"PRId32", len:%d",
-		   path, queue_pos, len);
+
+	RETURN_IF_ERROR(zk_set_data(path, (char *)ev, len, -1), "");
+	sd_debug("update path:%s, queue_pos:%010" PRId32 ", len:%d", path,
+		 queue_pos, len);
+	return ZOK;
 }
 
-static void zk_queue_pop_advance(struct zk_event *ev)
+static int zk_queue_pop_advance(struct zk_event *ev)
 {
-	int rc, len;
+	int len;
 	char path[MAX_NODE_STR_LEN];
 
 	len = sizeof(*ev);
 	snprintf(path, sizeof(path), QUEUE_ZNODE "/%010"PRId32, queue_pos);
-	rc = zk_get_data(path, ev, &len);
-	if (rc != ZOK)
-		panic("failed to get data from %s, %s", path, zerror(rc));
-	sd_dprintf("%s, type:%d, len:%d, pos:%"PRId32, path, ev->type, len,
-		   queue_pos);
+
+	RETURN_IF_ERROR(zk_get_data(path, ev, &len), "path %s", path);
+	sd_debug("%s, type:%d, len:%d, pos:%" PRId32, path, ev->type, len,
+		 queue_pos);
 	queue_pos++;
-}
-
-static int zk_member_empty(void)
-{
-	struct String_vector strs;
-
-	zk_get_children(MEMBER_ZNODE, &strs);
-	return (strs.count == 0);
+	return ZOK;
 }
 
 static inline void zk_tree_add(struct zk_node *node)
 {
 	struct zk_node *zk = xzalloc(sizeof(*zk));
 	*zk = *node;
-	pthread_rwlock_wrlock(&zk_tree_lock);
+	sd_write_lock(&zk_tree_lock);
 	if (zk_tree_insert(zk)) {
 		free(zk);
 		goto out;
@@ -389,7 +450,7 @@ static inline void zk_tree_add(struct zk_node *node)
 	 */
 	sd_nodes[nr_sd_nodes++] = zk->node;
 out:
-	pthread_rwlock_unlock(&zk_tree_lock);
+	sd_unlock(&zk_tree_lock);
 }
 
 static inline void zk_tree_del_nolock(struct zk_node *node)
@@ -400,9 +461,9 @@ static inline void zk_tree_del_nolock(struct zk_node *node)
 
 static inline void zk_tree_del(struct zk_node *node)
 {
-	pthread_rwlock_wrlock(&zk_tree_lock);
+	sd_write_lock(&zk_tree_lock);
 	zk_tree_del_nolock(node);
-	pthread_rwlock_unlock(&zk_tree_lock);
+	sd_unlock(&zk_tree_lock);
 }
 
 static inline void zk_tree_destroy(void)
@@ -410,13 +471,13 @@ static inline void zk_tree_destroy(void)
 	struct zk_node *zk;
 	int i;
 
-	pthread_rwlock_wrlock(&zk_tree_lock);
+	sd_write_lock(&zk_tree_lock);
 	for (i = 0; i < nr_sd_nodes; i++) {
 		zk = zk_tree_search_nolock(&sd_nodes[i].nid);
 		if (zk)
 			zk_tree_del_nolock(zk);
 	}
-	pthread_rwlock_unlock(&zk_tree_lock);
+	sd_unlock(&zk_tree_lock);
 }
 
 static inline void build_node_list(void)
@@ -429,43 +490,16 @@ static inline void build_node_list(void)
 		zk = rb_entry(n, struct zk_node, rb);
 		sd_nodes[nr_sd_nodes++] = zk->node;
 	}
-	sd_dprintf("nr_sd_nodes:%zu", nr_sd_nodes);
+	sd_debug("nr_sd_nodes:%zu", nr_sd_nodes);
 }
 
-static inline int zk_master_create(void)
+static int zk_queue_init(void)
 {
-	return zk_create_node(MASTER_ZNONE, "", 0, &ZOO_OPEN_ACL_UNSAFE,
-			      ZOO_EPHEMERAL, NULL, 0);
-}
-
-static bool is_master(void)
-{
-	struct rb_node *n;
-	struct zk_node *zk = NULL;
-
-	if (!nr_sd_nodes) {
-		if (zk_member_empty())
-			return true;
-		else
-			return false;
-	}
-
-	for (n = rb_first(&zk_node_root); n; n = rb_next(n)) {
-		zk = rb_entry(n, struct zk_node, rb);
-		if (!zk->gone)
-			break;
-	}
-	if (zk && node_eq(&zk->node, &this_node.node))
-		return true;
-
-	return false;
-}
-
-static void zk_queue_init(void)
-{
-	zk_init_node(BASE_ZNODE);
-	zk_init_node(QUEUE_ZNODE);
-	zk_init_node(MEMBER_ZNODE);
+	RETURN_IF_ERROR(zk_init_node(BASE_ZNODE), "path %s", BASE_ZNODE);
+	RETURN_IF_ERROR(zk_init_node(MASTER_ZNONE), "path %s", MASTER_ZNONE);
+	RETURN_IF_ERROR(zk_init_node(QUEUE_ZNODE), "path %s", QUEUE_ZNODE);
+	RETURN_IF_ERROR(zk_init_node(MEMBER_ZNODE), "path %s", MEMBER_ZNODE);
+	return ZOK;
 }
 
 /* Calculate a unique 64 bit integer from this_node and the sequence number. */
@@ -484,6 +518,7 @@ static int add_event(enum zk_event_type type, struct zk_node *znode, void *buf,
 		     size_t buf_len)
 {
 	struct zk_event ev;
+	int rc;
 
 	ev.id = get_uniq_id();
 	ev.type = type;
@@ -491,8 +526,13 @@ static int add_event(enum zk_event_type type, struct zk_node *znode, void *buf,
 	ev.buf_len = buf_len;
 	if (buf)
 		memcpy(ev.buf, buf, buf_len);
-	zk_queue_push(&ev);
-	return 0;
+	rc = zk_queue_push(&ev);
+	if (rc == ZOK)
+		return SD_RES_SUCCESS;
+	else {
+		sd_err("failed, type: %d, %s", type, zerror(rc));
+		return SD_RES_CLUSTER_ERROR;
+	}
 }
 
 static void zk_watcher(zhandle_t *zh, int type, int state, const char *path,
@@ -502,17 +542,32 @@ static void zk_watcher(zhandle_t *zh, int type, int state, const char *path,
 	char str[MAX_NODE_STR_LEN], *p;
 	int ret;
 
+	if (type == ZOO_SESSION_EVENT && state == ZOO_EXPIRED_SESSION_STATE) {
+		/*
+		 * do reconnect in main thread to avoid on-the-fly zookeeper
+		 * operations.
+		 */
+		eventfd_xwrite(efd, 1);
+		return;
+	}
+
 /* CREATED_EVENT 1, DELETED_EVENT 2, CHANGED_EVENT 3, CHILD_EVENT 4 */
-	sd_dprintf("path:%s, type:%d", path, type);
+	sd_debug("path:%s, type:%d", path, type);
 	if (type == ZOO_CREATED_EVENT || type == ZOO_CHANGED_EVENT) {
 		ret = sscanf(path, MEMBER_ZNODE "/%s", str);
 		if (ret == 1)
 			zk_node_exists(path);
 		/* kick off the event handler */
-		eventfd_write(efd, 1);
+		eventfd_xwrite(efd, 1);
 	} else if (type == ZOO_DELETED_EVENT) {
 		struct zk_node *n;
 
+		ret = sscanf(path, MASTER_ZNONE "/%s", str);
+		if (ret == 1) {
+			zk_compete_master();
+			return;
+		}
+
 		ret = sscanf(path, MEMBER_ZNODE "/%s", str);
 		if (ret != 1)
 			return;
@@ -520,19 +575,18 @@ static void zk_watcher(zhandle_t *zh, int type, int state, const char *path,
 		p++;
 		str_to_node(p, &znode.node);
 		/* FIXME: remove redundant leave events */
-		pthread_rwlock_rdlock(&zk_tree_lock);
+		sd_read_lock(&zk_tree_lock);
 		n = zk_tree_search_nolock(&znode.node.nid);
 		if (n)
 			n->gone = true;
-		pthread_rwlock_unlock(&zk_tree_lock);
+		sd_unlock(&zk_tree_lock);
 		if (n)
 			add_event(EVENT_LEAVE, &znode, NULL, 0);
 	}
-
 }
 
 /*
- * We plcaehode the enough space to piggyback the nodes information on join
+ * We placehold the enough space to piggyback the nodes information on join
  * response message so that every node can see the same membership view.
  */
 static int add_join_event(void *msg, size_t msg_len)
@@ -542,14 +596,193 @@ static int add_join_event(void *msg, size_t msg_len)
 
 	assert(len <= SD_MAX_EVENT_BUF_SIZE);
 	ev.id = get_uniq_id();
-	ev.type = EVENT_JOIN_REQUEST;
+	ev.type = EVENT_JOIN;
 	ev.sender = this_node;
 	ev.msg_len = msg_len;
 	ev.buf_len = len;
 	if (msg)
 		memcpy(ev.buf, msg, msg_len);
-	zk_queue_push(&ev);
-	return 0;
+	return zk_queue_push(&ev);
+}
+
+static int zk_get_least_seq(const char *parent, char *least_seq_path,
+			    int path_len, void *buf, int *buf_len)
+{
+	char path[MAX_NODE_STR_LEN], *p, *tmp;
+	struct String_vector strs;
+	int rc, least_seq = INT_MAX , seq;
+
+	while (true) {
+		RETURN_IF_ERROR(zk_get_children(parent, &strs), "");
+
+		FOR_EACH_ZNODE(parent, path, &strs) {
+			p = strrchr(path, '/');
+			seq = strtol(++p, &tmp, 10);
+			if (seq < least_seq)
+				least_seq = seq;
+		}
+
+		snprintf(path, MAX_NODE_STR_LEN, "%s/%010"PRId32,
+			 parent, least_seq);
+		rc = zk_get_data(path, buf, buf_len);
+		switch (rc) {
+		case ZOK:
+			strncpy(least_seq_path, path, path_len);
+			return ZOK;
+		case ZNONODE:
+			break;
+		default:
+			sd_err("failed, %s", zerror(rc));
+			return rc;
+		}
+	}
+}
+
+static int zk_find_master(int *master_seq, char *master_name)
+{
+	int rc, len = MAX_NODE_STR_LEN;
+	char master_compete_path[MAX_NODE_STR_LEN];
+
+	if (*master_seq < 0) {
+		RETURN_IF_ERROR(zk_get_least_seq(MASTER_ZNONE,
+						 master_compete_path,
+						 MAX_NODE_STR_LEN, master_name,
+						 &len), "");
+		sscanf(master_compete_path, MASTER_ZNONE "/%"PRId32,
+		       master_seq);
+		return ZOK;
+	} else {
+		while (true) {
+			snprintf(master_compete_path, len,
+				 MASTER_ZNONE "/%010"PRId32, *master_seq);
+			rc = zk_get_data(master_compete_path, master_name,
+					 &len);
+			switch (rc) {
+			case ZOK:
+				return ZOK;
+			case ZNONODE:
+				sd_info("detect master leave, "
+					"start to compete master");
+				(*master_seq)++;
+				break;
+			default:
+				sd_err("failed, %s", zerror(rc));
+				return rc;
+			}
+		}
+	}
+}
+
+/*
+ * block until last sheep joined
+ * last_sheep returns sequence number of last sheep or -1 if no previous sheep
+ */
+static int zk_verify_last_sheep_join(int seq, int *last_sheep)
+{
+	int rc, len = MAX_NODE_STR_LEN;
+	char path[MAX_NODE_STR_LEN], name[MAX_NODE_STR_LEN];
+
+	for (*last_sheep = seq - 1; *last_sheep >= 0; (*last_sheep)--) {
+		snprintf(path, MAX_NODE_STR_LEN, MASTER_ZNONE "/%010"PRId32,
+			 *last_sheep);
+		rc = zk_get_data(path, name, &len);
+		switch (rc) {
+		case ZNONODE:
+			continue;
+		case ZOK:
+			break;
+		default:
+			sd_err("failed, %s", zerror(rc));
+			return rc;
+		}
+
+		if (!strcmp(name, node_to_str(&this_node.node)))
+			continue;
+
+		snprintf(path, MAX_NODE_STR_LEN, MEMBER_ZNODE "/%s", name);
+		rc = zk_node_exists(path);
+		switch (rc) {
+		case ZOK:
+			return ZOK;
+		case ZNONODE:
+			(*last_sheep)++;
+			break;
+		default:
+			sd_err("failed, %s", zerror(rc));
+			return rc;
+		}
+	}
+	return ZOK;
+}
+
+/*
+ * Create sequential node under MASTER_ZNODE.
+ * Sheep with least sequential number win the competition.
+ */
+static void zk_compete_master(void)
+{
+	int rc, last_joined_sheep;
+	char master_name[MAX_NODE_STR_LEN];
+	char my_compete_path[MAX_NODE_STR_LEN];
+	static int master_seq = -1, my_seq;
+
+	/*
+	 * This is to protect master_seq and my_seq because this function will
+	 * be called by both main thread and zookeeper's event thread.
+	 */
+	sd_write_lock(&zk_compete_master_lock);
+
+	if (uatomic_is_true(&is_master) || uatomic_is_true(&stop))
+		goto out_unlock;
+
+	if (!joined) {
+		sd_debug("start to compete master for the first time");
+		do {
+			if (uatomic_is_true(&stop))
+				goto out_unlock;
+			/* duplicate sequential node has no side-effect */
+			rc = zk_create_seq_node(MASTER_ZNONE "/",
+						node_to_str(&this_node.node),
+						MAX_NODE_STR_LEN,
+						my_compete_path,
+						MAX_NODE_STR_LEN, true);
+		} while (rc == ZOPERATIONTIMEOUT || rc == ZCONNECTIONLOSS);
+		CHECK_ZK_RC(rc, MASTER_ZNONE "/");
+		if (rc != ZOK)
+			goto out_unlock;
+
+		sd_debug("my compete path: %s", my_compete_path);
+		sscanf(my_compete_path, MASTER_ZNONE "/%"PRId32,
+		       &my_seq);
+	}
+
+	if (zk_find_master(&master_seq, master_name) != ZOK)
+		goto out_unlock;
+
+	if (!strcmp(master_name, node_to_str(&this_node.node)))
+		goto success;
+	else if (joined) {
+		sd_debug("lost");
+		goto out_unlock;
+	} else {
+		if (zk_verify_last_sheep_join(my_seq,
+					      &last_joined_sheep) != ZOK)
+			goto out_unlock;
+
+		if (last_joined_sheep < 0) {
+			/* all previous sheep has quit, i'm master */
+			master_seq = my_seq;
+			goto success;
+		} else {
+			sd_debug("lost");
+			goto out_unlock;
+		}
+	}
+success:
+	uatomic_set_true(&is_master);
+	sd_debug("success");
+out_unlock:
+	sd_unlock(&zk_compete_master_lock);
 }
 
 static int zk_join(const struct sd_node *myself,
@@ -563,22 +796,25 @@ static int zk_join(const struct sd_node *myself,
 	snprintf(path, sizeof(path), MEMBER_ZNODE "/%s", node_to_str(myself));
 	rc = zk_node_exists(path);
 	if (rc == ZOK) {
-		sd_eprintf("Previous zookeeper session exist, shoot myself.");
+		sd_err("Previous zookeeper session exist, shoot myself.");
 		exit(1);
 	}
 
-	/* For concurrent nodes setup, we allow only one to continue */
-	while (zk_member_empty() && zk_master_create() != ZOK)
-		;/* wait */
+	zk_compete_master();
+	RETURN_IF_ERROR(add_join_event(opaque, opaque_len), "");
 
-	return add_join_event(opaque, opaque_len);
+	return ZOK;
 }
 
 static int zk_leave(void)
 {
 	char path[PATH_MAX];
+
+	sd_info("leaving from cluster");
+	uatomic_set_true(&stop);
+
 	snprintf(path, sizeof(path), MEMBER_ZNODE"/%s",
-			node_to_str(&this_node.node));
+		 node_to_str(&this_node.node));
 	add_event(EVENT_LEAVE, &this_node, NULL, 0);
 	zk_delete_node(path, -1);
 	return 0;
@@ -589,51 +825,40 @@ static int zk_notify(void *msg, size_t msg_len)
 	return add_event(EVENT_NOTIFY, &this_node, msg, msg_len);
 }
 
-static void zk_block(void)
+static int zk_block(void)
 {
-	add_event(EVENT_BLOCK, &this_node, NULL, 0);
+	return add_event(EVENT_BLOCK, &this_node, NULL, 0);
 }
 
-static void zk_unblock(void *msg, size_t msg_len)
+static int zk_unblock(void *msg, size_t msg_len)
 {
-	add_event(EVENT_UNBLOCK, &this_node, msg, msg_len);
+	return add_event(EVENT_UNBLOCK, &this_node, msg, msg_len);
 }
 
-static void zk_handle_join_request(struct zk_event *ev)
+static void zk_handle_join(struct zk_event *ev)
 {
-	enum cluster_join_result res;
-
-	sd_dprintf("sender: %s", node_to_str(&ev->sender.node));
-	if (!is_master()) {
+	sd_debug("sender: %s", node_to_str(&ev->sender.node));
+	if (!uatomic_is_true(&is_master)) {
 		/* Let's await master acking the join-request */
 		queue_pos--;
 		return;
 	}
 
-	res = sd_check_join_cb(&ev->sender.node, ev->buf);
-	ev->join_result = res;
+	sd_join_handler(&ev->sender.node, sd_nodes, nr_sd_nodes, ev->buf);
 	push_join_response(ev);
-	if (res == CJ_RES_MASTER_TRANSFER) {
-		sd_eprintf("failed to join sheepdog cluster: "
-			   "please retry when master is up");
-		add_event(EVENT_LEAVE, &this_node, NULL, 0);
-		exit(1);
-	}
-	sd_dprintf("I'm the master now");
+
+	sd_debug("I'm the master now");
 }
 
 static void watch_all_nodes(void)
 {
 	struct String_vector strs;
-	struct zk_node znode;
 	char path[MAX_NODE_STR_LEN];
-	int len = sizeof(znode);
 
-	if (zk_member_empty())
-		return;
+	RETURN_VOID_IF_ERROR(zk_get_children(MEMBER_ZNODE, &strs), "");
 
 	FOR_EACH_ZNODE(MEMBER_ZNODE, path, &strs) {
-		zk_get_data(path, &znode, &len);
+		RETURN_VOID_IF_ERROR(zk_node_exists(path), "");
 	}
 }
 
@@ -643,7 +868,7 @@ static void init_node_list(struct zk_event *ev)
 	size_t node_nr = ev->nr_nodes;
 	int i;
 
-	sd_dprintf("%zu", node_nr);
+	sd_debug("%zu", node_nr);
 	for (i = 0; i < node_nr; i++) {
 		struct zk_node zk;
 		mempcpy(&zk.node, p, sizeof(struct sd_node));
@@ -654,47 +879,36 @@ static void init_node_list(struct zk_event *ev)
 	watch_all_nodes();
 }
 
-static void zk_handle_join_response(struct zk_event *ev)
+static void zk_handle_accept(struct zk_event *ev)
 {
 	char path[MAX_NODE_STR_LEN];
+	int rc;
 
-	sd_dprintf("JOIN RESPONSE");
+	sd_debug("ACCEPT");
 	if (node_eq(&ev->sender.node, &this_node.node))
 		/* newly joined node */
 		init_node_list(ev);
 
-	if (ev->join_result == CJ_RES_MASTER_TRANSFER)
-		/*
-		 * Sheepdog assumes that only one sheep is alive in
-		 * MASTER_TRANSFER scenario. So only the joining sheep is
-		 * supposed to return single node view to sd_join_handler().
-		 */
-		zk_tree_destroy();
+	sd_debug("%s", node_to_str(&ev->sender.node));
 
-	sd_dprintf("%s, %d", node_to_str(&ev->sender.node), ev->join_result);
-	switch (ev->join_result) {
-	case CJ_RES_SUCCESS:
-	case CJ_RES_JOIN_LATER:
-	case CJ_RES_MASTER_TRANSFER:
-		snprintf(path, sizeof(path), MEMBER_ZNODE"/%s",
-			 node_to_str(&ev->sender.node));
-		if (node_eq(&ev->sender.node, &this_node.node)) {
-			sd_dprintf("create path:%s", path);
-			zk_create_node(path, (char *)&ev->sender,
-				       sizeof(ev->sender), &ZOO_OPEN_ACL_UNSAFE,
-				       ZOO_EPHEMERAL, NULL, 0);
-		} else
-			zk_node_exists(path);
+	snprintf(path, sizeof(path), MEMBER_ZNODE"/%s",
+		 node_to_str(&ev->sender.node));
+	if (node_eq(&ev->sender.node, &this_node.node)) {
+		joined = true;
+		sd_debug("create path:%s", path);
+		rc = zk_create_node(path,
+				    (char *)zoo_client_id(zhandle),
+				    sizeof(clientid_t),
+				    &ZOO_OPEN_ACL_UNSAFE,
+				    ZOO_EPHEMERAL, NULL, 0);
+		RETURN_VOID_IF_ERROR(rc, "");
+	} else
+		zk_node_exists(path);
 
-		zk_tree_add(&ev->sender);
-		break;
-	default:
-		break;
-	}
+	zk_tree_add(&ev->sender);
 
 	build_node_list();
-	sd_join_handler(&ev->sender.node, sd_nodes, nr_sd_nodes,
-			ev->join_result, ev->buf);
+	sd_accept_handler(&ev->sender.node, sd_nodes, nr_sd_nodes, ev->buf);
 }
 
 static void kick_block_event(void)
@@ -725,8 +939,8 @@ static void zk_handle_leave(struct zk_event *ev)
 	struct zk_node *n = zk_tree_search(&ev->sender.node.nid);
 
 	if (!n) {
-		sd_dprintf("can't find this leave node:%s, ignore it.",
-			   node_to_str(&ev->sender.node));
+		sd_debug("can't find this leave node:%s, ignore it.",
+			 node_to_str(&ev->sender.node));
 		return;
 	}
 	block_event_list_del(n);
@@ -739,7 +953,7 @@ static void zk_handle_block(struct zk_event *ev)
 {
 	struct zk_node *block = xzalloc(sizeof(*block));
 
-	sd_dprintf("BLOCK");
+	sd_debug("BLOCK");
 	block->node = ev->sender.node;
 	list_add_tail(&block->list, &zk_block_list);
 	block = list_first_entry(&zk_block_list, typeof(*block), list);
@@ -751,12 +965,11 @@ static void zk_handle_unblock(struct zk_event *ev)
 {
 	struct zk_node *block;
 
-	sd_dprintf("UNBLOCK");
+	sd_debug("UNBLOCK");
 	if (list_empty(&zk_block_list))
 		return;
 	block = list_first_entry(&zk_block_list, typeof(*block), list);
-	if (block->callbacked)
-		add_event(EVENT_NOTIFY, block, ev->buf, ev->buf_len);
+	sd_notify_handler(&ev->sender.node, ev->buf, ev->buf_len);
 
 	list_del(&block->list);
 	free(block);
@@ -764,54 +977,104 @@ static void zk_handle_unblock(struct zk_event *ev)
 
 static void zk_handle_notify(struct zk_event *ev)
 {
-	sd_dprintf("NOTIFY");
+	sd_debug("NOTIFY");
 	sd_notify_handler(&ev->sender.node, ev->buf, ev->buf_len);
 }
 
+static void zk_handle_update_node(struct zk_event *ev)
+{
+	struct zk_node *t;
+	struct sd_node *snode = &ev->sender.node;
+
+	sd_debug("%s", node_to_str(snode));
+
+	if (node_eq(snode, &this_node.node))
+		this_node.node = *snode;
+
+	sd_read_lock(&zk_tree_lock);
+	t = zk_tree_search_nolock(&snode->nid);
+	assert(t);
+	t->node = *snode;
+	build_node_list();
+	sd_unlock(&zk_tree_lock);
+	sd_update_node_handler(snode);
+}
+
 static void (*const zk_event_handlers[])(struct zk_event *ev) = {
-	[EVENT_JOIN_REQUEST]	= zk_handle_join_request,
-	[EVENT_JOIN_RESPONSE]	= zk_handle_join_response,
+	[EVENT_JOIN]		= zk_handle_join,
+	[EVENT_ACCEPT]		= zk_handle_accept,
 	[EVENT_LEAVE]		= zk_handle_leave,
 	[EVENT_BLOCK]		= zk_handle_block,
 	[EVENT_UNBLOCK]		= zk_handle_unblock,
 	[EVENT_NOTIFY]		= zk_handle_notify,
+	[EVENT_UPDATE_NODE]	= zk_handle_update_node,
 };
 
 static const int zk_max_event_handlers = ARRAY_SIZE(zk_event_handlers);
 
+/*
+ * This method should be done in main thread and triggered when zk_watcher()
+ * receives a session timeout event.
+ * All other zk operations who receive 'ZINVALIDSTATE' return code should drop
+ * control of main thread as soon as possible. So that this method can be
+ * executed and re-establish a new session with zookeeper server.
+ */
+static inline void handle_session_expire(void)
+{
+	/* clean memory states */
+	close(efd);
+	zk_tree_destroy();
+	INIT_RB_ROOT(&zk_node_root);
+	INIT_LIST_HEAD(&zk_block_list);
+	nr_sd_nodes = 0;
+	first_push = true;
+	joined = false;
+	memset(sd_nodes, 0, sizeof(struct sd_node) * SD_MAX_NODES);
+
+	while (sd_reconnect_handler()) {
+		sd_err("failed to reconnect. sleep and retry...");
+		sleep(1);
+	}
+}
+
 static void zk_event_handler(int listen_fd, int events, void *data)
 {
-	eventfd_t value;
 	struct zk_event ev;
+	bool peek;
 
-	sd_dprintf("%d, %d", events, queue_pos);
+	sd_debug("%d, %d", events, queue_pos);
 	if (events & EPOLLHUP) {
-		sd_eprintf("zookeeper driver received EPOLLHUP event,"
-			   " exiting.");
+		sd_err("zookeeper driver received EPOLLHUP event, exiting.");
 		log_close();
 		exit(1);
 	}
 
-	if (eventfd_read(efd, &value) < 0) {
-		sd_eprintf("%m");
+	eventfd_xread(efd);
+
+	if (zoo_state(zhandle) == ZOO_EXPIRED_SESSION_STATE) {
+		sd_err("detect a session timeout. reconnecting...");
+		handle_session_expire();
+		sd_info("reconnected");
+		eventfd_xwrite(efd, 1);
 		return;
 	}
 
-	if (!zk_queue_peek())
+	RETURN_VOID_IF_ERROR(zk_queue_peek(&peek), "");
+	if (!peek)
 		goto kick_block_event;
 
-	zk_queue_pop_advance(&ev);
+	RETURN_VOID_IF_ERROR(zk_queue_pop_advance(&ev), "");
 	if (ev.type < zk_max_event_handlers && zk_event_handlers[ev.type])
 		zk_event_handlers[ev.type](&ev);
 	else
 		panic("unhandled type %d", ev.type);
 
-	 /* Someone has created next event, go kick event handler. */
-	if (zk_queue_peek()) {
-		eventfd_write(efd, 1);
+	RETURN_VOID_IF_ERROR(zk_queue_peek(&peek), "");
+	if (peek) {
+		/* Someone has created next event, go kick event handler. */
+		eventfd_xwrite(efd, 1);
 		return;
 	}
-
 kick_block_event:
 	/*
 	 * Kick block event only if there is no nonblock event. We perfer to
@@ -830,62 +1093,53 @@ static int zk_init(const char *option)
 	int ret, timeout = SESSION_TIMEOUT;
 
 	if (!option) {
-		sd_eprintf("You must specify zookeeper servers.");
+		sd_err("You must specify zookeeper servers.");
 		return -1;
 	}
 
 	hosts = strtok((char *)option, "=");
 	if ((to = strtok(NULL, "="))) {
 		if (sscanf(to, "%u", &timeout) != 1) {
-			sd_eprintf("Invalid paramter for timeout");
+			sd_err("Invalid paramter for timeout");
 			return -1;
 		}
 		p = strstr(hosts, "timeout");
 		*--p = '\0';
 	}
-	sd_dprintf("version %d.%d.%d, address %s, timeout %d",
-		   ZOO_MAJOR_VERSION, ZOO_MINOR_VERSION, ZOO_PATCH_VERSION,
-		   hosts, timeout);
+	sd_debug("version %d.%d.%d, address %s, timeout %d", ZOO_MAJOR_VERSION,
+		 ZOO_MINOR_VERSION, ZOO_PATCH_VERSION, hosts, timeout);
 	zhandle = zookeeper_init(hosts, zk_watcher, timeout, NULL, NULL, 0);
 	if (!zhandle) {
-		sd_eprintf("failed to connect to zk server %s", option);
+		sd_err("failed to connect to zk server %s", option);
 		return -1;
 	}
 
-	zk_queue_init();
+	uatomic_set_false(&stop);
+	uatomic_set_false(&is_master);
+	if (zk_queue_init() != ZOK)
+		return -1;
 
 	efd = eventfd(0, EFD_NONBLOCK);
 	if (efd < 0) {
-		sd_eprintf("failed to create an event fd: %m");
+		sd_err("failed to create an event fd: %m");
 		return -1;
 	}
 
 	ret = register_event(efd, zk_event_handler, NULL);
 	if (ret) {
-		sd_eprintf("failed to register zookeeper event handler (%d)",
-			   ret);
+		sd_err("failed to register zookeeper event handler (%d)", ret);
 		return -1;
 	}
 
 	return 0;
 }
 
-static void zk_update_node(struct sd_node *node)
+static int zk_update_node(struct sd_node *node)
 {
-	struct zk_node n = {
+	struct zk_node znode = {
 		.node = *node,
 	};
-	struct zk_node *t;
-
-	sd_dprintf("%s", node_to_str(&n.node));
-
-	pthread_rwlock_rdlock(&zk_tree_lock);
-	t = zk_tree_search_nolock(&n.node.nid);
-	if (t) {
-		t->node = n.node;
-		build_node_list();
-	}
-	pthread_rwlock_unlock(&zk_tree_lock);
+	return add_event(EVENT_UPDATE_NODE, &znode, NULL, 0);
 }
 
 static struct cluster_driver cdrv_zookeeper = {
@@ -898,6 +1152,7 @@ static struct cluster_driver cdrv_zookeeper = {
 	.block      = zk_block,
 	.unblock    = zk_unblock,
 	.update_node = zk_update_node,
+	.get_local_addr = get_local_addr,
 };
 
 cdrv_register(cdrv_zookeeper);
