@@ -9,12 +9,6 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <errno.h>
-
 #include "sheep_priv.h"
 
 #define SD_FORMAT_VERSION 0x0002
@@ -39,9 +33,9 @@ static int write_config(void)
 	int ret;
 
 	ret = atomic_create_and_write(config_path, (char *)&config,
-				sizeof(config));
+				      sizeof(config), true);
 	if (ret < 0) {
-		sd_eprintf("atomic_create_and_write() failed");
+		sd_err("atomic_create_and_write() failed");
 		return SD_RES_EIO;
 	}
 
@@ -59,7 +53,7 @@ static void check_tmp_config(void)
 	if (!ret || ret != ENOENT)
 		return;
 
-	sd_iprintf("removed temporal config file");
+	sd_info("removed temporal config file");
 }
 
 int init_config_file(void)
@@ -71,7 +65,7 @@ int init_config_file(void)
 	fd = open(config_path, O_RDONLY);
 	if (fd < 0) {
 		if (errno != ENOENT) {
-			sd_eprintf("failed to read config file, %m");
+			sd_err("failed to read config file, %m");
 			return -1;
 		}
 		goto create;
@@ -83,13 +77,13 @@ int init_config_file(void)
 		goto create;
 	}
 	if (ret < 0) {
-		sd_eprintf("failed to read config file, %m");
+		sd_err("failed to read config file, %m");
 		goto out;
 	}
 
 	if (config.version != SD_FORMAT_VERSION) {
-		sd_eprintf("This sheep version is not compatible with"
-			   " the existing data layout, %d", config.version);
+		sd_err("This sheep version is not compatible with"
+		       " the existing data layout, %d", config.version);
 		if (sys->upgrade) {
 			/* upgrade sheep store */
 			ret = sd_migrate_store(config.version, SD_FORMAT_VERSION);
@@ -97,8 +91,8 @@ int init_config_file(void)
 				/* reload config file */
 				ret = xpread(fd, &config, sizeof(config), 0);
 				if (ret != sizeof(config)) {
-					sd_eprintf("failed to reload config"
-						   " file, %m");
+					sd_err("failed to reload config file,"
+					       " %m");
 					ret = -1;
 				} else
 					ret = 0;
@@ -106,11 +100,12 @@ int init_config_file(void)
 			goto out;
 		}
 
-		sd_eprintf("use '-u' option to upgrade sheep store");
+		sd_err("use '-u' option to upgrade sheep store");
 		ret = -1;
 		goto out;
 	}
 	ret = 0;
+	get_cluster_config(&sys->cinfo);
 out:
 	close(fd);
 
@@ -131,57 +126,24 @@ void init_config_path(const char *base_path)
 	snprintf(config_path, len, "%s" CONFIG_PATH, base_path);
 }
 
-int set_cluster_ctime(uint64_t ct)
-{
-	config.ctime = ct;
-
-	return write_config();
-}
-
-uint64_t get_cluster_ctime(void)
-{
-	return config.ctime;
-}
-
-int set_cluster_copies(uint8_t copies)
-{
-	config.copies = copies;
-
-	return write_config();
-}
-
-int get_cluster_copies(uint8_t *copies)
-{
-	*copies = config.copies;
-
-	return SD_RES_SUCCESS;
-}
-
-int set_cluster_flags(uint16_t flags)
-{
-	config.flags = flags;
-
-	return write_config();
-}
-
-int get_cluster_flags(uint16_t *flags)
-{
-	*flags = config.flags;
-
-	return SD_RES_SUCCESS;
-}
-
-int set_cluster_store(const char *name)
+int set_cluster_config(const struct cluster_info *cinfo)
 {
+	config.ctime = cinfo->ctime;
+	config.copies = cinfo->nr_copies;
+	config.flags = cinfo->flags;
 	memset(config.store, 0, sizeof(config.store));
-	pstrcpy((char *)config.store, sizeof(config.store), name);
+	pstrcpy((char *)config.store, sizeof(config.store),
+		(char *)cinfo->store);
 
 	return write_config();
 }
 
-int get_cluster_store(char *buf)
+int get_cluster_config(struct cluster_info *cinfo)
 {
-	memcpy(buf, config.store, sizeof(config.store));
+	cinfo->ctime = config.ctime;
+	cinfo->nr_copies = config.copies;
+	cinfo->flags = config.flags;
+	memcpy(cinfo->store, config.store, sizeof(config.store));
 
 	return SD_RES_SUCCESS;
 }
@@ -200,6 +162,15 @@ int get_node_space(uint64_t *space)
 	return SD_RES_SUCCESS;
 }
 
+bool is_cluster_formatted(void)
+{
+	struct cluster_info cinfo;
+
+	get_cluster_config(&cinfo);
+
+	return cinfo.ctime != 0;
+}
+
 static inline __attribute__((used)) void __sd_config_format_build_bug_ons(void)
 {
 	/* never called, only for checking BUILD_BUG_ON()s */
