@@ -9,10 +9,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <poll.h>
 
 #include "sheep_priv.h"
 
@@ -46,6 +42,12 @@ int gateway_read_obj(struct request *req)
 	}
 
 	nr_copies = get_req_copy_number(req);
+
+	if (nr_copies == 0) {
+		sd_debug("there is no living nodes");
+		return SD_RES_HALT;
+	}
+
 	oid_to_vnodes(req->vinfo->vnodes, req->vinfo->nr_vnodes, oid,
 		      nr_copies, obj_vnodes);
 	for (i = 0; i < nr_copies; i++) {
@@ -56,8 +58,8 @@ int gateway_read_obj(struct request *req)
 		if (ret == SD_RES_SUCCESS)
 			goto out;
 
-		sd_eprintf("local read %"PRIx64" failed, %s", oid,
-			   sd_strerror(ret));
+		sd_err("local read %"PRIx64" failed, %s", oid,
+		       sd_strerror(ret));
 		break;
 	}
 
@@ -89,8 +91,8 @@ out:
 	if (ret == SD_RES_SUCCESS &&
 	    req->rq.proto_ver < SD_PROTO_VER_TRIM_ZERO_SECTORS) {
 		/* the client doesn't support trimming zero bytes */
-		untrim_zero_sectors(req->data, req->rp.obj.offset,
-				    req->rp.data_length, req->rq.data_length);
+		untrim_zero_blocks(req->data, req->rp.obj.offset,
+				   req->rp.data_length, req->rq.data_length);
 		req->rp.data_length = req->rq.data_length;
 		req->rp.obj.offset = 0;
 	}
@@ -110,7 +112,7 @@ struct write_info {
 
 static inline void write_info_update(struct write_info *wi, int pos)
 {
-	sd_dprintf("%d, %d", wi->nr_sent, pos);
+	sd_debug("%d, %d", wi->nr_sent, pos);
 	wi->nr_sent--;
 	memmove(wi->ent + pos, wi->ent + pos + 1,
 		sizeof(struct write_info_entry) * (wi->nr_sent - pos));
@@ -118,13 +120,13 @@ static inline void write_info_update(struct write_info *wi, int pos)
 
 static inline void finish_one_write(struct write_info *wi, int i)
 {
-	sheep_put_sockfd(wi->ent[i].nid, wi->ent[i].sfd);
+	sockfd_cache_put(wi->ent[i].nid, wi->ent[i].sfd);
 	write_info_update(wi, i);
 }
 
 static inline void finish_one_write_err(struct write_info *wi, int i)
 {
-	sheep_del_sockfd(wi->ent[i].nid, wi->ent[i].sfd);
+	sockfd_cache_del(wi->ent[i].nid, wi->ent[i].sfd);
 	write_info_update(wi, i);
 }
 
@@ -170,17 +172,16 @@ again:
 		 */
 		if (sheep_need_retry(req->rq.epoch) && repeat) {
 			repeat--;
-			sd_printf(SDOG_WARNING,
-				  "poll timeout %d, disks of some nodes or"
-				   " network is busy. Going to poll-wait again",
-				   wi->nr_sent);
+			sd_warn("poll timeout %d, disks of some nodes or "
+				"network is busy. Going to poll-wait again",
+				wi->nr_sent);
 			goto again;
 		}
 
 		nr_sent = wi->nr_sent;
 		/* XXX Blinedly close all the connections */
 		for (i = 0; i < nr_sent; i++)
-			sheep_del_sockfd(wi->ent[i].nid, wi->ent[i].sfd);
+			sockfd_cache_del(wi->ent[i].nid, wi->ent[i].sfd);
 
 		return SD_RES_NETWORK_ERROR;
 	}
@@ -191,15 +192,15 @@ again:
 			break;
 	if (i < nr_sent) {
 		int re = pi.pfds[i].revents;
-		sd_dprintf("%d, revents %x", i, re);
+		sd_debug("%d, revents %x", i, re);
 		if (re & (POLLERR | POLLHUP | POLLNVAL)) {
 			err_ret = SD_RES_NETWORK_ERROR;
 			finish_one_write_err(wi, i);
 			goto finish_write;
 		}
 		if (do_read(pi.pfds[i].fd, rsp, sizeof(*rsp), sheep_need_retry,
-			    req->rq.epoch)) {
-			sd_eprintf("remote node might have gone away");
+			    req->rq.epoch, MAX_RETRY_COUNT)) {
+			sd_err("remote node might have gone away");
 			err_ret = SD_RES_NETWORK_ERROR;
 			finish_one_write_err(wi, i);
 			goto finish_write;
@@ -207,8 +208,8 @@ again:
 
 		ret = rsp->result;
 		if (ret != SD_RES_SUCCESS) {
-			sd_eprintf("fail %"PRIx64", %s", req->rq.obj.oid,
-				   sd_strerror(ret));
+			sd_err("fail %"PRIx64", %s", req->rq.obj.oid,
+			       sd_strerror(ret));
 			err_ret = ret;
 		}
 		finish_one_write(wi, i);
@@ -263,7 +264,7 @@ static int gateway_forward_request(struct request *req)
 	struct sd_req hdr;
 	const struct sd_node *target_nodes[SD_MAX_NODES];
 
-	sd_dprintf("%"PRIx64, oid);
+	sd_debug("%"PRIx64, oid);
 
 	gateway_init_fwd_hdr(&hdr, &req->rq);
 	op = get_sd_op(hdr.opcode);
@@ -272,6 +273,11 @@ static int gateway_forward_request(struct request *req)
 	nr_to_send = init_target_nodes(req, oid, target_nodes);
 	write_info_init(&wi, nr_to_send);
 
+	if (nr_to_send == 0) {
+		sd_debug("there is no living nodes");
+		return SD_RES_HALT;
+	}
+
 	for (i = 0; i < nr_to_send; i++) {
 		struct sockfd *sfd;
 		const struct node_id *nid;
@@ -282,18 +288,19 @@ static int gateway_forward_request(struct request *req)
 		}
 
 		nid = &target_nodes[i]->nid;
-		sfd = sheep_get_sockfd(nid);
+		sfd = sockfd_cache_get(nid);
 		if (!sfd) {
 			err_ret = SD_RES_NETWORK_ERROR;
 			break;
 		}
 
 		ret = send_req(sfd->fd, &hdr, req->data, wlen,
-			       sheep_need_retry, req->rq.epoch);
+			       sheep_need_retry, req->rq.epoch,
+			       MAX_RETRY_COUNT);
 		if (ret) {
-			sheep_del_sockfd(nid, sfd);
+			sockfd_cache_del_node(nid);
 			err_ret = SD_RES_NETWORK_ERROR;
-			sd_dprintf("fail %d", ret);
+			sd_debug("fail %d", ret);
 			break;
 		}
 		write_info_advance(&wi, nid, sfd);
@@ -304,13 +311,13 @@ static int gateway_forward_request(struct request *req)
 		ret = sheep_do_op_work(op, req);
 
 		if (ret != SD_RES_SUCCESS) {
-			sd_eprintf("fail to write local %"PRIx64", %s", oid,
-				   sd_strerror(ret));
+			sd_err("fail to write local %"PRIx64", %s", oid,
+			       sd_strerror(ret));
 			err_ret = ret;
 		}
 	}
 
-	sd_dprintf("nr_sent %d, err %x", wi.nr_sent, err_ret);
+	sd_debug("nr_sent %d, err %x", wi.nr_sent, err_ret);
 	if (wi.nr_sent > 0) {
 		ret = wait_forward_request(&wi, req);
 		if (ret != SD_RES_SUCCESS)
