@@ -10,14 +10,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-#include <errno.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <unistd.h>
-#include <pthread.h>
-#include <stdint.h>
-#include <sys/mman.h>
 
 #include "sheep_priv.h"
 
@@ -28,6 +20,13 @@ struct journal_file {
 	uatomic_bool in_commit;
 };
 
+/*
+ * CAUTION: This definition of struct journal_descriptor must be same
+ * to the definition in tests/dynamorio/journaling/journaling.c. We
+ * have to update the definition in the DR client definition if we
+ * update the below definition because there's no technique for
+ * keeping the consistency automatically.
+ */
 struct journal_descriptor {
 	uint32_t magic;
 	uint16_t flag;
@@ -68,11 +67,11 @@ static int create_journal_file(const char *root, const char *name)
 	snprintf(path, sizeof(path), "%s/%s", root, name);
 	fd = open(path, flags, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 	if (fd < 0) {
-		sd_eprintf("open %s %m", name);
+		sd_err("open %s %m", name);
 		return -1;
 	}
 	if (prealloc(fd, jfile_size) < 0) {
-		sd_eprintf("prealloc %s %m", name);
+		sd_err("prealloc %s %m", name);
 		return -1;
 	}
 
@@ -93,19 +92,19 @@ static int get_old_new_jfile(const char *p, int *old, int *new)
 		if (errno == ENOENT)
 			return 0;
 
-		sd_eprintf("open1 %m");
+		sd_err("open1 %m");
 		return -1;
 	}
 	snprintf(path, sizeof(path), "%s/%s", p, jfile_name[1]);
 	fd2 = open(path, flags);
 	if (fd2 < 0) {
-		sd_eprintf("open2 %m");
+		sd_err("open2 %m");
 		close(fd1);
 		return -1;
 	}
 
 	if (fstat(fd1, &st1) < 0 || fstat(fd2, &st2) < 0) {
-		sd_eprintf("stat %m");
+		sd_err("stat %m");
 		goto out;
 	}
 
@@ -135,25 +134,6 @@ static bool journal_entry_full_write(struct journal_descriptor *jd)
 	return true;
 }
 
-static void journal_get_path(struct journal_descriptor *jd, char *path)
-{
-	switch (jd->flag) {
-	case JF_STORE:
-	case JF_REMOVE_OBJ:
-		snprintf(path, PATH_MAX, "%s/%016"PRIx64,
-			 md_get_object_path(jd->oid), jd->oid);
-		if (jd->flag == JF_STORE)
-			sd_iprintf("%s, size %"PRIu64", off %"PRIu64", %d",
-				path, jd->size, jd->offset, jd->create);
-		else		/* JF_REMOVE_OBJ */
-			sd_iprintf("%s (remove)", path);
-		break;
-	default:
-		panic("unknown type of journal flag: %d", jd->flag);
-		break;
-	}
-}
-
 static int replay_journal_entry(struct journal_descriptor *jd)
 {
 	char path[PATH_MAX];
@@ -162,23 +142,29 @@ static int replay_journal_entry(struct journal_descriptor *jd)
 	void *buf = NULL;
 	char *p = (char *)jd;
 
+	snprintf(path, PATH_MAX, "%s/%016"PRIx64,
+		 md_get_object_path(jd->oid), jd->oid);
+
 	if (jd->flag == JF_REMOVE_OBJ) {
-		journal_get_path(jd, path);
+		sd_info("%s (remove)", path);
 		unlink(path);
+
 		return 0;
 	}
 
+	sd_info("%s, size %" PRIu64 ", off %" PRIu64 ", %d", path, jd->size,
+		jd->offset, jd->create);
+
 	if (jd->create)
 		flags |= O_CREAT;
 
-	journal_get_path(jd, path);
 	fd = open(path, flags, sd_def_fmode);
 	if (fd < 0) {
-		sd_eprintf("open %m");
+		sd_err("open %m");
 		return -1;
 	}
 
-	if (jd->create && jd->flag == JF_STORE) {
+	if (jd->create) {
 		ret = prealloc(fd, get_objsize(jd->oid));
 		if (ret < 0)
 			goto out;
@@ -188,8 +174,7 @@ static int replay_journal_entry(struct journal_descriptor *jd)
 	memcpy(buf, p, jd->size);
 	size = xpwrite(fd, buf, jd->size, jd->offset);
 	if (size != jd->size) {
-		sd_eprintf("write %zd, size %" PRIu64 ", errno %m", size,
-			   jd->size);
+		sd_err("write %zd, size %" PRIu64 ", errno %m", size, jd->size);
 		ret = -1;
 		goto out;
 	}
@@ -207,7 +192,7 @@ static int do_recover(int fd)
 	struct stat st;
 
 	if (fstat(fd, &st) < 0) {
-		sd_eprintf("fstat %m");
+		sd_err("fstat %m");
 		return -1;
 	}
 
@@ -224,7 +209,7 @@ static int do_recover(int fd)
 	map = mmap(NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
 	close(fd);
 	if (map == MAP_FAILED) {
-		sd_eprintf("%m");
+		sd_err("%m");
 		return -1;
 	}
 
@@ -305,12 +290,12 @@ void clean_journal_file(const char *p)
 	snprintf(path, sizeof(path), "%s/%s", p, jfile_name[0]);
 	ret = unlink(path);
 	if (ret < 0)
-		sd_eprintf("unlink(%s): %m", path);
+		sd_err("unlink(%s): %m", path);
 
 	snprintf(path, sizeof(path), "%s/%s", p, jfile_name[1]);
 	ret = unlink(path);
 	if (ret < 0)
-		sd_eprintf("unlink(%s): %m", path);
+		sd_err("unlink(%s): %m", path);
 }
 
 static inline bool jfile_enough_space(size_t size)
@@ -332,13 +317,13 @@ static void *commit_data(void *ignored)
 
 	/* Tell runtime to release resources after termination */
 	err = pthread_detach(pthread_self());
-	if (err)
+	if (unlikely(err))
 		panic("%s", strerror(err));
 
 	sync();
-	if (ftruncate(jfile.commit_fd, 0) < 0)
+	if (unlikely(xftruncate(jfile.commit_fd, 0) < 0))
 		panic("truncate %m");
-	if (prealloc(jfile.commit_fd, jfile_size) < 0)
+	if (unlikely(prealloc(jfile.commit_fd, jfile_size) < 0))
 		panic("prealloc");
 
 	uatomic_set_false(&jfile.in_commit);
@@ -353,9 +338,9 @@ static void switch_journal_file(void)
 	pthread_t thread;
 
 retry:
-	if (!uatomic_set_true(&jfile.in_commit)) {
-		sd_eprintf("journal file in committing, "
-			   "you might need enlarge jfile size");
+	if (unlikely(!uatomic_set_true(&jfile.in_commit))) {
+		sd_err("journal file in committing, "
+		       "you might need enlarge jfile size");
 		usleep(100000); /* Wait until committing is finished */
 		goto retry;
 	}
@@ -368,7 +353,7 @@ retry:
 	jfile.pos = 0;
 
 	err = pthread_create(&thread, NULL, commit_data, NULL);
-	if (err)
+	if (unlikely(err))
 		panic("%s", strerror(err));
 }
 
@@ -407,8 +392,8 @@ static int journal_file_write(struct journal_descriptor *jd, const char *buf)
 	 * Feel free to correct me If I am wrong.
 	 */
 	written = xpwrite(jfile.fd, wbuffer, wsize, woff);
-	if (written != wsize) {
-		sd_eprintf("failed, written %zd, len %zd", written, wsize);
+	if (unlikely(written != wsize)) {
+		sd_err("failed, written %zd, len %zd", written, wsize);
 		/* FIXME: teach journal file handle EIO gracefully */
 		ret = SD_RES_EIO;
 		goto out;
