@@ -11,23 +11,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <pthread.h>
-#include <errno.h>
-#include <sys/file.h>
-#include <dirent.h>
-#include <urcu/uatomic.h>
-#include <sys/eventfd.h>
-
 #include "sheep_priv.h"
-#include "util.h"
-#include "strbuf.h"
-#include "rbtree.h"
 
 /*
  * Object Cache ID
@@ -44,10 +28,11 @@
 
 #define CACHE_INDEX_MASK      (CACHE_CREATE_BIT)
 
-#define CACHE_BLOCK_SIZE      ((UINT64_C(1) << 10) * 64) /* 64 KB */
-
 #define CACHE_OBJECT_SIZE (SD_DATA_OBJ_SIZE / 1024 / 1024) /* M */
 
+/* Kick background pusher if dirty_count greater than it */
+#define MAX_DIRTY_OBJECT_COUNT	10 /* Just a random number, no rationale */
+
 struct global_cache {
 	uint32_t capacity; /* The real capacity of object cache of this node */
 	uatomic_bool in_reclaim; /* If the relcaimer is working */
@@ -55,26 +40,35 @@ struct global_cache {
 
 struct object_cache_entry {
 	uint32_t idx; /* Index of this entry */
-	int refcnt; /* Reference count of this entry */
+	refcnt_t refcnt; /* Reference count of this entry */
 	uint64_t bmap; /* Each bit represents one dirty block in object */
 	struct object_cache *oc; /* Object cache this entry belongs to */
 	struct rb_node node; /* For lru tree of object cache */
 	struct list_head dirty_list; /* For dirty list of object cache */
 	struct list_head lru_list; /* For lru list of object cache */
 
-	pthread_rwlock_t lock; /* Entry lock */
+	struct sd_lock lock; /* Entry lock */
 };
 
 struct object_cache {
 	uint32_t vid; /* The VID of this VDI */
 	uint32_t push_count; /* How many push threads queued in push phase. */
+	uint32_t dirty_count; /* How many dirty object in this cache */
+	uint32_t total_count; /* Count of objects include dirty and clean */
 	struct hlist_node hash; /* VDI is linked to the global hash lists */
 	struct rb_root lru_tree; /* For faster object search */
 	struct list_head lru_head; /* Per VDI LRU list for reclaimer */
 	struct list_head dirty_head; /* Dirty objects linked to this list */
 	int push_efd; /* Used to synchronize between pusher and push threads */
+	uatomic_bool in_push; /* Whether if pusher is running */
 
-	pthread_rwlock_t lock; /* Cache lock */
+	struct sd_lock lock; /* Cache lock */
+};
+
+struct push_work {
+	struct work work;
+	struct object_cache_entry *entry;
+	struct object_cache *oc;
 };
 
 static struct global_cache gcache;
@@ -84,12 +78,14 @@ static int def_open_flags = O_RDWR;
 #define HASH_BITS	5
 #define HASH_SIZE	(1 << HASH_BITS)
 
-static pthread_rwlock_t hashtable_lock[HASH_SIZE] = {
-	[0 ... HASH_SIZE - 1] = PTHREAD_RWLOCK_INITIALIZER
+static struct sd_lock hashtable_lock[HASH_SIZE] = {
+	[0 ... HASH_SIZE - 1] = SD_LOCK_INITIALIZER
 };
 
 static struct hlist_head cache_hashtable[HASH_SIZE];
 
+static int object_cache_push(struct object_cache *oc);
+
 static inline bool entry_is_dirty(const struct object_cache_entry *entry)
 {
 	return !!entry->bmap;
@@ -119,34 +115,43 @@ static inline bool idx_has_vdi_bit(uint32_t idx)
 	return !!(idx & CACHE_VDI_BIT);
 }
 
-static uint64_t calc_object_bmap(size_t len, off_t offset)
+static inline size_t get_cache_block_size(uint64_t oid)
+{
+	size_t bsize = DIV_ROUND_UP(get_objsize(oid),
+				    sizeof(uint64_t) * BITS_PER_BYTE);
+
+	return round_up(bsize, BLOCK_SIZE); /* To be FS friendly */
+}
+
+static uint64_t calc_object_bmap(uint64_t oid, size_t len, off_t offset)
 {
 	int start, end, nr;
-	unsigned long bmap = 0;
+	uint64_t bmap = 0;
+	size_t bsize = get_cache_block_size(oid);
 
-	start = offset / CACHE_BLOCK_SIZE;
-	end = DIV_ROUND_UP(len + offset, CACHE_BLOCK_SIZE);
+	start = offset / bsize;
+	end = DIV_ROUND_UP(len + offset, bsize);
 	nr = end - start;
 
 	while (nr--)
 		set_bit(start + nr, &bmap);
 
-	return (uint64_t)bmap;
+	return bmap;
 }
 
 static inline void get_cache_entry(struct object_cache_entry *entry)
 {
-	uatomic_inc(&entry->refcnt);
+	refcount_inc(&entry->refcnt);
 }
 
 static inline void put_cache_entry(struct object_cache_entry *entry)
 {
-	uatomic_dec(&entry->refcnt);
+	refcount_dec(&entry->refcnt);
 }
 
 static inline bool entry_in_use(struct object_cache_entry *entry)
 {
-	return uatomic_read(&entry->refcnt) > 0;
+	return refcount_read(&entry->refcnt) > 0;
 }
 
 /*
@@ -163,32 +168,32 @@ static inline bool entry_in_use(struct object_cache_entry *entry)
  */
 static inline void read_lock_cache(struct object_cache *oc)
 {
-	pthread_rwlock_rdlock(&oc->lock);
+	sd_read_lock(&oc->lock);
 }
 
 static inline void write_lock_cache(struct object_cache *oc)
 {
-	pthread_rwlock_wrlock(&oc->lock);
+	sd_write_lock(&oc->lock);
 }
 
 static inline void unlock_cache(struct object_cache *oc)
 {
-	pthread_rwlock_unlock(&oc->lock);
+	sd_unlock(&oc->lock);
 }
 
 static inline void read_lock_entry(struct object_cache_entry *entry)
 {
-	pthread_rwlock_rdlock(&entry->lock);
+	sd_read_lock(&entry->lock);
 }
 
 static inline void write_lock_entry(struct object_cache_entry *entry)
 {
-	pthread_rwlock_wrlock(&entry->lock);
+	sd_write_lock(&entry->lock);
 }
 
 static inline void unlock_entry(struct object_cache_entry *entry)
 {
-	pthread_rwlock_unlock(&entry->lock);
+	sd_unlock(&entry->lock);
 }
 
 static struct object_cache_entry *
@@ -238,6 +243,55 @@ static struct object_cache_entry *lru_tree_search(struct rb_root *root,
 	return NULL;
 }
 
+static void do_background_push(struct work *work)
+{
+	struct push_work *pw = container_of(work, struct push_work, work);
+	struct object_cache *oc = pw->oc;
+
+	if (!uatomic_set_true(&oc->in_push))
+		return;
+
+	object_cache_push(oc);
+	uatomic_set_false(&oc->in_push);
+}
+
+static void background_push_done(struct work *work)
+{
+	struct push_work *pw = container_of(work, struct push_work, work);
+	free(pw);
+}
+
+static void kick_background_pusher(struct object_cache *oc)
+{
+	struct push_work *pw;
+
+	pw = xzalloc(sizeof(struct push_work));
+	pw->oc = oc;
+	pw->work.fn = do_background_push;
+	pw->work.done = background_push_done;
+	queue_work(sys->oc_push_wqueue, &pw->work);
+}
+
+static void del_from_dirty_list(struct object_cache_entry *entry)
+{
+	struct object_cache *oc = entry->oc;
+
+	list_del_init(&entry->dirty_list);
+	uatomic_dec(&oc->dirty_count);
+}
+
+static void add_to_dirty_list(struct object_cache_entry *entry)
+{
+	struct object_cache *oc = entry->oc;
+
+	list_add_tail(&entry->dirty_list, &oc->dirty_head);
+	/* FIXME read sys->status atomically */
+	if (uatomic_add_return(&oc->dirty_count, 1) > MAX_DIRTY_OBJECT_COUNT
+	    && !uatomic_is_true(&oc->in_push)
+	    && sys->cinfo.status == SD_STATUS_OK)
+		kick_background_pusher(oc);
+}
+
 static inline void
 free_cache_entry(struct object_cache_entry *entry)
 {
@@ -245,9 +299,10 @@ free_cache_entry(struct object_cache_entry *entry)
 
 	rb_erase(&entry->node, &oc->lru_tree);
 	list_del_init(&entry->lru_list);
+	oc->total_count--;
 	if (!list_empty(&entry->dirty_list))
-		list_del_init(&entry->dirty_list);
-	pthread_rwlock_destroy(&entry->lock);
+		del_from_dirty_list(entry);
+	sd_destroy_lock(&entry->lock);
 	free(entry);
 }
 
@@ -266,9 +321,9 @@ static int remove_cache_object(struct object_cache *oc, uint32_t idx)
 
 	snprintf(path, sizeof(path), "%s/%06"PRIx32"/%08"PRIx32,
 		 object_cache_dir, oc->vid, idx);
-	sd_dprintf("%"PRIx64, idx_to_oid(oc->vid, idx));
-	if (unlink(path) < 0) {
-		sd_eprintf("failed to remove cached object %m");
+	sd_debug("%"PRIx64, idx_to_oid(oc->vid, idx));
+	if (unlikely(unlink(path) < 0)) {
+		sd_err("failed to remove cached object %m");
 		if (errno == ENOENT)
 			return SD_RES_SUCCESS;
 		ret = SD_RES_EIO;
@@ -294,17 +349,17 @@ static int read_cache_object_noupdate(uint32_t vid, uint32_t idx, void *buf,
 	}
 
 	fd = open(p, flags, sd_def_fmode);
-	if (fd < 0) {
-		sd_eprintf("%m");
+	if (unlikely(fd < 0)) {
+		sd_err("%m");
 		ret = SD_RES_EIO;
 		goto out;
 	}
 
 	size = xpread(fd, buf, count, offset);
 
-	if (size != count) {
-		sd_eprintf("size %zu, count:%zu, offset %jd %m", size, count,
-			   (intmax_t)offset);
+	if (unlikely(size != count)) {
+		sd_err("size %zu, count:%zu, offset %jd %m", size, count,
+		       (intmax_t)offset);
 		ret = SD_RES_EIO;
 		goto out_close;
 	}
@@ -330,17 +385,17 @@ static int write_cache_object_noupdate(uint32_t vid, uint32_t idx, void *buf,
 	}
 
 	fd = open(p, flags, sd_def_fmode);
-	if (fd < 0) {
-		sd_eprintf("%m");
+	if (unlikely(fd < 0)) {
+		sd_err("%m");
 		ret = SD_RES_EIO;
 		goto out;
 	}
 
 	size = xpwrite(fd, buf, count, offset);
 
-	if (size != count) {
-		sd_eprintf("size %zu, count:%zu, offset %jd %m", size, count,
-			   (intmax_t)offset);
+	if (unlikely(size != count)) {
+		sd_err("size %zu, count:%zu, offset %jd %m", size, count,
+		       (intmax_t)offset);
 		ret = SD_RES_EIO;
 		goto out_close;
 	}
@@ -387,9 +442,9 @@ static int write_cache_object(struct object_cache_entry *entry, void *buf,
 	}
 	write_lock_cache(oc);
 	if (writeback) {
-		entry->bmap |= calc_object_bmap(count, offset);
+		entry->bmap |= calc_object_bmap(oid, count, offset);
 		if (list_empty(&entry->dirty_list))
-			list_add_tail(&entry->dirty_list, &oc->dirty_head);
+			add_to_dirty_list(entry);
 	}
 	list_move_tail(&entry->lru_list, &oc->lru_head);
 	unlock_cache(oc);
@@ -411,8 +466,8 @@ static int write_cache_object(struct object_cache_entry *entry, void *buf,
 
 	ret = exec_local_req(&hdr, buf);
 	if (ret != SD_RES_SUCCESS) {
-		sd_eprintf("failed to write object %" PRIx64 ", %s", oid,
-			   sd_strerror(ret));
+		sd_err("failed to write object %" PRIx64 ", %s", oid,
+		       sd_strerror(ret));
 		return ret;
 	}
 out:
@@ -425,32 +480,24 @@ static int push_cache_object(uint32_t vid, uint32_t idx, uint64_t bmap,
 	struct sd_req hdr;
 	void *buf;
 	off_t offset;
-	unsigned data_length;
-	int ret = SD_RES_NO_MEM;
 	uint64_t oid = idx_to_oid(vid, idx);
+	size_t data_length, bsize = get_cache_block_size(oid);
+	int ret = SD_RES_NO_MEM;
 	int first_bit, last_bit;
 
-	sd_dprintf("%"PRIx64", create %d", oid, create);
-
 	if (!bmap) {
-		sd_dprintf("WARN: nothing to flush");
+		sd_debug("WARN: nothing to flush %"PRIx64, oid);
 		return SD_RES_SUCCESS;
 	}
 
 	first_bit = ffsll(bmap) - 1;
 	last_bit = fls64(bmap) - 1;
 
-	sd_dprintf("bmap:0x%"PRIx64", first_bit:%d, last_bit:%d", bmap,
-		   first_bit, last_bit);
-	offset = first_bit * CACHE_BLOCK_SIZE;
-	data_length = (last_bit - first_bit + 1) * CACHE_BLOCK_SIZE;
-
-	/*
-	 * CACHE_BLOCK_SIZE may not be divisible by SD_INODE_SIZE,
-	 * so (offset + data_length) could larger than SD_INODE_SIZE
-	 */
-	if (is_vdi_obj(oid) && (offset + data_length) > SD_INODE_SIZE)
-		data_length = SD_INODE_SIZE - offset;
+	sd_debug("%"PRIx64" bmap(%zd):0x%"PRIx64", first_bit:%d, last_bit:%d",
+		 oid, bsize, bmap, first_bit, last_bit);
+	offset = first_bit * bsize;
+	data_length = min((last_bit - first_bit + 1) * bsize,
+			  get_objsize(oid) - offset);
 
 	buf = xvalloc(data_length);
 	ret = read_cache_object_noupdate(vid, idx, buf, data_length, offset);
@@ -468,7 +515,8 @@ static int push_cache_object(uint32_t vid, uint32_t idx, uint64_t bmap,
 
 	ret = exec_local_req(&hdr, buf);
 	if (ret != SD_RES_SUCCESS)
-		sd_eprintf("failed to push object %s", sd_strerror(ret));
+		sd_err("failed to push object %" PRIx64 ", %s", oid,
+		       sd_strerror(ret));
 out:
 	free(buf);
 	return ret;
@@ -498,18 +546,25 @@ static void do_reclaim_object(struct object_cache *oc)
 	list_for_each_entry_safe(entry, t, &oc->lru_head, lru_list) {
 		oid = idx_to_oid(oc->vid, entry_idx(entry));
 		if (entry_in_use(entry)) {
-			sd_dprintf("%"PRIx64" is in use, skip...", oid);
+			sd_debug("%"PRIx64" is in use, skip...", oid);
 			continue;
 		}
-		if (entry_is_dirty(entry)) {
-			sd_dprintf("%"PRIx64" is dirty, skip...", oid);
+
+		/*
+		 * The shared snapshot objects won't be released after being
+		 * pulled and if sheep restarts, the remaining snapshot objects
+		 * will be marked as dirty. So for these kind of objects, we
+		 * can reclaim them safely.
+		 */
+		if (entry_is_dirty(entry) && !oid_is_readonly(oid)) {
+			sd_debug("%"PRIx64" is dirty, skip...", oid);
 			continue;
 		}
 		if (remove_cache_object(oc, entry_idx(entry)) != SD_RES_SUCCESS)
 			continue;
 		free_cache_entry(entry);
 		cap = uatomic_sub_return(&gcache.capacity, CACHE_OBJECT_SIZE);
-		sd_dprintf("%"PRIx64" reclaimed. capacity:%"PRId32, oid, cap);
+		sd_debug("%"PRIx64" reclaimed. capacity:%"PRId32, oid, cap);
 		if (cap <= HIGH_WATERMARK)
 			break;
 	}
@@ -536,21 +591,21 @@ static void do_reclaim(struct work *work)
 		int idx = (i + j) % HASH_SIZE;
 		struct hlist_head *head = cache_hashtable + idx;
 
-		pthread_rwlock_rdlock(&hashtable_lock[idx]);
+		sd_read_lock(&hashtable_lock[idx]);
 		hlist_for_each_entry(cache, node, head, hash) {
 			uint32_t cap;
 
 			do_reclaim_object(cache);
 			cap = uatomic_read(&gcache.capacity);
 			if (cap <= HIGH_WATERMARK) {
-				pthread_rwlock_unlock(&hashtable_lock[idx]);
-				sd_dprintf("complete, capacity %"PRIu32, cap);
+				sd_unlock(&hashtable_lock[idx]);
+				sd_debug("complete, capacity %"PRIu32, cap);
 				return;
 			}
 		}
-		pthread_rwlock_unlock(&hashtable_lock[idx]);
+		sd_unlock(&hashtable_lock[idx]);
 	}
-	sd_dprintf("finished");
+	sd_debug("finished");
 }
 
 static void reclaim_done(struct work *work)
@@ -567,7 +622,7 @@ static int create_dir_for(uint32_t vid)
 
 	snprintf(p, sizeof(p), "%s/%06"PRIx32, object_cache_dir, vid);
 	if (xmkdir(p, sd_def_dmode) < 0) {
-		sd_eprintf("%s, %m", p);
+		sd_err("%s, %m", p);
 		ret = -1;
 	}
 	return ret;
@@ -581,9 +636,9 @@ static struct object_cache *find_object_cache(uint32_t vid, bool create)
 	struct hlist_node *node;
 
 	if (create)
-		pthread_rwlock_wrlock(&hashtable_lock[h]);
+		sd_write_lock(&hashtable_lock[h]);
 	else
-		pthread_rwlock_rdlock(&hashtable_lock[h]);
+		sd_read_lock(&hashtable_lock[h]);
 
 	if (hlist_empty(head))
 		goto not_found;
@@ -603,13 +658,13 @@ not_found:
 		INIT_LIST_HEAD(&cache->dirty_head);
 		INIT_LIST_HEAD(&cache->lru_head);
 
-		pthread_rwlock_init(&cache->lock, NULL);
+		sd_init_lock(&cache->lock);
 		hlist_add_head(&cache->hash, head);
 	} else {
 		cache = NULL;
 	}
 out:
-	pthread_rwlock_unlock(&hashtable_lock[h]);
+	sd_unlock(&hashtable_lock[h]);
 	return cache;
 }
 
@@ -642,7 +697,7 @@ alloc_cache_entry(struct object_cache *oc, uint32_t idx)
 	entry = xzalloc(sizeof(*entry));
 	entry->oc = oc;
 	entry->idx = idx;
-	pthread_rwlock_init(&entry->lock, NULL);
+	sd_init_lock(&entry->lock);
 	INIT_LIST_HEAD(&entry->dirty_list);
 	INIT_LIST_HEAD(&entry->lru_list);
 
@@ -653,18 +708,19 @@ static void add_to_lru_cache(struct object_cache *oc, uint32_t idx, bool create)
 {
 	struct object_cache_entry *entry = alloc_cache_entry(oc, idx);
 
-	sd_dprintf("oid %"PRIx64" added", idx_to_oid(oc->vid, idx));
+	sd_debug("oid %"PRIx64" added", idx_to_oid(oc->vid, idx));
 
 	write_lock_cache(oc);
-	if (lru_tree_insert(&oc->lru_tree, entry))
+	if (unlikely(lru_tree_insert(&oc->lru_tree, entry)))
 		panic("the object already exist");
 	uatomic_add(&gcache.capacity, CACHE_OBJECT_SIZE);
 	list_add_tail(&entry->lru_list, &oc->lru_head);
+	oc->total_count++;
 	if (create) {
 		/* Cache lock assure it is not raced with pusher */
 		entry->bmap = UINT64_MAX;
 		entry->idx |= CACHE_CREATE_BIT;
-		list_add_tail(&entry->dirty_list, &oc->dirty_head);
+		add_to_dirty_list(entry);
 	}
 	unlock_cache(oc);
 }
@@ -674,8 +730,8 @@ static inline int lookup_path(char *path)
 	int ret = SD_RES_SUCCESS;
 
 	if (access(path, R_OK | W_OK) < 0) {
-		if (errno != ENOENT) {
-			sd_dprintf("%m");
+		if (unlikely(errno != ENOENT)) {
+			sd_debug("%m");
 			ret = SD_RES_EIO;
 		} else {
 			ret = SD_RES_NO_CACHE;
@@ -697,13 +753,13 @@ static int object_cache_lookup(struct object_cache *oc, uint32_t idx,
 
 	flags |= O_CREAT | O_TRUNC;
 	fd = open(path, flags, sd_def_fmode);
-	if (fd < 0) {
-		sd_dprintf("%s, %m", path);
+	if (unlikely(fd < 0)) {
+		sd_debug("%s, %m", path);
 		ret = SD_RES_EIO;
 		goto out;
 	}
 	ret = prealloc(fd, get_objsize(idx_to_oid(oc->vid, idx)));
-	if (ret < 0) {
+	if (unlikely(ret < 0)) {
 		ret = SD_RES_EIO;
 		goto out_close;
 	}
@@ -727,11 +783,11 @@ static int create_cache_object(struct object_cache *oc, uint32_t idx,
 		object_cache_dir, oc->vid, idx);
 	fd = open(tmp_path, flags, sd_def_fmode);
 	if (fd < 0) {
-		if (errno == EEXIST) {
-			sd_dprintf("%08"PRIx32" already created", idx);
+		if (likely(errno == EEXIST)) {
+			sd_debug("%08"PRIx32" already created", idx);
 			goto out;
 		}
-		sd_dprintf("%m");
+		sd_debug("%m");
 		ret = SD_RES_EIO;
 		goto out;
 	}
@@ -739,35 +795,35 @@ static int create_cache_object(struct object_cache *oc, uint32_t idx,
 	/* We need to extend it if the buffer is trimmed */
 	if (offset != 0 || buf_size != obj_size) {
 		ret = prealloc(fd, obj_size);
-		if (ret < 0) {
+		if (unlikely(ret < 0)) {
 			ret = SD_RES_EIO;
-			sd_eprintf("%m");
+			sd_err("%m");
 			goto out_close;
 		}
 	}
 
 	ret = xpwrite(fd, buffer, buf_size, offset);
-	if (ret != buf_size) {
+	if (unlikely(ret != buf_size)) {
 		ret = SD_RES_EIO;
-		sd_eprintf("failed, vid %"PRIx32", idx %"PRIx32, oc->vid, idx);
+		sd_err("failed, vid %"PRIx32", idx %"PRIx32, oc->vid, idx);
 		goto out_close;
 	}
 	/* This is intended to take care of partial write due to crash */
 	snprintf(path, sizeof(path), "%s/%06"PRIx32"/%08"PRIx32,
 		 object_cache_dir, oc->vid, idx);
 	ret = link(tmp_path, path);
-	if (ret < 0) {
+	if (unlikely(ret < 0)) {
 		if (errno == EEXIST) {
 			ret = SD_RES_OID_EXIST;
 			goto out_close;
 		}
-		sd_dprintf("failed to link %s to %s: %m", tmp_path, path);
+		sd_debug("failed to link %s to %s: %m", tmp_path, path);
 		/* FIXME: teach object cache handle EIO gracefully */
 		ret = SD_RES_EIO;
 		goto out_close;
 	}
 	ret = SD_RES_SUCCESS;
-	sd_dprintf("%08"PRIx32" size %zu", idx, obj_size);
+	sd_debug("%08"PRIx32" size %zu", idx, obj_size);
 out_close:
 	close(fd);
 	unlink(tmp_path);
@@ -794,7 +850,7 @@ static int object_cache_pull(struct object_cache *oc, uint32_t idx)
 	if (ret != SD_RES_SUCCESS)
 		goto err;
 
-	sd_dprintf("oid %"PRIx64" pulled successfully", oid);
+	sd_debug("oid %"PRIx64" pulled successfully", oid);
 	ret = create_cache_object(oc, idx, buf, rsp->data_length,
 				  rsp->obj.offset, data_length);
 	/*
@@ -820,11 +876,6 @@ err:
 	return ret;
 }
 
-struct push_work {
-	struct work work;
-	struct object_cache_entry *entry;
-};
-
 static void do_push_object(struct work *work)
 {
 	struct push_work *pw = container_of(work, struct push_work, work);
@@ -832,20 +883,30 @@ static void do_push_object(struct work *work)
 	struct object_cache *oc = entry->oc;
 	uint64_t oid = idx_to_oid(oc->vid, entry_idx(entry));
 
-	sd_dprintf("%"PRIx64, oid);
+	sd_debug("%"PRIx64, oid);
 
 	read_lock_entry(entry);
-	if (push_cache_object(oc->vid, entry_idx(entry), entry->bmap,
-			      !!(entry->idx & CACHE_CREATE_BIT))
-	    != SD_RES_SUCCESS)
+	/*
+	 * We might happen to push readonly object in following scenario
+	 * 1. sheep pulled some read-only objects
+	 * 2. sheep crashed
+	 * 3. sheep restarted and marked all the objects in cache dirty blindly
+	 */
+	if (oid_is_readonly(idx_to_oid(oc->vid, entry_idx(entry))))
+		goto clean;
+
+	if (unlikely(push_cache_object(oc->vid, entry_idx(entry), entry->bmap,
+				       !!(entry->idx & CACHE_CREATE_BIT))
+		     != SD_RES_SUCCESS))
 		panic("push failed but should never fail");
+clean:
 	if (uatomic_sub_return(&oc->push_count, 1) == 0)
-		eventfd_write(oc->push_efd, 1);
+		eventfd_xwrite(oc->push_efd, 1);
 	entry->idx &= ~CACHE_CREATE_BIT;
 	entry->bmap = 0;
 	unlock_entry(entry);
 
-	sd_dprintf("%"PRIx64" done", oid);
+	sd_debug("%"PRIx64" done", oid);
 	put_cache_entry(entry);
 }
 
@@ -867,32 +928,30 @@ static void push_object_done(struct work *work)
 static int object_cache_push(struct object_cache *oc)
 {
 	struct object_cache_entry *entry, *t;
-	eventfd_t value;
 
 	write_lock_cache(oc);
 	if (list_empty(&oc->dirty_head)) {
 		unlock_cache(oc);
 		return SD_RES_SUCCESS;
 	}
+
+	uatomic_set(&oc->push_count, uatomic_read(&oc->dirty_count));
 	list_for_each_entry_safe(entry, t, &oc->dirty_head, dirty_list) {
 		struct push_work *pw;
 
 		get_cache_entry(entry);
-		uatomic_inc(&oc->push_count);
 		pw = xzalloc(sizeof(struct push_work));
 		pw->work.fn = do_push_object;
 		pw->work.done = push_object_done;
 		pw->entry = entry;
 		queue_work(sys->oc_push_wqueue, &pw->work);
-		list_del_init(&entry->dirty_list);
+		del_from_dirty_list(entry);
 	}
 	unlock_cache(oc);
-reread:
-	if (eventfd_read(oc->push_efd, &value) < 0) {
-		sd_eprintf("eventfd read failed, %m");
-		goto reread;
-	}
-	sd_dprintf("%"PRIx32" completed", oc->vid);
+
+	eventfd_xread(oc->push_efd);
+
+	sd_debug("%"PRIx32" completed", oc->vid);
 	return SD_RES_SUCCESS;
 }
 
@@ -921,9 +980,9 @@ void object_cache_delete(uint32_t vid)
 		return;
 
 	/* Firstly we free memeory */
-	pthread_rwlock_wrlock(&hashtable_lock[h]);
+	sd_write_lock(&hashtable_lock[h]);
 	hlist_del(&cache->hash);
-	pthread_rwlock_unlock(&hashtable_lock[h]);
+	sd_unlock(&hashtable_lock[h]);
 
 	write_lock_cache(cache);
 	list_for_each_entry_safe(entry, t, &cache->lru_head, lru_list) {
@@ -931,7 +990,7 @@ void object_cache_delete(uint32_t vid)
 		uatomic_sub(&gcache.capacity, CACHE_OBJECT_SIZE);
 	}
 	unlock_cache(cache);
-	pthread_rwlock_destroy(&cache->lock);
+	sd_destroy_lock(&cache->lock);
 	close(cache->push_efd);
 	free(cache);
 
@@ -957,6 +1016,23 @@ get_cache_entry_from(struct object_cache *cache, uint32_t idx)
 	return entry;
 }
 
+/* This helper increases the refcount */
+static struct object_cache_entry *oid_to_entry(uint64_t oid)
+{
+	uint32_t vid = oid_to_vid(oid);
+	uint32_t idx = object_cache_oid_to_idx(oid);
+	struct object_cache *cache;
+	struct object_cache_entry *entry;
+
+	cache = find_object_cache(vid, false);
+	entry = get_cache_entry_from(cache, idx);
+	if (!entry) {
+		sd_debug("%" PRIx64 " doesn't exist", oid);
+		return NULL;
+	}
+	return entry;
+}
+
 static int object_cache_flush_and_delete(struct object_cache *oc)
 {
 	DIR *dir;
@@ -967,11 +1043,11 @@ static int object_cache_flush_and_delete(struct object_cache *oc)
 	int ret = 0;
 	char p[PATH_MAX];
 
-	sd_dprintf("%"PRIx32, vid);
+	sd_debug("%"PRIx32, vid);
 	snprintf(p, sizeof(p), "%s/%06"PRIx32, object_cache_dir, vid);
 	dir = opendir(p);
 	if (!dir) {
-		sd_dprintf("%m");
+		sd_debug("%m");
 		ret = -1;
 		goto out;
 	}
@@ -980,9 +1056,9 @@ static int object_cache_flush_and_delete(struct object_cache *oc)
 		if (!strncmp(d->d_name, ".", 1))
 			continue;
 		if (strcmp(d->d_name + 8, ".tmp") == 0) {
-			sd_dprintf("try to del %s", d->d_name);
+			sd_debug("try to del %s", d->d_name);
 			if (unlinkat(dirfd(dir), d->d_name, 0) < 0)
-				sd_eprintf("%m");
+				sd_err("%m");
 			continue;
 		}
 
@@ -991,8 +1067,6 @@ static int object_cache_flush_and_delete(struct object_cache *oc)
 			continue;
 		if (push_cache_object(vid, idx, all, true) !=
 		    SD_RES_SUCCESS) {
-			sd_dprintf("failed to push %"PRIx64,
-				   idx_to_oid(vid, idx));
 			ret = -1;
 			goto out_close_dir;
 		}
@@ -1012,6 +1086,11 @@ bool bypass_object_cache(const struct request *req)
 	if (!sys->enable_object_cache || req->local)
 		return true;
 
+	/* For vmstate && vdi_attr object, we don't do caching */
+	if (is_vmstate_obj(oid) || is_vdi_attr_obj(oid) ||
+	    req->rq.flags & SD_FLAG_CMD_COW)
+		return true;
+
 	if (req->rq.flags & SD_FLAG_CMD_DIRECT) {
 		uint32_t vid = oid_to_vid(oid);
 		struct object_cache *cache;
@@ -1033,10 +1112,6 @@ bool bypass_object_cache(const struct request *req)
 		}
 	}
 
-	/* For vmstate && vdi_attr object, we don't do caching */
-	if (is_vmstate_obj(oid) || is_vdi_attr_obj(oid) ||
-	    req->rq.flags & SD_FLAG_CMD_COW)
-		return true;
 	return false;
 }
 
@@ -1051,8 +1126,8 @@ int object_cache_handle_request(struct request *req)
 	int ret;
 	bool create = false;
 
-	sd_dprintf("%08"PRIx32", len %"PRIu32", off %"PRIu64, idx,
-		   hdr->data_length, hdr->obj.offset);
+	sd_debug("%08" PRIx32 ", len %" PRIu32 ", off %" PRIu64, idx,
+		 hdr->data_length, hdr->obj.offset);
 
 	cache = find_object_cache(vid, true);
 
@@ -1073,7 +1148,7 @@ retry:
 
 	entry = get_cache_entry_from(cache, idx);
 	if (!entry) {
-		sd_dprintf("retry oid %"PRIx64, oid);
+		sd_debug("retry oid %"PRIx64, oid);
 		/*
 		 * For the case that object exists but isn't added to object
 		 * list yet, we call pthread_yield() to expect other thread can
@@ -1104,17 +1179,12 @@ err:
 int object_cache_write(uint64_t oid, char *data, unsigned int datalen,
 		       uint64_t offset, bool create)
 {
-	uint32_t vid = oid_to_vid(oid);
-	uint32_t idx = object_cache_oid_to_idx(oid);
-	struct object_cache *cache;
-	struct object_cache_entry *entry;
+	struct object_cache_entry *entry = oid_to_entry(oid);
 	int ret;
 
-	sd_dprintf("%" PRIx64, oid);
-	cache = find_object_cache(vid, false);
-	entry = get_cache_entry_from(cache, idx);
+	sd_debug("%" PRIx64, oid);
 	if (!entry) {
-		sd_dprintf("%" PRIx64 " doesn't exist", oid);
+		sd_debug("%" PRIx64 " doesn't exist", oid);
 		return SD_RES_NO_CACHE;
 	}
 
@@ -1126,17 +1196,12 @@ int object_cache_write(uint64_t oid, char *data, unsigned int datalen,
 int object_cache_read(uint64_t oid, char *data, unsigned int datalen,
 		      uint64_t offset)
 {
-	uint32_t vid = oid_to_vid(oid);
-	uint32_t idx = object_cache_oid_to_idx(oid);
-	struct object_cache *cache;
-	struct object_cache_entry *entry;
+	struct object_cache_entry *entry = oid_to_entry(oid);
 	int ret;
 
-	sd_dprintf("%" PRIx64, oid);
-	cache = find_object_cache(vid, false);
-	entry = get_cache_entry_from(cache, idx);
+	sd_debug("%" PRIx64, oid);
 	if (!entry) {
-		sd_dprintf("%" PRIx64 " doesn't exist", oid);
+		sd_debug("%" PRIx64 " doesn't exist", oid);
 		return SD_RES_NO_CACHE;
 	}
 
@@ -1148,14 +1213,25 @@ int object_cache_read(uint64_t oid, char *data, unsigned int datalen,
 int object_cache_flush_vdi(uint32_t vid)
 {
 	struct object_cache *cache;
+	int ret;
 
 	cache = find_object_cache(vid, false);
 	if (!cache) {
-		sd_dprintf("%"PRIx32" not found", vid);
+		sd_debug("%"PRIx32" not found", vid);
 		return SD_RES_SUCCESS;
 	}
 
-	return object_cache_push(cache);
+	/*
+	 * We have to wait for last pusher finishing and push again so
+	 * that dirty bits produced while it is waiting are guaranteed
+	 * to be pushed back
+	 */
+	while (!uatomic_set_true(&cache->in_push))
+		usleep(100000);
+
+	ret = object_cache_push(cache);
+	uatomic_set_false(&cache->in_push);
+	return ret;
 }
 
 int object_cache_flush_and_del(const struct request *req)
@@ -1183,7 +1259,7 @@ static int load_cache_object(struct object_cache *cache)
 		 cache->vid);
 	dir = opendir(path);
 	if (!dir) {
-		sd_dprintf("%m");
+		sd_debug("%m");
 		ret = -1;
 		goto out;
 	}
@@ -1193,9 +1269,9 @@ static int load_cache_object(struct object_cache *cache)
 			continue;
 
 		if (strcmp(d->d_name + 8, ".tmp") == 0) {
-			sd_dprintf("try to del %s", d->d_name);
+			sd_debug("try to del %s", d->d_name);
 			if (unlinkat(dirfd(dir), d->d_name, 0) < 0)
-				sd_eprintf("%m");
+				sd_err("%m");
 			continue;
 		}
 
@@ -1210,7 +1286,7 @@ static int load_cache_object(struct object_cache *cache)
 		 * cluster isn't fully working.
 		 */
 		add_to_lru_cache(cache, idx, true);
-		sd_dprintf("%"PRIx64, idx_to_oid(cache->vid, idx));
+		sd_debug("%"PRIx64, idx_to_oid(cache->vid, idx));
 	}
 
 	closedir(dir);
@@ -1229,7 +1305,7 @@ static int load_cache(void)
 	snprintf(path, sizeof(path), "%s", object_cache_dir);
 	dir = opendir(path);
 	if (!dir) {
-		sd_dprintf("%m");
+		sd_debug("%m");
 		ret = -1;
 		goto out;
 	}
@@ -1249,6 +1325,41 @@ out:
 	return ret;
 }
 
+int object_cache_remove(uint64_t oid)
+{
+	/* Inc the entry refcount to exclude the reclaimer */
+	struct object_cache_entry *entry = oid_to_entry(oid);
+	struct object_cache *oc = entry->oc;
+	int ret;
+
+	if (!entry)
+		return SD_RES_NO_OBJ;
+
+	sd_debug("%" PRIx64, oid);
+	while (refcount_read(&entry->refcnt) > 1)
+		usleep(100000); /* Object might be in push */
+
+	write_lock_cache(oc);
+	/*
+	 * We assume no other thread will inc the refcount of this entry
+	 * before we call write_lock_cache(). object_cache_remove() is called
+	 * in the DISCARD context, which means nornamly no other read/write
+	 * requests.
+	 */
+	assert(refcount_read(&entry->refcnt) == 1);
+	ret = remove_cache_object(oc, entry_idx(entry));
+	if (ret != SD_RES_SUCCESS) {
+		unlock_cache(oc);
+		return ret;
+	}
+	free_cache_entry(entry);
+	unlock_cache(oc);
+
+	uatomic_sub(&gcache.capacity, CACHE_OBJECT_SIZE);
+
+	return SD_RES_SUCCESS;
+}
+
 int object_cache_init(const char *p)
 {
 	int ret = 0;
@@ -1256,13 +1367,13 @@ int object_cache_init(const char *p)
 
 	strbuf_addstr(&buf, p);
 	if (xmkdir(buf.buf, sd_def_dmode) < 0) {
-		sd_eprintf("%s %m", buf.buf);
+		sd_err("%s %m", buf.buf);
 		ret = -1;
 		goto err;
 	}
 	strbuf_addstr(&buf, "/cache");
 	if (xmkdir(buf.buf, sd_def_dmode) < 0) {
-		sd_eprintf("%s %m", buf.buf);
+		sd_err("%s %m", buf.buf);
 		ret = -1;
 		goto err;
 	}
@@ -1291,3 +1402,31 @@ void object_cache_format(void)
 	}
 	uatomic_set(&gcache.capacity, 0);
 }
+
+int object_cache_get_info(struct object_cache_info *info)
+{
+	int j = 0;
+
+	info->used = gcache.capacity * 1024 * 1024;
+	info->size = sys->object_cache_size * 1024 * 1024;
+
+	for (int i = 0; i < HASH_SIZE; i++) {
+		struct hlist_head *head = cache_hashtable + i;
+		struct object_cache *cache;
+		struct hlist_node *node;
+
+		sd_read_lock(&hashtable_lock[i]);
+		hlist_for_each_entry(cache, node, head, hash) {
+			read_lock_cache(cache);
+			info->caches[j].vid = cache->vid;
+			info->caches[j].dirty = cache->dirty_count;
+			info->caches[j].total = cache->total_count;
+			j++;
+			unlock_cache(cache);
+		}
+		sd_unlock(&hashtable_lock[i]);
+	}
+	info->count = j;
+
+	return sizeof(*info);
+}
