@@ -8,24 +8,9 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <dirent.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <mntent.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/statvfs.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <pthread.h>
 
 #include "sheep_priv.h"
-#include "strbuf.h"
 #include "trace/trace.h"
-#include "util.h"
 
 enum sd_op_type {
 	SD_OP_TYPE_CLUSTER = 1, /* cluster operations */
@@ -88,7 +73,8 @@ static int cluster_new_vdi(struct request *req)
 		.size = hdr->vdi.vdi_size,
 		.base_vid = hdr->vdi.base_vdi_id,
 		.create_snapshot = !!hdr->vdi.snapid,
-		.nr_copies = hdr->vdi.copies ? hdr->vdi.copies : sys->nr_copies,
+		.nr_copies = hdr->vdi.copies ? hdr->vdi.copies :
+				sys->cinfo.nr_copies,
 	};
 
 	if (hdr->data_length != SD_MAX_VDI_LEN)
@@ -108,9 +94,9 @@ static int post_cluster_new_vdi(const struct sd_req *req, struct sd_rsp *rsp,
 	unsigned long nr = rsp->vdi.vdi_id;
 	int ret = rsp->result;
 
-	sd_dprintf("done %d %lx", ret, nr);
+	sd_debug("done %d %lx", ret, nr);
 	if (ret == SD_RES_SUCCESS)
-		set_bit(nr, sys->vdi_inuse);
+		atomic_set_bit(nr, sys->vdi_inuse);
 
 	return ret;
 }
@@ -223,11 +209,11 @@ static int remove_epoch(uint32_t epoch)
 	int ret;
 	char path[PATH_MAX];
 
-	sd_dprintf("remove epoch %"PRIu32, epoch);
+	sd_debug("remove epoch %"PRIu32, epoch);
 	snprintf(path, sizeof(path), "%s%08u", epoch_path, epoch);
 	ret = unlink(path);
 	if (ret && ret != -ENOENT) {
-		sd_eprintf("failed to remove %s: %s", path, strerror(-ret));
+		sd_err("failed to remove %s: %s", path, strerror(-ret));
 		return SD_RES_EIO;
 	}
 
@@ -239,7 +225,6 @@ static int cluster_make_fs(const struct sd_req *req, struct sd_rsp *rsp,
 {
 	int i, ret;
 	uint32_t latest_epoch;
-	uint64_t created_time;
 	struct store_driver *driver;
 	char *store_name = data;
 
@@ -247,28 +232,25 @@ static int cluster_make_fs(const struct sd_req *req, struct sd_rsp *rsp,
 	if (!driver)
 		return SD_RES_NO_STORE;
 
+	pstrcpy((char *)sys->cinfo.store, sizeof(sys->cinfo.store),
+		store_name);
 	sd_store = driver;
 	latest_epoch = get_latest_epoch();
 
 	ret = sd_store->format();
 	if (ret != SD_RES_SUCCESS)
 		return ret;
-	if (set_cluster_store(store_name) < 0)
-		return SD_RES_EIO;
 
 	ret = sd_store->init();
 	if (ret != SD_RES_SUCCESS)
 		return ret;
 
-	sys->nr_copies = req->cluster.copies;
-	sys->flags = req->flags;
-	if (!sys->nr_copies)
-		sys->nr_copies = SD_DEFAULT_COPIES;
-
-	created_time = req->cluster.ctime;
-	set_cluster_ctime(created_time);
-	set_cluster_copies(sys->nr_copies);
-	set_cluster_flags(sys->flags);
+	sys->cinfo.nr_copies = req->cluster.copies;
+	sys->cinfo.flags = req->flags;
+	if (!sys->cinfo.nr_copies)
+		sys->cinfo.nr_copies = SD_DEFAULT_COPIES;
+	sys->cinfo.ctime = req->cluster.ctime;
+	set_cluster_config(&sys->cinfo);
 
 	for (i = 1; i <= latest_epoch; i++)
 		remove_epoch(i);
@@ -276,16 +258,13 @@ static int cluster_make_fs(const struct sd_req *req, struct sd_rsp *rsp,
 	memset(sys->vdi_inuse, 0, sizeof(sys->vdi_inuse));
 	clean_vdi_state();
 
-	sys->epoch = 1;
+	sys->cinfo.epoch = 0;
 
-	ret = log_current_epoch();
+	ret = inc_and_log_epoch();
 	if (ret)
 		return SD_RES_EIO;
 
-	if (have_enough_zones())
-		sys->status = SD_STATUS_OK;
-	else
-		sys->status = SD_STATUS_HALT;
+	sys->cinfo.status = SD_STATUS_OK;
 
 	return SD_RES_SUCCESS;
 }
@@ -293,14 +272,14 @@ static int cluster_make_fs(const struct sd_req *req, struct sd_rsp *rsp,
 static int cluster_shutdown(const struct sd_req *req, struct sd_rsp *rsp,
 			    void *data)
 {
-	sys->status = SD_STATUS_SHUTDOWN;
+	sys->cinfo.status = SD_STATUS_SHUTDOWN;
 	return SD_RES_SUCCESS;
 }
 
 static int cluster_enable_recover(const struct sd_req *req,
 				    struct sd_rsp *rsp, void *data)
 {
-	sys->disable_recovery = false;
+	sys->cinfo.disable_recovery = false;
 	resume_suspended_recovery();
 	return SD_RES_SUCCESS;
 }
@@ -308,7 +287,7 @@ static int cluster_enable_recover(const struct sd_req *req,
 static int cluster_disable_recover(const struct sd_req *req,
 				   struct sd_rsp *rsp, void *data)
 {
-	sys->disable_recovery = true;
+	sys->cinfo.disable_recovery = true;
 	return SD_RES_SUCCESS;
 }
 
@@ -351,17 +330,22 @@ static int cluster_get_vdi_attr(struct request *req)
 static int local_release_vdi(struct request *req)
 {
 	uint32_t vid = req->rq.vdi.base_vdi_id;
+	int ret;
+
+	if (!sys->enable_object_cache)
+		return SD_RES_SUCCESS;
 
 	if (!vid) {
-		sd_iprintf("Some VDI failed to release the object cache. "
-			   "Probably you are running old QEMU.");
+		sd_info("Some VDI failed to release the object cache. "
+			"Probably you are running old QEMU.");
 		return SD_RES_SUCCESS;
 	}
 
-	object_cache_flush_vdi(vid);
-	object_cache_delete(vid);
+	ret = object_cache_flush_vdi(vid);
+	if (ret == SD_RES_SUCCESS)
+		object_cache_delete(vid);
 
-	return SD_RES_SUCCESS;
+	return ret;
 }
 
 static int local_get_store_list(struct request *req)
@@ -403,8 +387,8 @@ static int local_stat_sheep(struct request *req)
 static int local_stat_recovery(const struct sd_req *req, struct sd_rsp *rsp,
 					void *data)
 {
-	if (node_in_recovery())
-		return SD_RES_NODE_IN_RECOVERY;
+	get_recovery_state(data);
+	rsp->data_length = sizeof(struct recovery_state);
 
 	return SD_RES_SUCCESS;
 }
@@ -412,56 +396,55 @@ static int local_stat_recovery(const struct sd_req *req, struct sd_rsp *rsp,
 static int local_stat_cluster(struct request *req)
 {
 	struct sd_rsp *rsp = &req->rp;
-	struct epoch_log *log;
-	int i, max_logs;
+	struct epoch_log *elog;
+	int i, max_elogs;
 	uint32_t epoch;
 
 	if (req->vinfo == NULL) {
-		sd_dprintf("cluster is not started up");
+		sd_debug("cluster is not started up");
 		goto out;
 	}
 
-	max_logs = req->rq.data_length / sizeof(*log);
+	max_elogs = req->rq.data_length / sizeof(*elog);
 	epoch = get_latest_epoch();
-	for (i = 0; i < max_logs; i++) {
+	for (i = 0; i < max_elogs; i++) {
 		size_t nr_nodes;
 
 		if (epoch <= 0)
 			break;
 
-		log = (struct epoch_log *)req->data + i;
-		memset(log, 0, sizeof(*log));
-		log->epoch = epoch;
-		log->ctime = get_cluster_ctime();
-		nr_nodes = epoch_log_read_with_timestamp(epoch, log->nodes,
-							 sizeof(log->nodes),
-							 (time_t *)&log->time);
+		elog = (struct epoch_log *)req->data + i;
+		memset(elog, 0, sizeof(*elog));
+		elog->epoch = epoch;
+		elog->ctime = sys->cinfo.ctime;
+		nr_nodes = epoch_log_read_with_timestamp(epoch, elog->nodes,
+							 sizeof(elog->nodes),
+							 (time_t *)&elog->time);
 		if (nr_nodes == -1)
-			nr_nodes = epoch_log_read_remote(epoch, log->nodes,
-							 sizeof(log->nodes),
-							 (time_t *)&log->time,
+			nr_nodes = epoch_log_read_remote(epoch, elog->nodes,
+							 sizeof(elog->nodes),
+							 (time_t *)&elog->time,
 							 req->vinfo);
 		assert(nr_nodes >= 0);
 		assert(nr_nodes <= SD_MAX_NODES);
-		log->nr_nodes = nr_nodes;
+		elog->nr_nodes = nr_nodes;
 
-		log->disable_recovery = sys->disable_recovery;
+		elog->disable_recovery = sys->cinfo.disable_recovery;
 
-		rsp->data_length += sizeof(*log);
+		rsp->data_length += sizeof(*elog);
 		epoch--;
 	}
 out:
-	switch (sys->status) {
+	switch (sys->cinfo.status) {
 	case SD_STATUS_OK:
 		return SD_RES_SUCCESS;
-	case SD_STATUS_WAIT_FOR_FORMAT:
-		return SD_RES_WAIT_FOR_FORMAT;
-	case SD_STATUS_WAIT_FOR_JOIN:
-		return SD_RES_WAIT_FOR_JOIN;
+	case SD_STATUS_WAIT:
+		if (sys->cinfo.ctime == 0)
+			return SD_RES_WAIT_FOR_FORMAT;
+		else
+			return SD_RES_WAIT_FOR_JOIN;
 	case SD_STATUS_SHUTDOWN:
 		return SD_RES_SHUTDOWN;
-	case SD_STATUS_HALT:
-		return SD_RES_HALT;
 	default:
 		return SD_RES_SYSTEM_ERROR;
 	}
@@ -478,7 +461,7 @@ static int local_get_epoch(struct request *req)
 	int nr_nodes, nodes_len;
 	time_t timestamp;
 
-	sd_dprintf("%d", epoch);
+	sd_debug("%d", epoch);
 
 	nr_nodes =
 		epoch_log_read_with_timestamp(epoch, req->data,
@@ -505,20 +488,19 @@ static int cluster_force_recover_work(struct request *req)
 	 * 2) some nodes are physically down (same epoch condition).
 	 * In both case, the nodes(s) stat is WAIT_FOR_JOIN.
 	 */
-	if (sys->status != SD_STATUS_WAIT_FOR_JOIN || req->vinfo == NULL)
+	if (sys->cinfo.status != SD_STATUS_WAIT || req->vinfo == NULL)
 		return SD_RES_FORCE_RECOVER;
 
 	old_vnode_info = get_vnode_info_epoch(epoch, req->vinfo);
 	if (!old_vnode_info) {
-		sd_printf(SDOG_EMERG, "cannot get vnode info for epoch %d",
-			  epoch);
+		sd_emerg("cannot get vnode info for epoch %d", epoch);
 		put_vnode_info(old_vnode_info);
 		return SD_RES_FORCE_RECOVER;
 	}
 
 	if (req->rq.data_length <
 	    sizeof(*old_vnode_info->nodes) * old_vnode_info->nr_nodes) {
-		sd_eprintf("too small buffer size, %d", req->rq.data_length);
+		sd_err("too small buffer size, %d", req->rq.data_length);
 		return SD_RES_INVALID_PARMS;
 	}
 
@@ -538,41 +520,25 @@ static int cluster_force_recover_main(const struct sd_req *req,
 {
 	struct vnode_info *old_vnode_info, *vnode_info;
 	int ret = SD_RES_SUCCESS;
-	uint8_t c;
-	uint16_t f;
 	struct sd_node *nodes = data;
 	size_t nr_nodes = rsp->data_length / sizeof(*nodes);
 
-	if (rsp->epoch != sys->epoch) {
-		sd_eprintf("epoch was incremented while cluster_force_recover");
+	if (rsp->epoch != sys->cinfo.epoch) {
+		sd_err("epoch was incremented while cluster_force_recover");
 		return SD_RES_FORCE_RECOVER;
 	}
 
-	ret = get_cluster_copies(&c);
+	ret = inc_and_log_epoch();
 	if (ret) {
-		sd_printf(SDOG_EMERG, "cannot get cluster copies");
-		goto err;
-	}
-	ret = get_cluster_flags(&f);
-	if (ret) {
-		sd_printf(SDOG_EMERG, "cannot get cluster flags");
+		sd_emerg("cannot update epoch log");
 		goto err;
 	}
 
-	sys->nr_copies = c;
-	sys->flags = f;
+	if (!is_cluster_formatted())
+		/* initialize config file */
+		set_cluster_config(&sys->cinfo);
 
-	sys->epoch++; /* some nodes are left, so we get a new epoch */
-	ret = log_current_epoch();
-	if (ret) {
-		sd_printf(SDOG_EMERG, "cannot update epoch log");
-		goto err;
-	}
-
-	if (have_enough_zones())
-		sys->status = SD_STATUS_OK;
-	else
-		sys->status = SD_STATUS_HALT;
+	sys->cinfo.status = SD_STATUS_OK;
 
 	vnode_info = get_vnode_info();
 	old_vnode_info = alloc_vnode_info(nodes, nr_nodes);
@@ -613,7 +579,7 @@ static int cluster_notify_vdi_add(const struct sd_req *req, struct sd_rsp *rsp,
 			      true);
 
 	if (req->vdi_state.set_bitmap)
-		set_bit(req->vdi_state.new_vid, sys->vdi_inuse);
+		atomic_set_bit(req->vdi_state.new_vid, sys->vdi_inuse);
 
 	add_vdi_state(req->vdi_state.new_vid, req->vdi_state.copies, false);
 
@@ -656,7 +622,7 @@ static int cluster_recovery_completion(const struct sd_req *req,
 		return SD_RES_SUCCESS;
 
 	if (latest_epoch < epoch) {
-		sd_dprintf("new epoch %d", epoch);
+		sd_debug("new epoch %d", epoch);
 		latest_epoch = epoch;
 		nr_recovereds = 0;
 	}
@@ -664,11 +630,11 @@ static int cluster_recovery_completion(const struct sd_req *req,
 	recovereds[nr_recovereds++] = *node;
 	xqsort(recovereds, nr_recovereds, node_cmp);
 
-	sd_dprintf("%s is recovered at epoch %d", node_to_str(node), epoch);
+	sd_debug("%s is recovered at epoch %d", node_to_str(node), epoch);
 	for (i = 0; i < nr_recovereds; i++)
-		sd_dprintf("[%x] %s", i, node_to_str(recovereds + i));
+		sd_debug("[%x] %s", i, node_to_str(recovereds + i));
 
-	if (sys->epoch != latest_epoch)
+	if (sys->cinfo.epoch != latest_epoch)
 		return SD_RES_SUCCESS;
 
 	vnode_info = get_vnode_info();
@@ -679,8 +645,9 @@ static int cluster_recovery_completion(const struct sd_req *req,
 				break;
 		}
 		if (i == nr_recovereds) {
-			sd_dprintf("all nodes are recovered, epoch %d", epoch);
-			if (sd_store->cleanup)
+			sd_debug("all nodes are recovered, epoch %d", epoch);
+			/* sd_store can be NULL if this node is a gateway */
+			if (sd_store && sd_store->cleanup)
 				sd_store->cleanup();
 		}
 	}
@@ -690,35 +657,6 @@ static int cluster_recovery_completion(const struct sd_req *req,
 	return SD_RES_SUCCESS;
 }
 
-static void do_reweight(struct work *work)
-{
-	struct sd_req hdr;
-	int ret;
-
-	sd_init_req(&hdr, SD_OP_UPDATE_SIZE);
-	hdr.flags = SD_FLAG_CMD_WRITE;
-	hdr.data_length = sizeof(sys->this_node);
-
-	ret = exec_local_req(&hdr, &sys->this_node);
-	if (ret != SD_RES_SUCCESS)
-		sd_eprintf("failed to update node size");
-}
-
-static void reweight_done(struct work *work)
-{
-	free(work);
-}
-
-static void reweight_node(void)
-{
-	struct work *rw = xzalloc(sizeof(*rw));
-
-	rw->fn = do_reweight;
-	rw->done = reweight_done;
-
-	queue_work(sys->recovery_wqueue, rw);
-}
-
 static bool node_size_varied(void)
 {
 	uint64_t new, used, old = sys->this_node.space;
@@ -737,8 +675,8 @@ static bool node_size_varied(void)
 	}
 
 	diff = new > old ? (double)(new - old) : (double)(old - new);
-	sd_dprintf("new %"PRIu64 ", old %"PRIu64", ratio %f", new, old,
-		   diff / (double)old);
+	sd_debug("new %"PRIu64 ", old %"PRIu64", ratio %f", new, old,
+		 diff / (double)old);
 	if (diff / (double)old < 0.01)
 		return false;
 
@@ -752,18 +690,7 @@ static int cluster_reweight(const struct sd_req *req, struct sd_rsp *rsp,
 			    void *data)
 {
 	if (node_size_varied())
-		reweight_node();
-
-	return SD_RES_SUCCESS;
-}
-
-static int cluster_update_size(const struct sd_req *req, struct sd_rsp *rsp,
-			       void *data)
-{
-	struct sd_node *node = (struct sd_node *)data;
-
-	update_node_size(node);
-	kick_node_recover();
+		return sys->cdrv->update_node(&sys->this_node);
 
 	return SD_RES_SUCCESS;
 }
@@ -806,6 +733,17 @@ static int local_get_hash(struct request *request)
 				  rsp->hash.digest);
 }
 
+static int local_get_cache_info(struct request *request)
+{
+	struct sd_rsp *rsp = &request->rp;
+
+	assert(request->rq.data_length == sizeof(struct object_cache_info));
+	rsp->data_length = object_cache_get_info((struct object_cache_info *)
+						 request->data);
+
+	return SD_RES_SUCCESS;
+}
+
 /* Return SD_RES_INVALID_PARMS to ask client not to send flush req again */
 static int local_flush_vdi(struct request *req)
 {
@@ -814,8 +752,6 @@ static int local_flush_vdi(struct request *req)
 	if (sys->enable_object_cache) {
 		uint32_t vid = oid_to_vid(req->rq.obj.oid);
 		ret = object_cache_flush_vdi(vid);
-		if (ret != SD_RES_SUCCESS)
-			return ret;
 	}
 
 	return ret;
@@ -827,13 +763,13 @@ static int local_discard_obj(struct request *req)
 	uint32_t vid = oid_to_vid(oid), zero = 0;
 	int ret, idx = data_oid_to_idx(oid);
 
-	sd_dprintf("%"PRIx64, oid);
+	sd_debug("%"PRIx64, oid);
 	ret = write_object(vid_to_vdi_oid(vid), (char *)&zero, sizeof(zero),
 			   SD_INODE_HEADER_SIZE + sizeof(vid) * idx, false);
 	if (ret != SD_RES_SUCCESS)
 		return ret;
 	if (remove_object(oid) != SD_RES_SUCCESS)
-		sd_eprintf("failed to remove %"PRIx64, oid);
+		sd_err("failed to remove %"PRIx64, oid);
 	/*
 	 * Return success even if remove_object fails because we have updated
 	 * inode successfully.
@@ -848,16 +784,24 @@ static int local_flush_and_del(struct request *req)
 	return object_cache_flush_and_del(req);
 }
 
-static int local_trace_ops(const struct sd_req *req, struct sd_rsp *rsp, void *data)
+static int local_trace_enable(const struct sd_req *req, struct sd_rsp *rsp,
+			      void *data)
 {
-	int enable = req->data_length, ret;
+	return trace_enable(data);
+}
 
-	if (enable)
-		ret = trace_enable();
-	else
-		ret = trace_disable();
+static int local_trace_disable(const struct sd_req *req, struct sd_rsp *rsp,
+			       void *data)
+{
+	return trace_disable(data);
+}
 
-	return ret;
+static int local_trace_status(const struct sd_req *req, struct sd_rsp *rsp,
+			      void *data)
+{
+	rsp->data_length = trace_status(data);
+
+	return SD_RES_SUCCESS;
 }
 
 static int local_trace_read_buf(struct request *request)
@@ -871,14 +815,14 @@ static int local_trace_read_buf(struct request *request)
 		return SD_RES_AGAIN;
 
 	rsp->data_length = ret;
-	sd_dprintf("%u", rsp->data_length);
+	sd_debug("%u", rsp->data_length);
 	return SD_RES_SUCCESS;
 }
 
 static int local_kill_node(const struct sd_req *req, struct sd_rsp *rsp,
 			   void *data)
 {
-	sys->status = SD_STATUS_KILLED;
+	sys->cinfo.status = SD_STATUS_KILLED;
 
 	return SD_RES_SUCCESS;
 }
@@ -907,8 +851,8 @@ static int read_copy_from_replica(struct request *req, uint32_t epoch,
 	ret = gateway_read_obj(&read_req);
 
 	if (ret == SD_RES_SUCCESS)
-		untrim_zero_sectors(buf, rsp->obj.offset, rsp->data_length,
-				    SD_DATA_OBJ_SIZE);
+		untrim_zero_blocks(buf, rsp->obj.offset, rsp->data_length,
+				   SD_DATA_OBJ_SIZE);
 
 	return ret;
 }
@@ -944,7 +888,7 @@ int peer_read_obj(struct request *req)
 
 	rsp->data_length = hdr->data_length;
 	rsp->obj.offset = 0;
-	trim_zero_sectors(req->data, &rsp->obj.offset, &rsp->data_length);
+	trim_zero_blocks(req->data, &rsp->obj.offset, &rsp->data_length);
 
 	if (hdr->obj.copies)
 		rsp->obj.copies = hdr->obj.copies;
@@ -993,14 +937,14 @@ int peer_create_and_write_obj(struct request *req)
 	iocb.epoch = epoch;
 	iocb.length = get_objsize(oid);
 	if (hdr->flags & SD_FLAG_CMD_COW) {
-		sd_dprintf("%" PRIx64 ", %" PRIx64, oid, hdr->obj.cow_oid);
+		sd_debug("%" PRIx64 ", %" PRIx64, oid, hdr->obj.cow_oid);
 
 		buf = xvalloc(SD_DATA_OBJ_SIZE);
 		if (hdr->data_length != SD_DATA_OBJ_SIZE) {
 			ret = read_copy_from_replica(req, hdr->epoch,
 						     hdr->obj.cow_oid, buf);
 			if (ret != SD_RES_SUCCESS) {
-				sd_eprintf("failed to read cow object");
+				sd_err("failed to read cow object");
 				goto out;
 			}
 		}
@@ -1009,8 +953,8 @@ int peer_create_and_write_obj(struct request *req)
 		memcpy(&cow_hdr, hdr, sizeof(cow_hdr));
 		cow_hdr.data_length = SD_DATA_OBJ_SIZE;
 		cow_hdr.obj.offset = 0;
-		trim_zero_sectors(buf, &cow_hdr.obj.offset,
-				  &cow_hdr.data_length);
+		trim_zero_blocks(buf, &cow_hdr.obj.offset,
+				 &cow_hdr.data_length);
 
 		ret = do_create_and_write_obj(&iocb, &cow_hdr, epoch, buf);
 	} else
@@ -1121,12 +1065,6 @@ static struct sd_op_template sd_ops[] = {
 		.process_main = cluster_reweight,
 	},
 
-	[SD_OP_UPDATE_SIZE] = {
-		.name = "UPDATE_SIZE",
-		.type = SD_OP_TYPE_CLUSTER,
-		.process_main = cluster_update_size,
-	},
-
 	[SD_OP_ENABLE_RECOVER] = {
 		.name = "ENABLE_RECOVER",
 		.type = SD_OP_TYPE_CLUSTER,
@@ -1223,11 +1161,25 @@ static struct sd_op_template sd_ops[] = {
 		.process_work = local_flush_and_del,
 	},
 
-	[SD_OP_TRACE] = {
-		.name = "TRACE",
+	[SD_OP_TRACE_ENABLE] = {
+		.name = "TRACE_ENABLE",
 		.type = SD_OP_TYPE_LOCAL,
 		.force = true,
-		.process_main = local_trace_ops,
+		.process_main = local_trace_enable,
+	},
+
+	[SD_OP_TRACE_DISABLE] = {
+		.name = "TRACE_DISABLE",
+		.type = SD_OP_TYPE_LOCAL,
+		.force = true,
+		.process_main = local_trace_disable,
+	},
+
+	[SD_OP_TRACE_STATUS] = {
+		.name = "TRACE_STATUS",
+		.type = SD_OP_TYPE_LOCAL,
+		.force = true,
+		.process_main = local_trace_status,
 	},
 
 	[SD_OP_TRACE_READ_BUF] = {
@@ -1268,6 +1220,12 @@ static struct sd_op_template sd_ops[] = {
 		.process_work = local_get_hash,
 	},
 
+	[SD_OP_GET_CACHE_INFO] = {
+		.name = "GET_CACHE_INFO",
+		.type = SD_OP_TYPE_LOCAL,
+		.process_work = local_get_cache_info,
+	},
+
 	/* gateway I/O operations */
 	[SD_OP_CREATE_AND_WRITE_OBJ] = {
 		.name = "CREATE_AND_WRITE_OBJ",
@@ -1372,16 +1330,15 @@ void do_process_work(struct work *work)
 	struct request *req = container_of(work, struct request, work);
 	int ret = SD_RES_SUCCESS;
 
-	sd_dprintf("%x, %" PRIx64", %"PRIu32, req->rq.opcode, req->rq.obj.oid,
-		   req->rq.epoch);
+	sd_debug("%x, %" PRIx64", %"PRIu32, req->rq.opcode, req->rq.obj.oid,
+		 req->rq.epoch);
 
 	if (req->op->process_work)
 		ret = req->op->process_work(req);
 
 	if (ret != SD_RES_SUCCESS) {
-		sd_dprintf("failed: %x, %" PRIx64" , %u, %s",
-			   req->rq.opcode, req->rq.obj.oid, req->rq.epoch,
-			   sd_strerror(ret));
+		sd_debug("failed: %x, %" PRIx64" , %u, %s", req->rq.opcode,
+			 req->rq.obj.oid, req->rq.epoch, sd_strerror(ret));
 	}
 
 	req->rp.result = ret;
