@@ -8,21 +8,9 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/types.h>
 
 #include "sheep_priv.h"
 
-
-enum rw_state {
-	RW_PREPARE_LIST, /* the recovery thread is preparing object list */
-	RW_RECOVER_OBJ, /* the thread is recoering objects */
-	RW_NOTIFY_COMPLETION, /* the thread is notifying recovery completion */
-};
-
 /* base structure for the recovery thread */
 struct recovery_work {
 	uint32_t epoch;
@@ -38,7 +26,7 @@ struct recovery_work {
 struct recovery_list_work {
 	struct recovery_work base;
 
-	int count;
+	uint64_t count;
 	uint64_t *oids;
 };
 
@@ -51,7 +39,7 @@ struct recovery_obj_work {
 
 	/* local replica in the stale directory */
 	uint32_t local_epoch;
-	uint8_t local_sha1[SHA1_LEN];
+	uint8_t local_sha1[SHA1_DIGEST_SIZE];
 };
 
 /*
@@ -64,7 +52,7 @@ struct recovery_info {
 
 	uint32_t epoch;
 	uint32_t tgt_epoch;
-	uint32_t done;
+	uint64_t done;
 
 	/*
 	 * true when automatic recovery is disabled
@@ -73,17 +61,17 @@ struct recovery_info {
 	bool suspended;
 	bool notify_complete;
 
-	int count;
+	uint64_t count;
 	uint64_t *oids;
 	uint64_t *prio_oids;
-	int nr_prio_oids;
-	int nr_scheduled_prio_oids;
+	uint64_t nr_prio_oids;
+	uint64_t nr_scheduled_prio_oids;
 
 	struct vnode_info *old_vinfo;
 	struct vnode_info *cur_vinfo;
 };
 
-struct recovery_info *next_rinfo;
+static struct recovery_info *next_rinfo;
 static main_thread(struct recovery_info *) current_rinfo;
 
 static void queue_recovery_work(struct recovery_info *rinfo);
@@ -97,11 +85,7 @@ static int obj_cmp(const uint64_t *oid1, const uint64_t *oid2)
 	const uint64_t hval1 = fnv_64a_buf(oid1, sizeof(*oid1), FNV1A_64_INIT);
 	const uint64_t hval2 = fnv_64a_buf(oid2, sizeof(*oid2), FNV1A_64_INIT);
 
-	if (hval1 < hval2)
-		return -1;
-	if (hval1 > hval2)
-		return 1;
-	return 0;
+	return intcmp(hval1, hval2);
 }
 
 static inline bool node_is_gateway_only(void)
@@ -111,7 +95,7 @@ static inline bool node_is_gateway_only(void)
 
 /* recover object from vnode */
 static int recover_object_from(struct recovery_obj_work *row,
-			       const struct sd_vnode *vnode, uint32_t tgt_epoch)
+			       const struct sd_node *node, uint32_t tgt_epoch)
 {
 	uint64_t oid = row->oid;
 	uint32_t local_epoch = row->local_epoch;
@@ -124,7 +108,7 @@ static int recover_object_from(struct recovery_obj_work *row,
 	struct sd_rsp *rsp = (struct sd_rsp *)&hdr;
 	struct siocb iocb = { 0 };
 
-	if (vnode_is_local(vnode)) {
+	if (node_is_local(node)) {
 		if (tgt_epoch < sys_epoch())
 			return sd_store->link(oid, tgt_epoch);
 
@@ -136,13 +120,13 @@ static int recover_object_from(struct recovery_obj_work *row,
 		sd_init_req(&hdr, SD_OP_GET_HASH);
 		hdr.obj.oid = oid;
 		hdr.obj.tgt_epoch = tgt_epoch;
-		ret = sheep_exec_req(&vnode->nid, &hdr, NULL);
+		ret = sheep_exec_req(&node->nid, &hdr, NULL);
 		if (ret != SD_RES_SUCCESS)
 			return ret;
 
-		if (memcmp(rsp->hash.digest, sha1, sizeof(SHA1_LEN)) == 0) {
-			sd_dprintf("use local replica at epoch %d",
-				   local_epoch);
+		if (memcmp(rsp->hash.digest, sha1,
+			   sizeof(SHA1_DIGEST_SIZE)) == 0) {
+			sd_debug("use local replica at epoch %d", local_epoch);
 			ret = sd_store->link(oid, local_epoch);
 			if (ret == SD_RES_SUCCESS)
 				return ret;
@@ -160,7 +144,7 @@ static int recover_object_from(struct recovery_obj_work *row,
 	hdr.obj.oid = oid;
 	hdr.obj.tgt_epoch = tgt_epoch;
 
-	ret = sheep_exec_req(&vnode->nid, &hdr, buf);
+	ret = sheep_exec_req(&node->nid, &hdr, buf);
 	if (ret == SD_RES_SUCCESS) {
 		iocb.epoch = epoch;
 		iocb.length = rsp->data_length;
@@ -173,6 +157,18 @@ static int recover_object_from(struct recovery_obj_work *row,
 	return ret;
 }
 
+/*
+ * A node that does not match any node in current node list means the node has
+ * left the cluster, then it's an invalid node.
+ */
+static bool invalid_node(const struct sd_node *n, struct vnode_info *info)
+{
+
+	if (xbsearch(n, info->nodes, info->nr_nodes, node_cmp))
+		return false;
+	return true;
+}
+
 static int recover_object_from_replica(struct recovery_obj_work *row,
 				       struct vnode_info *old,
 				       uint32_t tgt_epoch)
@@ -198,16 +194,20 @@ static int recover_object_from_replica(struct recovery_obj_work *row,
 
 	/* Let's do a breadth-first search */
 	for (int i = 0; i < nr_copies; i++) {
-		const struct sd_vnode *vnode;
+		const struct sd_node *node;
 		int idx = (i + start) % nr_copies;
 
-		vnode = oid_to_vnode(old->vnodes, old->nr_vnodes, oid, idx);
+		node = oid_to_node(old->vnodes, old->nr_vnodes, oid, idx,
+				   old->nodes);
 
-		ret = recover_object_from(row, vnode, tgt_epoch);
+		if (invalid_node(node, row->base.cur_vinfo))
+			continue;
+
+		ret = recover_object_from(row, node, tgt_epoch);
 		switch (ret) {
 		case SD_RES_SUCCESS:
-			sd_dprintf("recovered oid %"PRIx64" from %d "
-				   "to epoch %d", oid, tgt_epoch, epoch);
+			sd_debug("recovered oid %"PRIx64" from %d to epoch %d",
+				 oid, tgt_epoch, epoch);
 			objlist_cache_insert(oid);
 			return ret;
 		case SD_RES_OLD_NODE_VER:
@@ -249,8 +249,8 @@ static int do_recover_object(struct recovery_obj_work *row)
 
 	old = grab_vnode_info(rw->old_vinfo);
 again:
-	sd_dprintf("try recover object %"PRIx64" from epoch %"PRIu32, oid,
-		   tgt_epoch);
+	sd_debug("try recover object %"PRIx64" from epoch %"PRIu32, oid,
+		 tgt_epoch);
 
 	ret = recover_object_from_replica(row, old, tgt_epoch);
 
@@ -262,16 +262,16 @@ again:
 		row->stop = true;
 		break;
 	case SD_RES_STALE_OBJ:
-		sd_printf(SDOG_ALERT, "cannot access any replicas of "
-			  "%"PRIx64" at epoch %d", oid, tgt_epoch);
-		sd_printf(SDOG_ALERT, "clients may see old data");
+		sd_alert("cannot access any replicas of %"PRIx64" at epoch %d",
+			 oid, tgt_epoch);
+		sd_alert("clients may see old data");
 		/* fall through */
 	default:
 		/* No luck, roll back to an older configuration and try again */
 rollback:
 		tgt_epoch--;
 		if (tgt_epoch < 1) {
-			sd_eprintf("can not recover oid %"PRIx64, oid);
+			sd_err("can not recover oid %"PRIx64, oid);
 			ret = -1;
 			break;
 		}
@@ -279,8 +279,8 @@ rollback:
 		new_old = get_vnode_info_epoch(tgt_epoch, rw->cur_vinfo);
 		if (!new_old) {
 			/* We rollback in case we don't get a valid epoch */
-			sd_printf(SDOG_ALERT, "cannot get epoch %d", tgt_epoch);
-			sd_printf(SDOG_ALERT, "clients may see old data");
+			sd_alert("cannot get epoch %d", tgt_epoch);
+			sd_alert("clients may see old data");
 			goto rollback;
 		}
 
@@ -304,7 +304,7 @@ static void recover_object_work(struct work *work)
 	int ret, epoch;
 
 	if (sd_store->exist(oid)) {
-		sd_dprintf("the object is already recovered");
+		sd_debug("the object is already recovered");
 		return;
 	}
 
@@ -312,7 +312,7 @@ static void recover_object_work(struct work *work)
 	for (epoch = sys_epoch() - 1; epoch > 0; epoch--) {
 		ret = sd_store->get_hash(oid, epoch, row->local_sha1);
 		if (ret == SD_RES_SUCCESS) {
-			sd_dprintf("replica found in local at epoch %d", epoch);
+			sd_debug("replica found in local at epoch %d", epoch);
 			row->local_epoch = epoch;
 			break;
 		}
@@ -320,7 +320,7 @@ static void recover_object_work(struct work *work)
 
 	ret = do_recover_object(row);
 	if (ret < 0)
-		sd_eprintf("failed to recover object %"PRIx64, oid);
+		sd_err("failed to recover object %"PRIx64, oid);
 }
 
 bool node_in_recovery(void)
@@ -331,7 +331,7 @@ bool node_in_recovery(void)
 static inline void prepare_schedule_oid(uint64_t oid)
 {
 	struct recovery_info *rinfo = main_thread_get(current_rinfo);
-	int i;
+	uint64_t i;
 
 	for (i = 0; i < rinfo->nr_prio_oids; i++)
 		if (rinfo->prio_oids[i] == oid)
@@ -342,8 +342,8 @@ static inline void prepare_schedule_oid(uint64_t oid)
 	 */
 	for (i = 0; i < rinfo->done; i++)
 		if (rinfo->oids[i] == oid) {
-			sd_dprintf("%"PRIx64" not recovered, don't schedule it",
-				   oid);
+			sd_debug("%"PRIx64" not recovered, don't schedule it",
+				 oid);
 			return;
 		}
 	/* When recovery is not suspended, oid is currently being recovered */
@@ -354,7 +354,7 @@ static inline void prepare_schedule_oid(uint64_t oid)
 	rinfo->prio_oids = xrealloc(rinfo->prio_oids,
 				    rinfo->nr_prio_oids * sizeof(uint64_t));
 	rinfo->prio_oids[rinfo->nr_prio_oids - 1] = oid;
-	sd_dprintf("%"PRIx64" nr_prio_oids %d", oid, rinfo->nr_prio_oids);
+	sd_debug("%"PRIx64" nr_prio_oids %"PRIu64, oid, rinfo->nr_prio_oids);
 
 	resume_suspended_recovery();
 }
@@ -362,13 +362,13 @@ static inline void prepare_schedule_oid(uint64_t oid)
 bool oid_in_recovery(uint64_t oid)
 {
 	struct recovery_info *rinfo = main_thread_get(current_rinfo);
-	int i;
+	uint64_t i;
 
 	if (!node_in_recovery())
 		return false;
 
 	if (sd_store->exist(oid)) {
-		sd_dprintf("the object %" PRIx64 " is already recoverd", oid);
+		sd_debug("the object %" PRIx64 " is already recoverd", oid);
 		return false;
 	}
 
@@ -393,7 +393,7 @@ bool oid_in_recovery(uint64_t oid)
 	 * in the list
 	 */
 	if (i == rinfo->count) {
-		sd_eprintf("%"PRIx64" is not in the recovery list", oid);
+		sd_err("%"PRIx64" is not in the recovery list", oid);
 		return false;
 	}
 
@@ -456,7 +456,7 @@ static inline bool run_next_rw(void)
 	main_thread_set(current_rinfo, nrinfo);
 	wakeup_all_requests();
 	queue_recovery_work(nrinfo);
-	sd_dprintf("recovery work is superseded");
+	sd_debug("recovery work is superseded");
 	return true;
 }
 
@@ -474,8 +474,7 @@ static void notify_recovery_completion_work(struct work *work)
 
 	ret = exec_local_req(&hdr, &sys->this_node);
 	if (ret != SD_RES_SUCCESS)
-		sd_eprintf("failed to notify recovery completion, %d",
-			   rw->epoch);
+		sd_err("failed to notify recovery completion, %d", rw->epoch);
 }
 
 static void notify_recovery_completion_main(struct work *work)
@@ -499,14 +498,12 @@ static inline void finish_recovery(struct recovery_info *rinfo)
 
 	free_recovery_info(rinfo);
 
-	sd_dprintf("recovery complete: new epoch %"PRIu32, recovered_epoch);
+	sd_debug("recovery complete: new epoch %"PRIu32, recovered_epoch);
 }
 
 static inline bool oid_in_prio_oids(struct recovery_info *rinfo, uint64_t oid)
 {
-	int i;
-
-	for (i = 0; i < rinfo->nr_prio_oids; i++)
+	for (uint64_t i = 0; i < rinfo->nr_prio_oids; i++)
 		if (rinfo->prio_oids[i] == oid)
 			return true;
 	return false;
@@ -522,7 +519,7 @@ static inline bool oid_in_prio_oids(struct recovery_info *rinfo, uint64_t oid)
  */
 static inline void finish_schedule_oids(struct recovery_info *rinfo)
 {
-	int i, nr_recovered = rinfo->done, new_idx;
+	uint64_t i, nr_recovered = rinfo->done, new_idx;
 	uint64_t *new_oids;
 
 	/* If I am the last oid, done */
@@ -541,9 +538,10 @@ static inline void finish_schedule_oids(struct recovery_info *rinfo)
 		new_oids[new_idx++] = rinfo->oids[i];
 	}
 	/* rw->count should eq new_idx, otherwise something is wrong */
-	sd_dprintf("%snr_recovered %d, nr_prio_oids %d, count %d = new %d",
-		   rinfo->count == new_idx ? "" : "WARN: ", nr_recovered,
-		   rinfo->nr_prio_oids, rinfo->count, new_idx);
+	sd_debug("%snr_recovered %" PRIu64 ", nr_prio_oids %" PRIu64 ", count %"
+		 PRIu64 " = new %" PRIu64,
+		 rinfo->count == new_idx ? "" : "WARN: ", nr_recovered,
+		 rinfo->nr_prio_oids, rinfo->count, new_idx);
 
 	free(rinfo->oids);
 	rinfo->oids = new_oids;
@@ -574,8 +572,8 @@ static void recover_next_object(struct recovery_info *rinfo)
 	if (rinfo->nr_prio_oids)
 		finish_schedule_oids(rinfo);
 
-	if (sys->disable_recovery && !has_scheduled_objects(rinfo)) {
-		sd_dprintf("suspended");
+	if (sys->cinfo.disable_recovery && !has_scheduled_objects(rinfo)) {
+		sd_debug("suspended");
 		rinfo->suspended = true;
 		/* suspend until resume_suspended_recovery() is called */
 		return;
@@ -613,16 +611,17 @@ static void recover_object_main(struct work *work)
 		 * lifted and flush wait queue to requeue those
 		 * requests
 		 */
-		wakeup_all_requests();
-		sd_dprintf("recovery is stopped");
+		rinfo->notify_complete = false;
+		finish_recovery(rinfo);
+		sd_debug("recovery is stopped");
 		goto out;
 	}
 
 	wakeup_requests_on_oid(row->oid);
 	rinfo->done++;
 
-	sd_eprintf("done:%"PRIu32" count:%"PRIu32", oid:%"PRIx64, rinfo->done,
-		   rinfo->count, row->oid);
+	sd_info("object %"PRIx64" is recovered (%"PRIu64"/%"PRIu64")", row->oid,
+		rinfo->done, rinfo->count);
 
 	if (rinfo->done < rinfo->count) {
 		recover_next_object(rinfo);
@@ -665,15 +664,13 @@ static void finish_object_list(struct work *work)
 static uint64_t *fetch_object_list(struct sd_node *e, uint32_t epoch,
 				   size_t *nr_oids)
 {
-	char name[128];
 	struct sd_req hdr;
 	struct sd_rsp *rsp = (struct sd_rsp *)&hdr;
 	size_t buf_size = list_buffer_size;
 	uint64_t *buf = xmalloc(buf_size);
 	int ret;
 
-	addr_to_str(name, sizeof(name), e->nid.addr, 0);
-	sd_dprintf("%s %"PRIu32, name, e->nid.port);
+	sd_debug("%s", addr_to_str(e->nid.addr, e->nid.port));
 
 retry:
 	sd_init_req(&hdr, SD_OP_GET_OBJ_LIST);
@@ -689,16 +686,16 @@ retry:
 		buf = xrealloc(buf, buf_size);
 		goto retry;
 	default:
-		sd_printf(SDOG_ALERT, "cannot get object list from %s:%d", name,
-			  e->nid.port);
-		sd_printf(SDOG_ALERT, "some objects may be not recovered at "
-			  "epoch %d", epoch);
+		sd_alert("cannot get object list from %s",
+			 addr_to_str(e->nid.addr, e->nid.port));
+		sd_alert("some objects may be not recovered at epoch %d",
+			 epoch);
 		free(buf);
 		return NULL;
 	}
 
 	*nr_oids = rsp->data_length / sizeof(uint64_t);
-	sd_dprintf("%zu", *nr_oids);
+	sd_debug("%zu", *nr_oids);
 	return buf;
 }
 
@@ -708,9 +705,9 @@ static void screen_object_list(struct recovery_list_work *rlw,
 {
 	struct recovery_work *rw = &rlw->base;
 	const struct sd_vnode *vnodes[SD_MAX_COPIES];
-	int old_count = rlw->count;
-	int nr_objs;
-	int i, j;
+	uint64_t old_count = rlw->count;
+	uint64_t nr_objs;
+	uint64_t i, j;
 
 	for (i = 0; i < nr_oids; i++) {
 		if (xbsearch(&oids[i], rlw->oids, old_count, obj_cmp))
@@ -719,8 +716,8 @@ static void screen_object_list(struct recovery_list_work *rlw,
 
 		nr_objs = get_obj_copy_number(oids[i], rw->cur_vinfo->nr_zones);
 		if (!nr_objs) {
-			sd_eprintf("ERROR: can not find copy number for object"
-				   " %" PRIx64, oids[i]);
+			sd_err("ERROR: can not find copy number for object %"
+			       PRIx64, oids[i]);
 			continue;
 		}
 		oid_to_vnodes(rw->cur_vinfo->vnodes, rw->cur_vinfo->nr_vnodes,
@@ -759,7 +756,7 @@ static void prepare_object_list(struct work *work)
 	if (node_is_gateway_only())
 		return;
 
-	sd_dprintf("%u", rw->epoch);
+	sd_debug("%u", rw->epoch);
 	wait_get_vdis_done();
 again:
 	/* We need to start at random node for better load balance */
@@ -768,7 +765,7 @@ again:
 		struct sd_node *node = cur + i;
 
 		if (uatomic_read(&next_rinfo)) {
-			sd_dprintf("go to the next recovery");
+			sd_debug("go to the next recovery");
 			return;
 		}
 
@@ -785,7 +782,7 @@ again:
 		goto again;
 	}
 
-	sd_dprintf("%d", rlw->count);
+	sd_debug("%"PRIu64, rlw->count);
 }
 
 int start_recovery(struct vnode_info *cur_vinfo, struct vnode_info *old_vinfo,
@@ -795,8 +792,9 @@ int start_recovery(struct vnode_info *cur_vinfo, struct vnode_info *old_vinfo,
 
 	rinfo = xzalloc(sizeof(struct recovery_info));
 	rinfo->state = RW_PREPARE_LIST;
-	rinfo->epoch = sys->epoch;
-	rinfo->tgt_epoch = epoch_lifted ? sys->epoch - 1 : sys->epoch;
+	rinfo->epoch = sys->cinfo.epoch;
+	rinfo->tgt_epoch = epoch_lifted ? sys->cinfo.epoch - 1 :
+		sys->cinfo.epoch;
 	rinfo->count = 0;
 	if (epoch_lifted)
 		rinfo->notify_complete = true; /* Reweight or node recovery */
@@ -815,7 +813,7 @@ int start_recovery(struct vnode_info *cur_vinfo, struct vnode_info *old_vinfo,
 		nrinfo = uatomic_xchg_ptr(&next_rinfo, rinfo);
 		if (nrinfo)
 			free_recovery_info(nrinfo);
-		sd_dprintf("recovery skipped");
+		sd_debug("recovery skipped");
 
 		/*
 		 * This is necesary to invoke run_next_rw when
@@ -870,3 +868,20 @@ static void queue_recovery_work(struct recovery_info *rinfo)
 
 	queue_work(sys->recovery_wqueue, &rw->work);
 }
+
+void get_recovery_state(struct recovery_state *state)
+{
+	struct recovery_info *rinfo = main_thread_get(current_rinfo);
+
+	memset(state, 0, sizeof(*state));
+
+	if (!rinfo) {
+		state->in_recovery = 0;
+		return;
+	}
+
+	state->in_recovery = 1;
+	state->state = rinfo->state;
+	state->nr_finished = rinfo->done;
+	state->nr_total = rinfo->count;
+}
