@@ -8,20 +8,10 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <netdb.h>
-#include <pthread.h>
-#include <sys/eventfd.h>
-#include <arpa/inet.h>
+
 #include <netinet/tcp.h>
-#include <sys/epoll.h>
-#include <fcntl.h>
-#include <errno.h>
 
 #include "sheep_priv.h"
-#include "util.h"
 
 static void requeue_request(struct request *req);
 
@@ -57,14 +47,14 @@ static void io_op_done(struct work *work)
 	case SD_RES_EIO:
 		req->rp.result = SD_RES_NETWORK_ERROR;
 
-		sd_eprintf("leaving sheepdog cluster");
+		sd_err("leaving sheepdog cluster");
 		leave_cluster();
 		break;
 	case SD_RES_SUCCESS:
 	case SD_RES_NETWORK_ERROR:
 		break;
 	default:
-		sd_dprintf("unhandled error %s", sd_strerror(req->rp.result));
+		sd_debug("unhandled error %s", sd_strerror(req->rp.result));
 		break;
 	}
 
@@ -106,7 +96,7 @@ static void gateway_op_done(struct work *work)
 
 	switch (req->rp.result) {
 	case SD_RES_OLD_NODE_VER:
-		if (req->rp.epoch > sys->epoch) {
+		if (req->rp.epoch > sys->cinfo.epoch) {
 			/*
 			 * Gateway of this node is expected to process this
 			 * request later when epoch is lifted.
@@ -120,13 +110,13 @@ static void gateway_op_done(struct work *work)
 	case SD_RES_WAIT_FOR_JOIN:
 	case SD_RES_WAIT_FOR_FORMAT:
 	case SD_RES_KILLED:
-		sd_dprintf("retrying failed I/O request op %s result %x epoch %"
-			   PRIu32", sys epoch %"PRIu32, op_name(req->op),
-			   req->rp.result, req->rq.epoch, sys->epoch);
+		sd_debug("retrying failed I/O request op %s result %x epoch %"
+			 PRIu32 ", sys epoch %" PRIu32, op_name(req->op),
+			 req->rp.result, req->rq.epoch, sys->cinfo.epoch);
 		goto retry;
 	case SD_RES_EIO:
 		if (is_access_local(req, hdr->obj.oid)) {
-			sd_eprintf("leaving sheepdog cluster");
+			sd_err("leaving sheepdog cluster");
 			leave_cluster();
 			goto retry;
 		}
@@ -134,7 +124,7 @@ static void gateway_op_done(struct work *work)
 	case SD_RES_SUCCESS:
 		break;
 	default:
-		sd_dprintf("unhandled error %s", sd_strerror(req->rp.result));
+		sd_debug("unhandled error %s", sd_strerror(req->rp.result));
 		break;
 	}
 
@@ -158,17 +148,17 @@ static void local_op_done(struct work *work)
 
 static int check_request_epoch(struct request *req)
 {
-	if (before(req->rq.epoch, sys->epoch)) {
-		sd_eprintf("old node version %u, %u (%s)",
-			   sys->epoch, req->rq.epoch, op_name(req->op));
+	if (before(req->rq.epoch, sys->cinfo.epoch)) {
+		sd_err("old node version %u, %u (%s)", sys->cinfo.epoch,
+		       req->rq.epoch, op_name(req->op));
 		/* Ask for sleeping req on requester's wait queue */
 		req->rp.result = SD_RES_OLD_NODE_VER;
-		req->rp.epoch = sys->epoch;
+		req->rp.epoch = sys->cinfo.epoch;
 		put_request(req);
 		return -1;
-	} else if (after(req->rq.epoch, sys->epoch)) {
-		sd_eprintf("new node version %u, %u (%s)",
-			   sys->epoch, req->rq.epoch, op_name(req->op));
+	} else if (after(req->rq.epoch, sys->cinfo.epoch)) {
+		sd_err("new node version %u, %u (%s)", sys->cinfo.epoch,
+		       req->rq.epoch, op_name(req->op));
 		/* Wait for local epoch to be lifted */
 		req->rp.result = SD_RES_NEW_NODE_VER;
 		sleep_on_wait_queue(req);
@@ -197,7 +187,7 @@ static bool request_in_recovery(struct request *req)
 	 */
 	if (oid_in_recovery(req->local_oid) &&
 	    !(req->rq.flags & SD_FLAG_CMD_RECOVERY)) {
-		sd_dprintf("%"PRIx64" wait on oid", req->local_oid);
+		sd_debug("%"PRIx64" wait on oid", req->local_oid);
 		sleep_on_wait_queue(req);
 		return true;
 	}
@@ -220,8 +210,8 @@ void wakeup_requests_on_epoch(void)
 			 * its epoch changes.
 			 */
 			assert(is_gateway_op(req->op));
-			sd_dprintf("gateway %"PRIx64, req->rq.obj.oid);
-			req->rq.epoch = sys->epoch;
+			sd_debug("gateway %"PRIx64, req->rq.obj.oid);
+			req->rq.epoch = sys->cinfo.epoch;
 			del_requeue_request(req);
 			break;
 		case SD_RES_NEW_NODE_VER:
@@ -230,7 +220,7 @@ void wakeup_requests_on_epoch(void)
 			 * changes.
 			 */
 			assert(!is_gateway_op(req->op));
-			sd_dprintf("peer %"PRIx64, req->rq.obj.oid);
+			sd_debug("peer %"PRIx64, req->rq.obj.oid);
 			del_requeue_request(req);
 			break;
 		default:
@@ -252,7 +242,7 @@ void wakeup_requests_on_oid(uint64_t oid)
 	list_for_each_entry_safe(req, t, &pending_list, request_list) {
 		if (req->local_oid != oid)
 			continue;
-		sd_dprintf("retry %" PRIx64, req->local_oid);
+		sd_debug("retry %" PRIx64, req->local_oid);
 		del_requeue_request(req);
 	}
 	list_splice_init(&pending_list, &sys->req_wait_queue);
@@ -266,7 +256,7 @@ void wakeup_all_requests(void)
 	list_splice_init(&sys->req_wait_queue, &pending_list);
 
 	list_for_each_entry_safe(req, n, &pending_list, request_list) {
-		sd_dprintf("%"PRIx64, req->rq.obj.oid);
+		sd_debug("%"PRIx64, req->rq.obj.oid);
 		del_requeue_request(req);
 	}
 }
@@ -300,8 +290,12 @@ static void queue_gateway_request(struct request *req)
 	 * If we go for cache object, we don't care if it is being recovered
 	 * Even if it doesn't exist in cache, we'll rely on cache layer to pull
 	 * it.
+	 *
+	 * Not ture for local request because it might go for backend store
+	 * such as pushing cache object, in this case we should check if request
+	 * is in recovery.
 	 */
-	if (sys->enable_object_cache)
+	if (sys->enable_object_cache && !req->local)
 		goto queue_work;
 
 	if (req->local_oid)
@@ -345,35 +339,26 @@ static void queue_request(struct request *req)
 
 	req->op = get_sd_op(hdr->opcode);
 	if (!req->op) {
-		sd_eprintf("invalid opcode %d", hdr->opcode);
+		sd_err("invalid opcode %d", hdr->opcode);
 		rsp->result = SD_RES_INVALID_PARMS;
 		goto done;
 	}
 
-	sd_dprintf("%s, %d", op_name(req->op), sys->status);
+	sd_debug("%s, %d", op_name(req->op), sys->cinfo.status);
 
-	switch (sys->status) {
+	switch (sys->cinfo.status) {
 	case SD_STATUS_KILLED:
 		rsp->result = SD_RES_KILLED;
 		goto done;
 	case SD_STATUS_SHUTDOWN:
 		rsp->result = SD_RES_SHUTDOWN;
 		goto done;
-	case SD_STATUS_WAIT_FOR_FORMAT:
+	case SD_STATUS_WAIT:
 		if (!is_force_op(req->op)) {
-			rsp->result = SD_RES_WAIT_FOR_FORMAT;
-			goto done;
-		}
-		break;
-	case SD_STATUS_WAIT_FOR_JOIN:
-		if (!is_force_op(req->op)) {
-			rsp->result = SD_RES_WAIT_FOR_JOIN;
-			goto done;
-		}
-		break;
-	case SD_STATUS_HALT:
-		if (!is_force_op(req->op)) {
-			rsp->result = SD_RES_HALT;
+			if (sys->cinfo.ctime == 0)
+				rsp->result = SD_RES_WAIT_FOR_FORMAT;
+			else
+				rsp->result = SD_RES_WAIT_FOR_JOIN;
 			goto done;
 		}
 		break;
@@ -386,16 +371,16 @@ static void queue_request(struct request *req)
 	if (is_peer_op(req->op)) {
 		queue_peer_request(req);
 	} else if (is_gateway_op(req->op)) {
-		hdr->epoch = sys->epoch;
+		hdr->epoch = sys->cinfo.epoch;
 		queue_gateway_request(req);
 	} else if (is_local_op(req->op)) {
-		hdr->epoch = sys->epoch;
+		hdr->epoch = sys->cinfo.epoch;
 		queue_local_request(req);
 	} else if (is_cluster_op(req->op)) {
-		hdr->epoch = sys->epoch;
+		hdr->epoch = sys->cinfo.epoch;
 		queue_cluster_request(req);
 	} else {
-		sd_eprintf("unknown operation %d", hdr->opcode);
+		sd_err("unknown operation %d", hdr->opcode);
 		rsp->result = SD_RES_SYSTEM_ERROR;
 		goto done;
 	}
@@ -430,6 +415,8 @@ static struct request *alloc_local_request(void *data, int data_length)
 
 	INIT_LIST_HEAD(&req->request_list);
 
+	refcount_set(&req->refcnt, 1);
+
 	return req;
 }
 
@@ -445,35 +432,27 @@ static void free_local_request(struct request *req)
  * This function takes advantage of gateway's retry mechanism and can be only
  * called from worker thread.
  */
-int exec_local_req(struct sd_req *rq, void *data)
+worker_fn int exec_local_req(struct sd_req *rq, void *data)
 {
 	struct request *req;
-	eventfd_t value = 1;
 	int ret;
 
-	assert(is_worker_thread());
-
 	req = alloc_local_request(data, rq->data_length);
 	req->rq = *rq;
 	req->local_req_efd = eventfd(0, 0);
+	if (req->local_req_efd < 0) {
+		/* Fake the result to ask for retry */
+		req->rp.result = SD_RES_NETWORK_ERROR;
+		goto out;
+	}
 
 	pthread_mutex_lock(&sys->local_req_lock);
 	list_add_tail(&req->request_list, &sys->local_req_queue);
 	pthread_mutex_unlock(&sys->local_req_lock);
 
-	eventfd_write(sys->local_req_efd, value);
-
-again:
-	/* In error case (for e.g, EINTR) just retry read */
-	ret = eventfd_read(req->local_req_efd, &value);
-	if (ret < 0) {
-		sd_eprintf("%m");
-		if (errno == EINTR)
-			goto again;
-		/* Fake the result to ask for retry */
-		req->rp.result = SD_RES_NETWORK_ERROR;
-	}
-
+	eventfd_xwrite(sys->local_req_efd, 1);
+	eventfd_xread(req->local_req_efd);
+out:
 	/* fill rq with response header as exec_req does */
 	memcpy(rq, &req->rp, sizeof(req->rp));
 
@@ -493,7 +472,7 @@ static struct request *alloc_request(struct client_info *ci, int data_length)
 		return NULL;
 
 	req->ci = ci;
-	ci->refcnt++;
+	refcount_inc(&ci->refcnt);
 	if (data_length) {
 		req->data_length = data_length;
 		req->data = valloc(data_length);
@@ -504,7 +483,7 @@ static struct request *alloc_request(struct client_info *ci, int data_length)
 	}
 
 	INIT_LIST_HEAD(&req->request_list);
-	uatomic_set(&req->refcnt, 1);
+	refcount_set(&req->refcnt, 1);
 
 	uatomic_inc(&sys->nr_outstanding_reqs);
 
@@ -515,22 +494,21 @@ static void free_request(struct request *req)
 {
 	uatomic_dec(&sys->nr_outstanding_reqs);
 
-	req->ci->refcnt--;
+	refcount_dec(&req->ci->refcnt);
 	put_vnode_info(req->vinfo);
 	free(req->data);
 	free(req);
 }
 
-void put_request(struct request *req)
+main_fn void put_request(struct request *req)
 {
 	struct client_info *ci = req->ci;
-	eventfd_t value = 1;
 
-	if (uatomic_sub_return(&req->refcnt, 1) > 0)
+	if (refcount_dec(&req->refcnt) > 0)
 		return;
 
 	if (req->local)
-		eventfd_write(req->local_req_efd, value);
+		eventfd_xwrite(req->local_req_efd, 1);
 	else {
 		if (conn_tx_on(&ci->conn)) {
 			clear_client_info(ci);
@@ -587,7 +565,7 @@ static inline int begin_rx(struct client_info *ci)
 		ret = rx(conn, C_IO_END);
 		break;
 	default:
-		sd_eprintf("bug: unknown state %d", conn->c_rx_state);
+		sd_err("bug: unknown state %d", conn->c_rx_state);
 	}
 
 	if (is_conn_dead(conn)) {
@@ -609,7 +587,7 @@ static inline void finish_rx(struct client_info *ci)
 	req = ci->rx_req;
 	init_rx_hdr(ci);
 
-	sd_dprintf("%d, %s:%d", ci->conn.fd, ci->conn.ipstr, ci->conn.port);
+	sd_debug("%d, %s:%d", ci->conn.fd, ci->conn.ipstr, ci->conn.port);
 	queue_request(req);
 }
 
@@ -641,7 +619,7 @@ static void init_tx_hdr(struct client_info *ci)
 	/* use cpu_to_le */
 	memcpy(rsp, &req->rp, sizeof(*rsp));
 
-	rsp->epoch = sys->epoch;
+	rsp->epoch = sys->cinfo.epoch;
 	rsp->opcode = req->rq.opcode;
 	rsp->id = req->rq.id;
 }
@@ -695,8 +673,8 @@ static inline int finish_tx(struct client_info *ci)
 {
 	/* Finish sending one response */
 	if (ci->conn.c_tx_state == C_IO_END) {
-		sd_dprintf("connection from: %d, %s:%d", ci->conn.fd,
-			   ci->conn.ipstr, ci->conn.port);
+		sd_debug("connection from: %d, %s:%d", ci->conn.fd,
+			 ci->conn.ipstr, ci->conn.port);
 		free_request(ci->tx_req);
 		ci->tx_req = NULL;
 	}
@@ -726,7 +704,7 @@ static void do_client_tx(struct client_info *ci)
 
 static void destroy_client(struct client_info *ci)
 {
-	sd_dprintf("connection from: %s:%d", ci->conn.ipstr, ci->conn.port);
+	sd_debug("connection from: %s:%d", ci->conn.ipstr, ci->conn.port);
 	close(ci->conn.fd);
 	free(ci);
 }
@@ -735,7 +713,7 @@ static void clear_client_info(struct client_info *ci)
 {
 	struct request *req, *t;
 
-	sd_dprintf("connection seems to be dead");
+	sd_debug("connection seems to be dead");
 
 	if (ci->rx_req) {
 		free_request(ci->rx_req);
@@ -754,10 +732,10 @@ static void clear_client_info(struct client_info *ci)
 
 	unregister_event(ci->conn.fd);
 
-	sd_dprintf("refcnt:%d, fd:%d, %s:%d", ci->refcnt, ci->conn.fd,
-		   ci->conn.ipstr, ci->conn.port);
+	sd_debug("refcnt:%d, fd:%d, %s:%d", refcount_read(&ci->refcnt),
+		 ci->conn.fd, ci->conn.ipstr, ci->conn.port);
 
-	if (ci->refcnt)
+	if (refcount_read(&ci->refcnt))
 		return;
 
 	destroy_client(ci);
@@ -791,7 +769,7 @@ static struct client_info *create_client(int fd, struct cluster_info *cluster)
 
 	ci->conn.fd = fd;
 	ci->conn.events = EPOLLIN;
-	ci->refcnt = 0;
+	refcount_set(&ci->refcnt, 0);
 
 	INIT_LIST_HEAD(&ci->done_reqs);
 
@@ -804,8 +782,8 @@ static void client_handler(int fd, int events, void *data)
 {
 	struct client_info *ci = (struct client_info *)data;
 
-	sd_dprintf("%x, rx %d, tx %d", events, ci->conn.c_rx_state,
-		   ci->conn.c_tx_state);
+	sd_debug("%x, rx %d, tx %d", events, ci->conn.c_rx_state,
+		 ci->conn.c_tx_state);
 
 	if (events & (EPOLLERR | EPOLLHUP) || is_conn_dead(&ci->conn))
 		return clear_client_info(ci);
@@ -825,8 +803,8 @@ static void listen_handler(int listen_fd, int events, void *data)
 	struct client_info *ci;
 	bool is_inet_socket = *(bool *)data;
 
-	if (sys->status == SD_STATUS_SHUTDOWN) {
-		sd_dprintf("unregistering connection %d", listen_fd);
+	if (sys->cinfo.status == SD_STATUS_SHUTDOWN) {
+		sd_debug("unregistering connection %d", listen_fd);
 		unregister_event(listen_fd);
 		return;
 	}
@@ -834,7 +812,7 @@ static void listen_handler(int listen_fd, int events, void *data)
 	namesize = sizeof(from);
 	fd = accept(listen_fd, (struct sockaddr *)&from, &namesize);
 	if (fd < 0) {
-		sd_eprintf("failed to accept a new connection: %m");
+		sd_err("failed to accept a new connection: %m");
 		return;
 	}
 
@@ -864,7 +842,7 @@ static void listen_handler(int listen_fd, int events, void *data)
 		return;
 	}
 
-	sd_dprintf("accepted a new connection: %d", fd);
+	sd_debug("accepted a new connection: %d", fd);
 }
 
 static int create_listen_port_fn(int fd, void *data)
@@ -894,17 +872,13 @@ int init_unix_domain_socket(const char *dir)
 
 static void local_req_handler(int listen_fd, int events, void *data)
 {
-	eventfd_t value;
 	struct request *req, *t;
 	LIST_HEAD(pending_list);
-	int ret;
 
 	if (events & EPOLLERR)
-		sd_eprintf("request handler error");
+		sd_err("request handler error");
 
-	ret = eventfd_read(listen_fd, &value);
-	if (ret < 0)
-		return;
+	eventfd_xread(listen_fd);
 
 	pthread_mutex_lock(&sys->local_req_lock);
 	list_splice_init(&sys->local_req_queue, &pending_list);
@@ -924,3 +898,34 @@ void local_req_init(void)
 		panic("failed to init local req efd");
 	register_event(sys->local_req_efd, local_req_handler, NULL);
 }
+
+worker_fn int sheep_exec_req(const struct node_id *nid, struct sd_req *hdr,
+			     void *buf)
+{
+	struct sd_rsp *rsp = (struct sd_rsp *)hdr;
+	struct sockfd *sfd;
+	int ret;
+
+	sfd = sockfd_cache_get(nid);
+	if (!sfd)
+		return SD_RES_NETWORK_ERROR;
+
+	ret = exec_req(sfd->fd, hdr, buf, sheep_need_retry, hdr->epoch,
+		       MAX_RETRY_COUNT);
+	if (ret) {
+		sd_debug("remote node might have gone away");
+		sockfd_cache_del(nid, sfd);
+		return SD_RES_NETWORK_ERROR;
+	}
+	ret = rsp->result;
+	if (ret != SD_RES_SUCCESS)
+		sd_err("failed %s", sd_strerror(ret));
+
+	sockfd_cache_put(nid, sfd);
+	return ret;
+}
+
+bool sheep_need_retry(uint32_t epoch)
+{
+	return sys_epoch() == epoch;
+}
