@@ -9,28 +9,12 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "../include/config.h"
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <time.h>
-#include <signal.h>
-#include <linux/limits.h>
-#include <sys/syslog.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/signalfd.h>
-#include <pthread.h>
-#include <sys/eventfd.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <arpa/inet.h>
 #include <sys/resource.h>
+#include <malloc.h>
 
 #include "sheep_priv.h"
 #include "trace/trace.h"
-#include "util.h"
 #include "option.h"
 
 #define EPOLL_SIZE 4096
@@ -40,48 +24,139 @@
 LIST_HEAD(cluster_drivers);
 static const char program_name[] = "sheep";
 
+static const char bind_help[] =
+"Example:\n\t$ sheep -b 192.168.1.1 ...\n"
+"This tries to teach sheep listen to NIC of 192.168.1.1.\n"
+"\nExample:\n\t$ sheep -b 0.0.0.0 ...\n"
+"This tries to teach sheep listen to all the NICs available. It can be useful\n"
+"when you want sheep to response dog without specified address and port.\n";
+
+static const char ioaddr_help[] =
+"Example:\n\t$ sheep -i host=192.168.1.1,port=7002 ...\n"
+"This tries to add a dedicated IO NIC of 192.168.1.1:7002 to transfer data.\n"
+"If IO NIC is down, sheep will fallback to non IO NIC to transfer data.\n";
+
+static const char journal_help[] =
+"Available arguments:\n"
+"\tsize=: size of the journal in megabyes\n"
+"\tdir=: path to the location of the journal (default: $STORE)\n"
+"\tskip: if specified, skip the recovery at startup\n"
+"\nExample:\n\t$ sheep -j dir=/journal,size=1024\n"
+"This tries to use /journal as the journal storage of the size 1024M\n";
+
+static const char loglevel_help[] =
+"Available log levels:\n"
+"  #    Level           Description\n"
+"  0    SDOG_EMERG      system has failed and is unusable\n"
+"  1    SDOG_ALERT      action must be taken immediately\n"
+"  2    SDOG_CRIT       critical conditions\n"
+"  3    SDOG_ERR        error conditions\n"
+"  4    SDOG_WARNING    warning conditions\n"
+"  5    SDOG_NOTICE     normal but significant conditions\n"
+"  6    SDOG_INFO       informational notices\n"
+"  7    SDOG_DEBUG      debugging messages\n"
+"\nExample:\n\t$ sheep -l 4 ...\n"
+"This only allows logs with level smaller than SDOG_WARNING to be logged\n";
+
+static const char http_help[] =
+"Example:\n\t$ sheep -r localhost:7001 ...\n"
+"This tries to enable sheep as http service backend and use localhost:7001 to\n"
+"communicate with http server. Not fully implemented yet.\n";
+
+static const char myaddr_help[] =
+"Example:\n\t$ sheep -y 192.168.1.1:7000 ...\n"
+"This tries to tell other nodes through what address they can talk to this\n"
+"sheep.\n";
+
+static const char zone_help[] =
+"Example:\n\t$ sheep -z 1 ...\n"
+"This tries to set the zone ID of this sheep to 1 and sheepdog won't store\n"
+"more than one copy of any object into this same zone\n";
+
+static const char cluster_help[] =
+"Available arguments:\n"
+"\tlocal: use local driver\n"
+"\tcorosync: use corosync driver (default)\n"
+"\tzookeeper: use zookeeper driver, need extra arguments\n"
+"\n\tzookeeper arguments: address-list,tiemout=value (default as 3000)\n"
+"\nExample:\n\t"
+"$ sheep -c zookeeperr:IP1:PORT1,IP2:PORT2,IP3:PORT3,timeout=1000 ...\n"
+"This tries to use 3 node zookeeper cluster, which can be reached by\n"
+"IP1:PORT1, IP2:PORT2, IP3:PORT3 to manage membership and broadcast message\n"
+"and set the timeout of node heartbeat as 1000 milliseconds\n";
+
+static const char cache_help[] =
+"Available arguments:\n"
+"\tsize=: size of the cache in megabyes\n"
+"\tdir=: path to the location of the cache (default: $STORE/cache)\n"
+"\tdirectio: use directio mode for cache IO, "
+"if not specified use buffered IO\n"
+"\nExample:\n\t$ sheep -w size=200000,dir=/my_ssd,directio ...\n"
+"This tries to use /my_ssd as the cache storage with 200G allocted to the\n"
+"cache in directio mode\n";
+
 static struct sd_option sheep_options[] = {
-	{'b', "bindaddr", true, "specify IP address of interface to listen on"},
-	{'c', "cluster", true, "specify the cluster driver"},
+	{'b', "bindaddr", true, "specify IP address of interface to listen on",
+	 bind_help},
+	{'c', "cluster", true,
+	 "specify the cluster driver (default: "DEFAULT_CLUSTER_DRIVER")",
+	 cluster_help},
 	{'d', "debug", false, "include debug messages in the log"},
 	{'D', "directio", false, "use direct IO for backend store"},
 	{'f', "foreground", false, "make the program run in the foreground"},
 	{'F', "log-format", true, "specify log format"},
 	{'g', "gateway", false, "make the progam run as a gateway mode"},
 	{'h', "help", false, "display this help and exit"},
-	{'i', "ioaddr", true, "use separate network card to handle IO requests"},
-	{'j', "journal", true, "use jouranl file to log all the write operations"},
-	{'l', "loglevel", true, "specify the level of logging detail"},
+	{'i', "ioaddr", true, "use separate network card to handle IO requests",
+	 ioaddr_help},
+	{'j', "journal", true, "use jouranl file to log all the write "
+	 "operations", journal_help},
+	{'l', "loglevel", true, "specify the level of logging detail "
+	 "(default: 6 [SDOG_INFO])", loglevel_help},
 	{'n', "nosync", false, "drop O_SYNC for write of backend"},
 	{'o', "stdout", false, "log to stdout instead of shared logger"},
-	{'p', "port", true, "specify the TCP port on which to listen"},
+	{'p', "port", true, "specify the TCP port on which to listen "
+	 "(default: 7000)"},
 	{'P', "pidfile", true, "create a pid file"},
+	{'r', "http", true, "enable http service", http_help},
 	{'u', "upgrade", false, "upgrade to the latest data layout"},
 	{'v', "version", false, "show the version"},
-	{'w', "enable-cache", true, "enable object cache"},
-	{'y', "myaddr", true, "specify the address advertised to other sheep"},
-	{'z', "zone", true, "specify the zone id"},
+	{'w', "cache", true, "enable object cache", cache_help},
+	{'y', "myaddr", true, "specify the address advertised to other sheep",
+	 myaddr_help},
+	{'z', "zone", true,
+	 "specify the zone id (default: determined by listen address)",
+	 zone_help},
 	{ 0, NULL, false, NULL },
 };
 
 static void usage(int status)
 {
-	if (status)
-		fprintf(stderr, "Try `%s --help' for more information.\n",
-			program_name);
-	else {
+	if (status) {
+		const char *help = option_get_help(sheep_options, optopt);
+
+		if (help) {
+			printf("%s", help);
+			goto out;
+		}
+
+		sd_err("Try '%s --help' for more information.", program_name);
+	} else {
 		struct sd_option *opt;
 
 		printf("Sheepdog daemon (version %s)\n"
-		       "Usage: %s [OPTION]... [PATH]\n"
+		       "Usage: %s [OPTION]... [PATH] (default: /tmp)\n"
 		       "Options:\n", PACKAGE_VERSION, program_name);
 
 		sd_for_each_option(opt, sheep_options) {
 			printf("  -%c, --%-18s%s\n", opt->ch, opt->name,
 			       opt->desc);
 		}
-	}
 
+		printf("\nTry '%s <option>', e.g., '%s -w', to get more detail "
+		       "about specific option\n", program_name, program_name);
+	}
+out:
 	exit(status);
 }
 
@@ -133,13 +208,13 @@ static void signal_handler(int listen_fd, int events, void *data)
 
 	ret = read(sigfd, &siginfo, sizeof(siginfo));
 	assert(ret == sizeof(siginfo));
-	sd_dprintf("signal %d", siginfo.ssi_signo);
+	sd_debug("signal %d", siginfo.ssi_signo);
 	switch (siginfo.ssi_signo) {
 	case SIGTERM:
-		sys->status = SD_STATUS_KILLED;
+		sys->cinfo.status = SD_STATUS_KILLED;
 		break;
 	default:
-		sd_eprintf("signal %d unhandled", siginfo.ssi_signo);
+		sd_err("signal %d unhandled", siginfo.ssi_signo);
 		break;
 	}
 }
@@ -149,35 +224,30 @@ static int init_signal(void)
 	sigset_t mask;
 	int ret;
 
-	ret = trace_init_signal();
-	if (ret)
-		return ret;
-
 	sigemptyset(&mask);
 	sigaddset(&mask, SIGTERM);
 	sigprocmask(SIG_BLOCK, &mask, NULL);
 
 	sigfd = signalfd(-1, &mask, SFD_NONBLOCK);
 	if (sigfd < 0) {
-		sd_eprintf("failed to create a signal fd: %m");
+		sd_err("failed to create a signal fd: %m");
 		return -1;
 	}
 
 	ret = register_event(sigfd, signal_handler, NULL);
 	if (ret) {
-		sd_eprintf("failed to register signal handler (%d)", ret);
+		sd_err("failed to register signal handler (%d)", ret);
 		return -1;
 	}
 
-	sd_dprintf("register signal_handler for %d", sigfd);
+	sd_debug("register signal_handler for %d", sigfd);
 
 	return 0;
 }
 
 static void crash_handler(int signo)
 {
-	sd_printf(SDOG_EMERG, "sheep exits unexpectedly (%s).",
-		  strsignal(signo));
+	sd_emerg("sheep exits unexpectedly (%s).", strsignal(signo));
 
 	sd_backtrace();
 	sd_dump_variable(__sys);
@@ -185,8 +255,8 @@ static void crash_handler(int signo)
 	reraise_crash_signal(signo, 1);
 }
 
-static struct cluster_info __sys;
-struct cluster_info *sys = &__sys;
+static struct system_info __sys;
+struct system_info *sys = &__sys;
 
 static void parse_arg(char *arg, const char *delim, void (*fn)(char *))
 {
@@ -217,9 +287,8 @@ static void object_cache_size_set(char *s)
 	return;
 
 err:
-	fprintf(stderr, "Invalid object cache option '%s': "
-		"size must be an integer between 1 and %"PRIu32" inclusive\n",
-		s, max_cache_size);
+	sd_err("Invalid object cache option '%s': size must be an integer "
+	       "between 1 and %" PRIu32 " inclusive", s, max_cache_size);
 	exit(1);
 }
 
@@ -263,7 +332,7 @@ static void _object_cache_set(char *s)
 		}
 	}
 
-	fprintf(stderr, "invalid object cache arg: %s\n", s);
+	sd_err("invalid object cache arg: %s", s);
 	exit(1);
 }
 
@@ -275,7 +344,7 @@ static void object_cache_set(char *arg)
 	parse_arg(arg, ",", _object_cache_set);
 
 	if (sys->object_cache_size == 0) {
-		fprintf(stderr, "object cache size is not set\n");
+		sd_err("object cache size is not set");
 		exit(1);
 	}
 }
@@ -297,15 +366,15 @@ static void init_journal_arg(char *arg)
 		arg += szl;
 		jsize = strtoll(arg, NULL, 10);
 		if (jsize < MIN_JOURNAL_SIZE || jsize == LLONG_MAX) {
-			fprintf(stderr, "invalid size %s, "
-				"must be bigger than %u(M)\n", arg,
+			sd_err("invalid size %s, must be bigger than %u(M)",
+			       arg,
 				MIN_JOURNAL_SIZE);
 			exit(1);
 		}
 	} else if (!strncmp(sp, arg, spl)) {
 		jskip = true;
 	} else {
-		fprintf(stderr, "invalid paramters %s\n", arg);
+		sd_err("invalid paramters %s", arg);
 		exit(1);
 	}
 }
@@ -323,9 +392,8 @@ static void init_io_arg(char *arg)
 		arg += hl;
 		io_pt = arg;
 	} else {
-		fprintf(stderr, "invalid paramters %s. "
-			"Use '-i host=a.b.c.d,port=xxx'\n",
-			arg);
+		sd_err("invalid paramters %s. Use '-i host=a.b.c.d,port=xxx'",
+		       arg);
 		exit(1);
 	}
 }
@@ -345,8 +413,7 @@ static size_t get_nr_nodes(void)
 
 static int create_work_queues(void)
 {
-	if (init_work_queue(get_nr_nodes, trace_register_thread,
-			    trace_unregister_thread))
+	if (init_work_queue(get_nr_nodes))
 		return -1;
 
 	sys->gateway_wqueue = create_work_queue("gway", WQ_UNLIMITED);
@@ -354,7 +421,6 @@ static int create_work_queues(void)
 	sys->recovery_wqueue = create_ordered_work_queue("rw");
 	sys->deletion_wqueue = create_ordered_work_queue("deletion");
 	sys->block_wqueue = create_ordered_work_queue("block");
-	sys->sockfd_wqueue = create_ordered_work_queue("sockfd");
 	sys->md_wqueue = create_ordered_work_queue("md");
 	if (sys->enable_object_cache) {
 		sys->oc_reclaim_wqueue =
@@ -364,9 +430,9 @@ static int create_work_queues(void)
 			return -1;
 	}
 	if (!sys->gateway_wqueue || !sys->io_wqueue || !sys->recovery_wqueue ||
-	    !sys->deletion_wqueue || !sys->block_wqueue ||
-	    !sys->sockfd_wqueue || !sys->md_wqueue)
+	    !sys->deletion_wqueue || !sys->block_wqueue || !sys->md_wqueue)
 			return -1;
+
 	return 0;
 }
 
@@ -385,23 +451,39 @@ static void check_host_env(void)
 	struct rlimit r;
 
 	if (getrlimit(RLIMIT_NOFILE, &r) < 0)
-		sd_eprintf("failed to get nofile %m");
+		sd_err("failed to get nofile %m");
 	/*
 	 * 1024 is default for NOFILE on most distributions, which is very
 	 * dangerous to run Sheepdog cluster.
 	 */
 	else if (r.rlim_cur == 1024)
-		sd_eprintf("WARN: Allowed open files 1024 too small, "
-			   "suggested %u", SD_RLIM_NOFILE);
+		sd_err("WARN: Allowed open files 1024 too small, suggested %u",
+		       SD_RLIM_NOFILE);
 	else if (r.rlim_cur < SD_RLIM_NOFILE)
-		sd_iprintf("Allowed open files %lu, suggested %u", r.rlim_cur,
-			   SD_RLIM_NOFILE);
+		sd_info("Allowed open files %lu, suggested %u", r.rlim_cur,
+			SD_RLIM_NOFILE);
 
 	if (getrlimit(RLIMIT_CORE, &r) < 0)
-		sd_eprintf("failed to get core %m");
+		sd_debug("failed to get core %m");
 	else if (r.rlim_cur < RLIM_INFINITY)
-		sd_iprintf("Allowed core file size %lu, suggested unlimited",
-			   r.rlim_cur);
+		sd_debug("Allowed core file size %lu, suggested unlimited",
+			 r.rlim_cur);
+
+	/*
+	 * Disable glibc's dynamic mmap threshold and set it as 512k.
+	 *
+	 * We have to disable dynamic threshold because its inefficiency to
+	 * release freed memory back to OS. Setting it as 512k practically means
+	 * allocation larger than or equal to 512k will use mmap() for malloc()
+	 * and munmap() for free(), guaranteeing allocated memory will not be
+	 * cached in the glibc's ptmalloc internal pool.
+	 *
+	 * 512k is not a well tested optimal value for IO request size, I choose
+	 * it because it is default value for disk drive that it can transfer at
+	 * a time. So default installation of guest will issue at most 512K
+	 * sized request.
+	 */
+	mallopt(M_MMAP_THRESHOLD, 512 * 1024);
 }
 
 static int lock_and_daemon(bool daemonize, const char *base_dir)
@@ -430,8 +512,7 @@ static int lock_and_daemon(bool daemonize, const char *base_dir)
 		}
 
 		if (setsid() == -1) {
-			sd_eprintf("becoming a leader of a new session"
-				" failed: %m");
+			sd_err("becoming a leader of a new session failed: %m");
 			status = 1;
 			goto end;
 		}
@@ -440,7 +521,7 @@ static int lock_and_daemon(bool daemonize, const char *base_dir)
 		case 0:
 			break;
 		case -1:
-			sd_eprintf("fork() failed during daemonize: %m");
+			sd_err("fork() failed during daemonize: %m");
 			status = 1;
 			goto end;
 		default:
@@ -449,14 +530,14 @@ static int lock_and_daemon(bool daemonize, const char *base_dir)
 		}
 
 		if (chdir("/")) {
-			sd_eprintf("chdir to / failed: %m");
+			sd_err("chdir to / failed: %m");
 			status = 1;
 			goto end;
 		}
 
 		devnull_fd = open("/dev/null", O_RDWR);
 		if (devnull_fd < 0) {
-			sd_eprintf("opening /dev/null failed: %m");
+			sd_err("opening /dev/null failed: %m");
 			status = 1;
 			goto end;
 		}
@@ -464,7 +545,7 @@ static int lock_and_daemon(bool daemonize, const char *base_dir)
 
 	ret = lock_base_dir(base_dir);
 	if (ret < 0) {
-		sd_eprintf("locking directory: %s failed", base_dir);
+		sd_err("locking directory: %s failed", base_dir);
 		status = 1;
 		goto end;
 	}
@@ -500,7 +581,7 @@ int main(int argc, char **argv)
 	int64_t zone = -1;
 	struct cluster_driver *cdrv;
 	struct option *long_options;
-	const char *log_format = "default";
+	const char *log_format = "server", *http_address = NULL;
 	static struct logger_user_info sheep_info;
 
 	install_crash_handler(crash_handler);
@@ -515,14 +596,16 @@ int main(int argc, char **argv)
 			port = strtol(optarg, &p, 10);
 			if (optarg == p || port < 1 || UINT16_MAX < port
 				|| *p != '\0') {
-				fprintf(stderr, "Invalid port number '%s'\n",
-					optarg);
+				sd_err("Invalid port number '%s'", optarg);
 				exit(1);
 			}
 			break;
 		case 'P':
 			pid_file = optarg;
 			break;
+		case 'r':
+			http_address = optarg;
+			break;
 		case 'f':
 			is_daemon = false;
 			break;
@@ -530,8 +613,7 @@ int main(int argc, char **argv)
 			log_level = strtol(optarg, &p, 10);
 			if (optarg == p || log_level < SDOG_EMERG ||
 			    SDOG_DEBUG < log_level || *p != '\0') {
-				fprintf(stderr, "Invalid log level '%s'\n",
-					optarg);
+				sd_err("Invalid log level '%s'", optarg);
 				sdlog_help();
 				exit(1);
 			}
@@ -541,8 +623,7 @@ int main(int argc, char **argv)
 			break;
 		case 'y':
 			if (!str_to_addr(optarg, sys->this_node.nid.addr)) {
-				fprintf(stderr, "Invalid address: '%s'\n",
-					optarg);
+				sd_err("Invalid address: '%s'", optarg);
 				exit(1);
 			}
 			explicit_addr = true;
@@ -565,9 +646,9 @@ int main(int argc, char **argv)
 			zone = strtol(optarg, &p, 10);
 			if (optarg == p || zone < 0 || UINT32_MAX < zone
 				|| *p != '\0') {
-				fprintf(stderr, "Invalid zone id '%s': "
-					"must be an integer between 0 and %u\n",
-					optarg, UINT32_MAX);
+				sd_err("Invalid zone id '%s': must be "
+				       "an integer between 0 and %u", optarg,
+				       UINT32_MAX);
 				exit(1);
 			}
 			sys->this_node.zone = zone;
@@ -578,7 +659,7 @@ int main(int argc, char **argv)
 		case 'c':
 			sys->cdrv = find_cdrv(optarg);
 			if (!sys->cdrv) {
-				fprintf(stderr, "Invalid cluster driver '%s'\n", optarg);
+				sd_err("Invalid cluster driver '%s'", optarg);
 				fprintf(stderr, "Supported drivers:");
 				FOR_EACH_CLUSTER_DRIVER(cdrv) {
 					fprintf(stderr, " %s", cdrv->name);
@@ -595,15 +676,13 @@ int main(int argc, char **argv)
 		case 'i':
 			parse_arg(optarg, ",", init_io_arg);
 			if (!str_to_addr(io_addr, sys->this_node.nid.io_addr)) {
-				fprintf(stderr, "Bad addr: '%s'\n",
-					io_addr);
+				sd_err("Bad addr: '%s'", io_addr);
 				exit(1);
 			}
 
 			if (io_pt)
 				if (sscanf(io_pt, "%u", &io_port) != 1) {
-					fprintf(stderr, "Bad port '%s'\n",
-						io_pt);
+					sd_err("Bad port '%s'", io_pt);
 					exit(1);
 				}
 			sys->this_node.nid.io_port = io_port;
@@ -612,8 +691,7 @@ int main(int argc, char **argv)
 			uatomic_set_true(&sys->use_journal);
 			parse_arg(optarg, ",", init_journal_arg);
 			if (!jsize) {
-				fprintf(stderr,
-					"you must specify size for journal\n");
+				sd_err("you must specify size for journal");
 				exit(1);
 			}
 			break;
@@ -639,10 +717,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	/*
-	 * early_log_init() must be called before any calling of
-	 * sd_printf() series
-	 */
 	sheep_info.port = port;
 	early_log_init(log_format, &sheep_info);
 
@@ -662,7 +736,7 @@ int main(int argc, char **argv)
 
 	dir = realpath(dirp, NULL);
 	if (!dir) {
-		fprintf(stderr, "%m\n");
+		sd_err("%m");
 		exit(1);
 	}
 
@@ -673,8 +747,7 @@ int main(int argc, char **argv)
 	if (lock_and_daemon(is_daemon, dir))
 		exit(1);
 
-	ret = log_init(program_name, LOG_SPACE_SIZE, to_stdout, log_level,
-		path);
+	ret = log_init(program_name, to_stdout, log_level, path);
 	if (ret)
 		exit(1);
 
@@ -715,7 +788,7 @@ int main(int argc, char **argv)
 
 	ret = create_cluster(port, zone, nr_vnodes, explicit_addr);
 	if (ret) {
-		sd_eprintf("failed to create sheepdog cluster");
+		sd_err("failed to create sheepdog cluster");
 		exit(1);
 	}
 
@@ -724,7 +797,7 @@ int main(int argc, char **argv)
 		if (!strlen(jpath))
 			/* internal journal */
 			memcpy(jpath, dir, strlen(dir));
-		sd_dprintf("%s, %zd, %d", jpath, jsize, jskip);
+		sd_debug("%s, %zd, %d", jpath, jsize, jskip);
 		ret = journal_file_init(jpath, jsize, jskip);
 		if (ret)
 			exit(1);
@@ -741,6 +814,10 @@ int main(int argc, char **argv)
 	if (ret)
 		exit(1);
 
+	ret = sockfd_init();
+	if (ret)
+		exit(1);
+
 	ret = init_store_driver(sys->gateway_only);
 	if (ret)
 		exit(1);
@@ -758,32 +835,34 @@ int main(int argc, char **argv)
 	if (ret)
 		exit(1);
 
+	if (http_address && http_init(http_address) != 0)
+		exit(1);
+
 	if (pid_file && (create_pidfile(pid_file) != 0)) {
-		fprintf(stderr, "failed to pid file '%s' - %m\n", pid_file);
+		sd_err("failed to pid file '%s' - %m", pid_file);
 		exit(1);
 	}
 
 	if (chdir(dir) < 0) {
-		fprintf(stderr, "failed to chdir to %s: %m\n", dir);
+		sd_err("failed to chdir to %s: %m", dir);
 		exit(1);
 	}
 
 	free(dir);
 	check_host_env();
-	sd_printf(SDOG_NOTICE, "sheepdog daemon (version %s) started",
-		  PACKAGE_VERSION);
+	sd_info("sheepdog daemon (version %s) started", PACKAGE_VERSION);
 
 	while (sys->nr_outstanding_reqs != 0 ||
-	       (sys->status != SD_STATUS_KILLED &&
-		sys->status != SD_STATUS_SHUTDOWN))
+	       (sys->cinfo.status != SD_STATUS_KILLED &&
+		sys->cinfo.status != SD_STATUS_SHUTDOWN))
 		event_loop(-1);
 
-	sd_printf(SDOG_INFO, "shutdown");
+	sd_info("shutdown");
 
 	leave_cluster();
 
 	if (uatomic_is_true(&sys->use_journal)) {
-		sd_iprintf("cleaning journal file");
+		sd_info("cleaning journal file");
 		clean_journal_file(jpath);
 	}
 
