@@ -15,16 +15,52 @@
 #include <stdbool.h>
 #include <urcu/uatomic.h>
 #include <time.h>
+#include <pthread.h>
+#include <math.h>
+#include <errno.h>
+#include <poll.h>
+#include <sys/statvfs.h>
+#include <sys/stat.h>
+#include <sys/xattr.h>
+#include <dirent.h>
+#include <string.h>
+#include <fcntl.h>
+#include <sys/mman.h>
+#include <sys/time.h>
+#include <sys/epoll.h>
+#include <signal.h>
 
 #include "sheepdog_proto.h"
 #include "event.h"
-#include "logger.h"
+#include "util.h"
 #include "work.h"
 #include "net.h"
 #include "sheep.h"
 #include "cluster.h"
 #include "rbtree.h"
 #include "strbuf.h"
+#include "sha1.h"
+#include "config.h"
+#include "sockfd_cache.h"
+
+ /*
+  * Functions that update global info must be called in the main
+  * thread.  Add main_fn markers to such functions.
+  *
+  * Functions that can sleep (e.g. disk I/Os or network I/Os) must be
+  * called in the worker threads.  Add worker_fn markers to such
+  * functions.
+  */
+#ifdef HAVE_TRACE
+#define MAIN_FN_SECTION ".sd_main"
+#define WORKER_FN_SECTION ".sd_worker"
+
+#define main_fn __attribute__((section(MAIN_FN_SECTION)))
+#define worker_fn __attribute__((section(WORKER_FN_SECTION)))
+#else
+#define main_fn
+#define worker_fn
+#endif
 
 struct client_info {
 	struct connection conn;
@@ -35,7 +71,14 @@ struct client_info {
 
 	struct list_head done_reqs;
 
-	int refcnt;
+	refcnt_t refcnt;
+};
+
+enum REQUST_STATUS {
+	REQUEST_INIT,
+	REQUEST_QUEUED,
+	REQUEST_DONE,
+	REQUEST_DROPPED
 };
 
 struct request {
@@ -51,7 +94,7 @@ struct request {
 	struct list_head request_list;
 	struct list_head pending_list;
 
-	int refcnt;
+	refcnt_t refcnt;
 	bool local;
 	int local_req_efd;
 
@@ -60,25 +103,21 @@ struct request {
 	struct vnode_info *vinfo;
 
 	struct work work;
+	enum REQUST_STATUS status;
 };
 
-struct cluster_info {
+struct system_info {
 	struct cluster_driver *cdrv;
 	const char *cdrv_option;
 
-	/* set after finishing the JOIN procedure */
-	bool join_finished;
 	struct sd_node this_node;
 
-	uint32_t epoch;
-	uint32_t status;
-	uint16_t flags;
+	struct cluster_info cinfo;
 
 	uint64_t disk_space;
 
 	DECLARE_BITMAP(vdi_inuse, SD_NR_VDIS);
 
-	uint8_t nr_copies;
 	int local_req_efd;
 
 	pthread_mutex_t local_req_lock;
@@ -87,7 +126,6 @@ struct cluster_info {
 	int nr_outstanding_reqs;
 
 	bool gateway_only;
-	bool disable_recovery;
 	bool nosync;
 
 	struct work_queue *gateway_wqueue;
@@ -96,10 +134,12 @@ struct cluster_info {
 	struct work_queue *recovery_wqueue;
 	struct work_queue *recovery_notify_wqueue;
 	struct work_queue *block_wqueue;
-	struct work_queue *sockfd_wqueue;
 	struct work_queue *oc_reclaim_wqueue;
 	struct work_queue *oc_push_wqueue;
 	struct work_queue *md_wqueue;
+#ifdef HAVE_HTTP
+	struct work_queue *http_wqueue;
+#endif
 
 	bool enable_object_cache;
 
@@ -200,7 +240,7 @@ static inline struct store_driver *find_store_driver(const char *name)
 	return NULL;
 }
 
-extern struct cluster_info *sys;
+extern struct system_info *sys;
 extern struct store_driver *sd_store;
 extern char *obj_path;
 extern char *epoch_path;
@@ -208,7 +248,7 @@ extern char *epoch_path;
 /* One should call this function to get sys->epoch outside main thread */
 static inline uint32_t sys_epoch(void)
 {
-	return uatomic_read(&sys->epoch);
+	return uatomic_read(&sys->cinfo.epoch);
 }
 
 static inline bool is_aligned_to_pagesize(void *p)
@@ -247,7 +287,6 @@ int get_vdi_attr(struct sheepdog_vdi_attr *vattr, int data_len, uint32_t vid,
 int local_get_node_list(const struct sd_req *req, struct sd_rsp *rsp,
 		void *data);
 
-bool have_enough_zones(void);
 struct vnode_info *grab_vnode_info(struct vnode_info *vnode_info);
 struct vnode_info *get_vnode_info(void);
 void put_vnode_info(struct vnode_info *vinfo);
@@ -271,17 +310,14 @@ int leave_cluster(void);
 void queue_cluster_request(struct request *req);
 
 int update_epoch_log(uint32_t epoch, struct sd_node *nodes, size_t nr_nodes);
-int log_current_epoch(void);
+int inc_and_log_epoch(void);
 
 extern char *config_path;
-int set_cluster_copies(uint8_t copies);
-int get_cluster_copies(uint8_t *copies);
-int set_cluster_flags(uint16_t flags);
-int get_cluster_flags(uint16_t *flags);
-int set_cluster_store(const char *name);
-int get_cluster_store(char *buf);
+int set_cluster_config(const struct cluster_info *cinfo);
+int get_cluster_config(struct cluster_info *cinfo);
 int set_node_space(uint64_t space);
 int get_node_space(uint64_t *space);
+bool is_cluster_formatted(void);
 
 int store_file_write(void *buffer, size_t len);
 void *store_file_read(void);
@@ -294,14 +330,13 @@ int epoch_log_read_remote(uint32_t epoch, struct sd_node *nodes, int len,
 uint32_t get_latest_epoch(void);
 void init_config_path(const char *base_path);
 int init_config_file(void);
-int set_cluster_ctime(uint64_t ctime);
-uint64_t get_cluster_ctime(void);
 int get_obj_list(const struct sd_req *, struct sd_rsp *, void *);
 int objlist_cache_cleanup(uint32_t vid);
 
 int start_recovery(struct vnode_info *cur_vinfo, struct vnode_info *, bool);
 bool oid_in_recovery(uint64_t oid);
 bool node_in_recovery(void);
+void get_recovery_state(struct recovery_state *state);
 
 int read_backend_object(uint64_t oid, char *data, unsigned int datalen,
 		       uint64_t offset);
@@ -376,20 +411,12 @@ int object_cache_flush_vdi(uint32_t vid);
 int object_cache_flush_and_del(const struct request *req);
 void object_cache_delete(uint32_t vid);
 int object_cache_init(const char *p);
+int object_cache_remove(uint64_t oid);
+int object_cache_get_info(struct object_cache_info *info);
 
 /* store layout migration */
 int sd_migrate_store(int from, int to);
 
-/* sockfd_cache */
-struct sockfd {
-	int fd;
-	int idx;
-};
-
-void sockfd_cache_del(const struct node_id *);
-void sockfd_cache_add(const struct node_id *);
-void sockfd_cache_add_group(const struct sd_node *nodes, int nr);
-
 struct sockfd *sheep_get_sockfd(const struct node_id *);
 void sheep_put_sockfd(const struct node_id *, struct sockfd *);
 void sheep_del_sockfd(const struct node_id *, struct sockfd *);
@@ -414,7 +441,16 @@ uint32_t md_get_info(struct sd_md_info *info);
 int md_plug_disks(char *disks);
 int md_unplug_disks(char *disks);
 uint64_t md_get_size(uint64_t *used);
-void kick_node_recover(void);
-void update_node_size(struct sd_node *node);
+
+/* http.c */
+#ifdef HAVE_HTTP
+int http_init(const char *address);
+#else
+static inline int http_init(const char *address)
+{
+	sd_notice("http service is not complied");
+	return 0;
+}
+#endif /* END BUILD_HTTP */
 
 #endif
