@@ -8,12 +8,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <pthread.h>
-#include <sys/time.h>
 
-#include "sheepdog_proto.h"
 #include "sheep_priv.h"
 
 struct vdi_state_entry {
@@ -25,7 +20,7 @@ struct vdi_state_entry {
 
 static uint32_t max_copies;
 static struct rb_root vdi_state_root = RB_ROOT;
-static pthread_rwlock_t vdi_state_lock = PTHREAD_RWLOCK_INITIALIZER;
+static struct sd_lock vdi_state_lock = SD_LOCK_INITIALIZER;
 
 static struct vdi_state_entry *vdi_state_search(struct rb_root *root,
 						uint32_t vid)
@@ -75,12 +70,12 @@ static bool vid_is_snapshot(uint32_t vid)
 {
 	struct vdi_state_entry *entry;
 
-	pthread_rwlock_rdlock(&vdi_state_lock);
+	sd_read_lock(&vdi_state_lock);
 	entry = vdi_state_search(&vdi_state_root, vid);
-	pthread_rwlock_unlock(&vdi_state_lock);
+	sd_unlock(&vdi_state_lock);
 
 	if (!entry) {
-		sd_eprintf("No VDI entry for %" PRIx32 " found", vid);
+		sd_err("No VDI entry for %" PRIx32 " found", vid);
 		return 0;
 	}
 
@@ -100,13 +95,14 @@ int get_vdi_copy_number(uint32_t vid)
 {
 	struct vdi_state_entry *entry;
 
-	pthread_rwlock_rdlock(&vdi_state_lock);
+	sd_read_lock(&vdi_state_lock);
 	entry = vdi_state_search(&vdi_state_root, vid);
-	pthread_rwlock_unlock(&vdi_state_lock);
+	sd_unlock(&vdi_state_lock);
 
 	if (!entry) {
-		sd_eprintf("No VDI copy entry for %" PRIx32 " found", vid);
-		return 0;
+		sd_alert("copy number for %" PRIx32 " not found, set %d", vid,
+			 sys->cinfo.nr_copies);
+		return sys->cinfo.nr_copies;
 	}
 
 	return entry->nr_copies;
@@ -134,7 +130,7 @@ int get_max_copy_number(void)
 	int nr_copies = uatomic_read(&max_copies);
 
 	if (nr_copies == 0)
-		nr_copies = sys->nr_copies;
+		nr_copies = sys->cinfo.nr_copies;
 
 	return nr_copies;
 }
@@ -148,9 +144,9 @@ int add_vdi_state(uint32_t vid, int nr_copies, bool snapshot)
 	entry->nr_copies = nr_copies;
 	entry->snapshot = snapshot;
 
-	sd_dprintf("%" PRIx32 ", %d", vid, nr_copies);
+	sd_debug("%" PRIx32 ", %d", vid, nr_copies);
 
-	pthread_rwlock_wrlock(&vdi_state_lock);
+	sd_write_lock(&vdi_state_lock);
 	old = vdi_state_insert(&vdi_state_root, entry);
 	if (old) {
 		free(entry);
@@ -162,7 +158,7 @@ int add_vdi_state(uint32_t vid, int nr_copies, bool snapshot)
 	if (uatomic_read(&max_copies) == 0 ||
 	    nr_copies > uatomic_read(&max_copies))
 		uatomic_set(&max_copies, nr_copies);
-	pthread_rwlock_unlock(&vdi_state_lock);
+	sd_unlock(&vdi_state_lock);
 
 	return SD_RES_SUCCESS;
 }
@@ -174,7 +170,7 @@ int fill_vdi_state_list(void *data)
 	struct vdi_state *vs = data;
 	struct vdi_state_entry *entry;
 
-	pthread_rwlock_rdlock(&vdi_state_lock);
+	sd_read_lock(&vdi_state_lock);
 	for (n = rb_first(&vdi_state_root); n; n = rb_next(n)) {
 		entry = rb_entry(n, struct vdi_state_entry, node);
 		memset(vs, 0, sizeof(*vs));
@@ -184,7 +180,7 @@ int fill_vdi_state_list(void *data)
 		vs++;
 		nr++;
 	}
-	pthread_rwlock_unlock(&vdi_state_lock);
+	sd_unlock(&vdi_state_lock);
 
 	return nr * sizeof(*vs);
 }
@@ -203,7 +199,7 @@ int vdi_exist(uint32_t vid)
 	ret = read_object(vid_to_vdi_oid(vid), (char *)inode,
 			  sizeof(*inode), 0);
 	if (ret != SD_RES_SUCCESS) {
-		sd_eprintf("fail to read vdi inode (%" PRIx32 ")", vid);
+		sd_err("fail to read vdi inode (%" PRIx32 ")", vid);
 		ret = 0;
 		goto out;
 	}
@@ -249,13 +245,13 @@ static int create_vdi_obj(struct vdi_iocb *iocb, uint32_t new_vid,
 
 	if (iocb->create_snapshot) {
 		if (cur_vid != iocb->base_vid) {
-			sd_printf(SDOG_INFO, "tree snapshot %s %" PRIx32 " %"
-				  PRIx32, name, cur_vid, iocb->base_vid);
+			sd_info("tree snapshot %s %" PRIx32 " %" PRIx32, name,
+				cur_vid, iocb->base_vid);
 
 			ret = read_object(vid_to_vdi_oid(cur_vid), (char *)cur,
 					  SD_INODE_HEADER_SIZE, 0);
 			if (ret != SD_RES_SUCCESS) {
-				sd_printf(SDOG_ERR, "failed");
+				sd_err("failed");
 				ret = SD_RES_BASE_VDI_READ;
 				goto out;
 			}
@@ -297,7 +293,7 @@ static int create_vdi_obj(struct vdi_iocb *iocb, uint32_t new_vid,
 		ret = write_object(vid_to_vdi_oid(cur_vid), (char *)cur,
 				   SD_INODE_HEADER_SIZE, 0, false);
 		if (ret != 0) {
-			sd_printf(SDOG_ERR, "failed");
+			sd_err("failed");
 			ret = SD_RES_BASE_VDI_READ;
 			goto out;
 		}
@@ -307,7 +303,7 @@ static int create_vdi_obj(struct vdi_iocb *iocb, uint32_t new_vid,
 		ret = write_object(vid_to_vdi_oid(iocb->base_vid), (char *)base,
 				   SD_INODE_HEADER_SIZE, 0, false);
 		if (ret != 0) {
-			sd_printf(SDOG_ERR, "failed");
+			sd_err("failed");
 			ret = SD_RES_BASE_VDI_WRITE;
 			goto out;
 		}
@@ -383,7 +379,7 @@ static int fill_vdi_info_range(uint32_t left, uint32_t right,
 
 	inode = malloc(SD_INODE_HEADER_SIZE);
 	if (!inode) {
-		sd_eprintf("failed to allocate memory");
+		sd_err("failed to allocate memory");
 		ret = SD_RES_NO_MEM;
 		goto out;
 	}
@@ -401,8 +397,8 @@ static int fill_vdi_info_range(uint32_t left, uint32_t right,
 		}
 
 		if (!strncmp(inode->name, name, strlen(inode->name))) {
-			sd_dprintf("%s = %s, %u = %u", iocb->tag, inode->tag,
-				   iocb->snapid, inode->snap_id);
+			sd_debug("%s = %s, %u = %u", iocb->tag, inode->tag,
+				 iocb->snapid, inode->snap_id);
 			if (vdi_has_tag(iocb)) {
 				/* Read, delete, clone on snapshots */
 				if (!vdi_is_snapshot(inode)) {
@@ -462,7 +458,7 @@ int vdi_lookup(struct vdi_iocb *iocb, struct vdi_info *info)
 
 	ret = get_vdi_bitmap_range(iocb->name, &left, &right);
 	info->free_bit = right;
-	sd_dprintf("%s left %lx right %lx, %x", iocb->name, left, right, ret);
+	sd_debug("%s left %lx right %lx, %x", iocb->name, left, right, ret);
 	switch (ret) {
 	case SD_RES_NO_VDI:
 	case SD_RES_FULL_VDI:
@@ -486,12 +482,26 @@ static int notify_vdi_add(uint32_t vdi_id, uint32_t nr_copies, uint32_t old_vid)
 
 	ret = exec_local_req(&hdr, NULL);
 	if (ret != SD_RES_SUCCESS)
-		sd_eprintf("fail to notify vdi add event(%" PRIx32 ", %d, %"
-			   PRIx32 ")", vdi_id, nr_copies, old_vid);
+		sd_err("fail to notify vdi add event(%" PRIx32 ", %d, %" PRIx32
+		       ")", vdi_id, nr_copies, old_vid);
 
 	return ret;
 }
 
+static void vdi_flush(uint32_t vid)
+{
+	struct sd_req hdr;
+	int ret;
+
+	sd_init_req(&hdr, SD_OP_FLUSH_VDI);
+	hdr.obj.oid = vid_to_vdi_oid(vid);
+
+	ret = exec_local_req(&hdr, NULL);
+	if (ret != SD_RES_SUCCESS)
+		sd_err("fail to flush vdi %" PRIx32 ", %s", vid,
+		       sd_strerror(ret));
+}
+
 /*
  * There are 3 create operation in SD:
  * 1. fresh create (expect NO_VDI returned from vdi_lookup)
@@ -512,25 +522,29 @@ int vdi_create(struct vdi_iocb *iocb, uint32_t *new_vid)
 	case SD_RES_SUCCESS:
 		if (!iocb->create_snapshot)
 			return SD_RES_VDI_EXIST;
+		if (sys->enable_object_cache)
+			vdi_flush(iocb->base_vid);
 		break;
 	case SD_RES_NO_VDI:
 		if (iocb->create_snapshot)
 			return ret;
 		break;
 	default:
-		sd_eprintf("%s", sd_strerror(ret));
+		sd_err("%s", sd_strerror(ret));
 		return ret;
 	}
 	if (!iocb->snapid)
 		iocb->snapid = 1;
 	*new_vid = info.free_bit;
-	notify_vdi_add(*new_vid, iocb->nr_copies, info.vid);
+	ret = notify_vdi_add(*new_vid, iocb->nr_copies, info.vid);
+	if (ret != SD_RES_SUCCESS)
+		return ret;
 
-	sd_dprintf("%s %s: size %" PRIu64 ", vid %" PRIx32 ", base %" PRIx32
-		   ", cur %" PRIx32 ", copies %d, snapid %"PRIu32,
-		   iocb->create_snapshot ? "snapshot" : "vdi", name, iocb->size,
-		   *new_vid, iocb->base_vid, info.vid, iocb->nr_copies,
-		   iocb->snapid);
+	sd_debug("%s %s: size %" PRIu64 ", vid %" PRIx32 ", base %" PRIx32
+		 ", cur %" PRIx32 ", copies %d, snapid %" PRIu32,
+		 iocb->create_snapshot ? "snapshot" : "vdi", name, iocb->size,
+		 *new_vid, iocb->base_vid, info.vid, iocb->nr_copies,
+		 iocb->snapid);
 
 	return create_vdi_obj(iocb, *new_vid, info.vid);
 }
@@ -615,8 +629,8 @@ static int notify_vdi_deletion(uint32_t vdi_id)
 
 	ret = exec_local_req(&hdr, &vdi_id);
 	if (ret != SD_RES_SUCCESS)
-		sd_eprintf("fail to notify vdi deletion(%" PRIx32 "), %d",
-			   vdi_id, ret);
+		sd_err("fail to notify vdi deletion(%" PRIx32 "), %d", vdi_id,
+		       ret);
 
 	return ret;
 }
@@ -628,11 +642,11 @@ static void delete_one(struct work *work)
 	int ret, i, nr_deleted;
 	struct sd_inode *inode = NULL;
 
-	sd_dprintf("%d %d, %16x", dw->done, dw->count, vdi_id);
+	sd_debug("%d %d, %16x", dw->done, dw->count, vdi_id);
 
 	inode = malloc(sizeof(*inode));
 	if (!inode) {
-		sd_eprintf("failed to allocate memory");
+		sd_err("failed to allocate memory");
 		return;
 	}
 
@@ -640,7 +654,7 @@ static void delete_one(struct work *work)
 			  (void *)inode, sizeof(*inode), 0);
 
 	if (ret != SD_RES_SUCCESS) {
-		sd_eprintf("cannot find VDI object");
+		sd_err("cannot find VDI object");
 		goto out;
 	}
 
@@ -656,22 +670,18 @@ static void delete_one(struct work *work)
 		oid = vid_to_data_oid(inode->data_vdi_id[i], i);
 
 		if (inode->data_vdi_id[i] != inode->vdi_id) {
-			sd_dprintf("object %" PRIx64 " is base's data, would"
-				   " not be deleted.", oid);
+			sd_debug("object %" PRIx64 " is base's data, would"
+				 " not be deleted.", oid);
 			continue;
 		}
 
 		ret = remove_object(oid);
 		if (ret != SD_RES_SUCCESS)
-			sd_eprintf("remove object %" PRIx64 " fail, %d", oid,
-				   ret);
+			sd_err("remove object %" PRIx64 " fail, %d", oid, ret);
 
 		nr_deleted++;
 	}
 
-	if (nr_deleted)
-		notify_vdi_deletion(vdi_id);
-
 	if (vdi_is_deleted(inode))
 		goto out;
 
@@ -680,6 +690,9 @@ static void delete_one(struct work *work)
 
 	write_object(vid_to_vdi_oid(vdi_id), (void *)inode,
 		     sizeof(*inode), 0, false);
+
+	if (nr_deleted)
+		notify_vdi_deletion(vdi_id);
 out:
 	free(inode);
 }
@@ -719,7 +732,7 @@ static int fill_vdi_list(struct deletion_work *dw, uint32_t root_vid)
 
 	inode = malloc(SD_INODE_HEADER_SIZE);
 	if (!inode) {
-		sd_eprintf("failed to allocate memory");
+		sd_err("failed to allocate memory");
 		goto err;
 	}
 
@@ -730,7 +743,7 @@ again:
 			  SD_INODE_HEADER_SIZE, 0);
 
 	if (ret != SD_RES_SUCCESS) {
-		sd_eprintf("cannot find VDI object");
+		sd_err("cannot find VDI object");
 		goto err;
 	}
 
@@ -763,7 +776,7 @@ static uint64_t get_vdi_root(uint32_t vid, bool *cloned)
 
 	inode = malloc(SD_INODE_HEADER_SIZE);
 	if (!inode) {
-		sd_eprintf("failed to allocate memory");
+		sd_err("failed to allocate memory");
 		vid = 0;
 		goto out;
 	}
@@ -774,13 +787,13 @@ next:
 	if (vid == inode->vdi_id && inode->snap_id == 1
 			&& inode->parent_vdi_id != 0
 			&& !inode->snap_ctime) {
-		sd_dprintf("vdi %" PRIx32 " is a cloned vdi.", vid);
+		sd_debug("vdi %" PRIx32 " is a cloned vdi.", vid);
 		/* current vdi is a cloned vdi */
 		*cloned = true;
 	}
 
 	if (ret != SD_RES_SUCCESS) {
-		sd_eprintf("cannot find VDI object");
+		sd_err("cannot find VDI object");
 		vid = 0;
 		goto out;
 	}
@@ -830,20 +843,19 @@ static int start_deletion(struct request *req, uint32_t vid)
 			dw->buf[0] = vid;
 			dw->count = 1;
 		} else {
-			sd_dprintf("snapshot chain has valid vdi, "
-				   "just mark vdi %" PRIx32 " as deleted.",
-				   dw->vid);
+			sd_debug("snapshot chain has valid vdi, just mark vdi %"
+				 PRIx32 " as deleted.", dw->vid);
 			delete_inode(dw);
 			return SD_RES_SUCCESS;
 		}
 	}
 
-	sd_dprintf("%d", dw->count);
+	sd_debug("%d", dw->count);
 
 	if (dw->count == 0)
 		goto out;
 
-	uatomic_inc(&req->refcnt);
+	refcount_inc(&req->refcnt);
 
 	if (list_empty(&deletion_work_list)) {
 		list_add_tail(&dw->list, &deletion_work_list);
@@ -929,7 +941,7 @@ int get_vdi_attr(struct sheepdog_vdi_attr *vattr, int data_len,
 		(*attrid)++;
 	}
 
-	sd_dprintf("there is no space for new VDIs");
+	sd_debug("there is no space for new VDIs");
 	ret = SD_RES_FULL_VDI;
 out:
 	return ret;
@@ -940,7 +952,7 @@ void clean_vdi_state(void)
 	struct rb_node *current_node = rb_first(&vdi_state_root);
 	struct vdi_state_entry *entry = NULL;
 
-	pthread_rwlock_wrlock(&vdi_state_lock);
+	sd_write_lock(&vdi_state_lock);
 	while (current_node) {
 		entry = rb_entry(current_node, struct vdi_state_entry, node);
 		rb_erase(current_node, &vdi_state_root);
@@ -949,5 +961,5 @@ void clean_vdi_state(void)
 		current_node = rb_first(&vdi_state_root);
 	}
 	INIT_RB_ROOT(&vdi_state_root);
-	pthread_rwlock_unlock(&vdi_state_lock);
+	sd_unlock(&vdi_state_lock);
 }
