@@ -36,7 +36,7 @@ static int sheepfs_debug;
 static int sheepfs_fg;
 int sheepfs_page_cache;
 int sheepfs_object_cache = true;
-char sdhost[32] = "localhost";
+char sdhost[32] = "127.0.0.1";
 int sdport = SD_LISTEN_PORT;
 
 static struct option const long_options[] = {
@@ -311,11 +311,10 @@ Options:\n\
 
 int main(int argc, char **argv)
 {
-	struct strbuf path = STRBUF_INIT;
+	struct strbuf path = STRBUF_INIT, *t;
 	int ch, longindex;
 	char *dir = NULL, *cwd;
 
-
 	while ((ch = getopt_long(argc, argv, short_options, long_options,
 				 &longindex)) >= 0) {
 		switch (ch) {
@@ -351,6 +350,13 @@ int main(int argc, char **argv)
 		}
 	}
 
+	t = sheepfs_run_cmd("dog");
+	if (!strlen(t->buf)) {
+		fprintf(stderr, "command dog not found\n");
+		exit(1);
+	}
+	strbuf_release(t);
+
 	if (optind != argc)
 		dir = argv[optind];
 	else
