@@ -62,7 +62,7 @@ size_t vdi_list_get_size(const char *path)
 	size_t len;
 	char cmd[COMMAND_LEN];
 
-	snprintf(cmd, sizeof(cmd), "collie vdi list -a %s -p %d",
+	snprintf(cmd, sizeof(cmd), "dog vdi list -a %s -p %d",
 		sdhost, sdport);
 	buf = sheepfs_run_cmd(cmd);
 	if (!buf)
