@@ -18,7 +18,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <time.h>
-#include <assert.h>
 #include <urcu/uatomic.h>
 #include <pthread.h>
 
@@ -63,7 +62,7 @@ struct vdi_inode {
 };
 
 static struct rb_root vdi_inode_tree = RB_ROOT;
-static pthread_rwlock_t vdi_inode_tree_lock = PTHREAD_RWLOCK_INITIALIZER;
+static struct sd_lock vdi_inode_tree_lock = SD_LOCK_INITIALIZER;
 
 static struct vdi_inode *vdi_inode_tree_insert(struct vdi_inode *new)
 {
@@ -148,9 +147,9 @@ static int volume_rw_object(char *buf, uint64_t oid, size_t size,
 	unsigned long idx = 0;
 	uint64_t cow_oid = 0;
 
-	pthread_rwlock_rdlock(&vdi_inode_tree_lock);
+	sd_read_lock(&vdi_inode_tree_lock);
 	vdi = vdi_inode_tree_search(vid);
-	pthread_rwlock_unlock(&vdi_inode_tree_lock);
+	sd_unlock(&vdi_inode_tree_lock);
 
 	if (is_data_obj(oid)) {
 		idx = data_oid_to_idx(oid);
@@ -194,7 +193,7 @@ static int volume_rw_object(char *buf, uint64_t oid, size_t size,
 		hdr.flags |= SD_FLAG_CMD_CACHE;
 
 	fd = get_socket_fd(vdi, &sock_idx);
-	ret = exec_req(fd, &hdr, buf, NULL, 0);
+	ret = exec_req(fd, &hdr, buf, NULL, 0, MAX_RETRY_COUNT);
 	put_socket_fd(vdi, sock_idx);
 
 	if (ret || rsp->result != SD_RES_SUCCESS) {
@@ -292,22 +291,28 @@ static int volume_do_sync(uint32_t vid)
 	int ret, fd, idx;
 	struct vdi_inode *vdi;
 
-	pthread_rwlock_rdlock(&vdi_inode_tree_lock);
+	sd_read_lock(&vdi_inode_tree_lock);
 	vdi = vdi_inode_tree_search(vid);
-	pthread_rwlock_unlock(&vdi_inode_tree_lock);
+	sd_unlock(&vdi_inode_tree_lock);
 
 	hdr.opcode = SD_OP_FLUSH_VDI;
 	hdr.obj.oid = vid_to_vdi_oid(vid);
 
 	fd = get_socket_fd(vdi, &idx);
-	ret = exec_req(fd, &hdr, NULL, NULL, 0);
+	ret = exec_req(fd, &hdr, NULL, NULL, 0, MAX_RETRY_COUNT);
 	put_socket_fd(vdi, idx);
 
+	/* if sheep daemon does not support FLUSH */
+	if (rsp->result == SD_RES_INVALID_PARMS) {
+		sheepfs_object_cache = false;
+		goto out;
+	}
+
 	if (ret || rsp->result != SD_RES_SUCCESS) {
 		sheepfs_pr("failed to flush vdi %"PRIx32"\n", vid);
 		return -1;
 	}
-
+out:
 	return 0;
 }
 
@@ -369,7 +374,7 @@ int reset_socket_pool(void)
 	struct vdi_inode *vdi;
 	int ret = 0;
 
-	pthread_rwlock_rdlock(&vdi_inode_tree_lock);
+	sd_read_lock(&vdi_inode_tree_lock);
 	for (node = rb_first(&vdi_inode_tree); node; node = rb_next(node)) {
 		vdi = rb_entry(node, struct vdi_inode, rb);
 		destroy_socket_pool(vdi->socket_pool, SOCKET_POOL_SIZE);
@@ -380,7 +385,7 @@ int reset_socket_pool(void)
 		}
 	}
 out:
-	pthread_rwlock_unlock(&vdi_inode_tree_lock);
+	sd_unlock(&vdi_inode_tree_lock);
 	return ret;
 }
 
@@ -391,7 +396,7 @@ static int init_vdi_info(const char *entry, uint32_t *vid, size_t *size)
 	struct vdi_inode *inode = NULL, *dummy;
 	char command[COMMAND_LEN];
 
-	snprintf(command, sizeof(command), "collie vdi list -r %s -a %s -p %d",
+	snprintf(command, sizeof(command), "dog vdi list -r %s -a %s -p %d",
 		 entry, sdhost, sdport);
 	buf = sheepfs_run_cmd(command);
 	if (!buf)
@@ -415,9 +420,9 @@ static int init_vdi_info(const char *entry, uint32_t *vid, size_t *size)
 		goto err;
 	}
 	/* we need insert inode before calling volume_rw_object */
-	pthread_rwlock_wrlock(&vdi_inode_tree_lock);
+	sd_write_lock(&vdi_inode_tree_lock);
 	dummy = vdi_inode_tree_insert(inode);
-	pthread_rwlock_unlock(&vdi_inode_tree_lock);
+	sd_unlock(&vdi_inode_tree_lock);
 	if (dummy)
 		goto err;
 	if (volume_rw_object(inode_buf, vid_to_vdi_oid(*vid), SD_INODE_SIZE,
@@ -479,15 +484,15 @@ static int volume_sync_and_delete(uint32_t vid)
 	int ret, fd, idx;
 	struct vdi_inode *vdi;
 
-	pthread_rwlock_rdlock(&vdi_inode_tree_lock);
+	sd_read_lock(&vdi_inode_tree_lock);
 	vdi = vdi_inode_tree_search(vid);
-	pthread_rwlock_unlock(&vdi_inode_tree_lock);
+	sd_unlock(&vdi_inode_tree_lock);
 
 	hdr.opcode = SD_OP_FLUSH_DEL_CACHE;
 	hdr.obj.oid = vid_to_vdi_oid(vid);
 
 	fd = get_socket_fd(vdi, &idx);
-	ret = exec_req(fd, &hdr, NULL, NULL, 0);
+	ret = exec_req(fd, &hdr, NULL, NULL, 0, MAX_RETRY_COUNT);
 	put_socket_fd(vdi, idx);
 
 	if (ret || rsp->result != SD_RES_SUCCESS) {
@@ -518,14 +523,14 @@ int volume_remove_entry(const char *entry)
 	if (sheepfs_object_cache && volume_sync_and_delete(vid) < 0)
 		return -1;
 
-	pthread_rwlock_rdlock(&vdi_inode_tree_lock);
+	sd_read_lock(&vdi_inode_tree_lock);
 	vdi = vdi_inode_tree_search(vid);
-	pthread_rwlock_unlock(&vdi_inode_tree_lock);
+	sd_unlock(&vdi_inode_tree_lock);
 	destroy_socket_pool(vdi->socket_pool, SOCKET_POOL_SIZE);
 
-	pthread_rwlock_wrlock(&vdi_inode_tree_lock);
+	sd_write_lock(&vdi_inode_tree_lock);
 	rb_erase(&vdi->rb, &vdi_inode_tree);
-	pthread_rwlock_unlock(&vdi_inode_tree_lock);
+	sd_unlock(&vdi_inode_tree_lock);
 
 	free(vdi->inode);
 	free(vdi);
