@@ -15,7 +15,6 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <errno.h>
-#include <assert.h>
 #include <getopt.h>
 
 #include <unistd.h>
@@ -63,19 +62,6 @@ struct sheep {
 };
 
 static LIST_HEAD(sheep_list_head);
-/*
- * nr_joined_sheep is a number of sheeps which is in state of
- * SHEEP_STATE_JOINED, not the length of sheep_list_head
- */
-static int nr_joined_sheep;
-
-/*
- * important invariant of shepherd: nr_joined_sheep ? !!master_sheep : true
- *
- * if there is at least one sheep which is in state of SHEEP_STATE_JOINED,
- * master sheep must be elected
- */
-static struct sheep *master_sheep;
 
 static bool running;
 static const char *progname;
@@ -118,61 +104,15 @@ static int remove_efd;
 
 static inline void remove_sheep(struct sheep *sheep)
 {
-	int ret;
-
-	sd_dprintf("remove_sheep() called, removing %s",
-		node_to_str(&sheep->node));
-
-	if (sheep->state == SHEEP_STATE_JOINED)
-		nr_joined_sheep--;
+	sd_debug("remove_sheep() called, removing %s",
+		 node_to_str(&sheep->node));
 
 	sheep->state = SHEEP_STATE_LEAVING;
-	ret = eventfd_write(remove_efd, 1);
-	if (ret < 0)
-		panic("eventfd_write() failed: %m");
+	eventfd_xwrite(remove_efd, 1);
 
 	event_force_refresh();
 }
 
-static int master_election(void)
-{
-	int ret, nr_failed = 0;
-	struct sheep *s;
-	struct sph_msg msg;
-
-	assert(!master_sheep);
-
-	if (!nr_joined_sheep)
-		return 0;
-
-	list_for_each_entry(s, &sheep_list_head, sheep_list) {
-		if (s->state != SHEEP_STATE_JOINED)
-			continue;
-
-		msg.type = SPH_SRV_MSG_MASTER_ELECTION;
-		msg.body_len = 0;
-
-		ret = xwrite(s->fd, &msg, sizeof(msg));
-		if (sizeof(msg) != ret) {
-			sd_eprintf("xwrite() for failed: %m");
-			goto election_failed;
-		}
-
-		master_sheep = s;
-		break;
-election_failed:
-		remove_sheep(s);
-		nr_failed++;
-	}
-
-	if (master_sheep) {
-		sd_iprintf("new master elected: %s",
-			node_to_str(&master_sheep->node));
-	}
-
-	return nr_failed;
-}
-
 static int notify_remove_sheep(struct sheep *leaving)
 {
 	int ret, failed = 0;
@@ -190,7 +130,7 @@ static int notify_remove_sheep(struct sheep *leaving)
 			&leaving->node, sizeof(struct sd_node));
 
 		if (sizeof(snd) + sizeof(struct sd_node) != ret) {
-			sd_eprintf("writev2() failed: %m");
+			sd_err("writev2() failed: %m");
 
 			remove_sheep(s);
 			failed++;
@@ -203,32 +143,17 @@ static int notify_remove_sheep(struct sheep *leaving)
 static void remove_handler(int fd, int events, void *data)
 {
 	struct sheep *s;
-	int ret, failed = 0;
-	eventfd_t val;
-	bool election = false;
-
-	ret = eventfd_read(remove_efd, &val);
-	if (ret < 0)
-		panic("eventfd_read() failed: %m");
-
-	sd_dprintf("removed sheeps: %" PRIu64, val);
-	assert(0 < val);
+	int nr_removed, failed = 0;
 
+	nr_removed = eventfd_xread(remove_efd);
 
+	sd_debug("removed sheeps");
 remove:
 	list_for_each_entry(s, &sheep_list_head, sheep_list) {
 		if (s->state != SHEEP_STATE_LEAVING)
 			continue;
 
-		sd_printf(SDOG_DEBUG, "removing the node: %s",
-			node_to_str(&s->node));
-
-		if (s == master_sheep) {
-			sd_printf(SDOG_DEBUG, "removing the master");
-
-			master_sheep = NULL;
-			election = true;
-		}
+		sd_debug("removing the node: %s", node_to_str(&s->node));
 
 		if (!is_sd_node_zero(&s->node))
 			/*
@@ -250,7 +175,7 @@ remove:
 	goto end;
 
 del:
-	sd_iprintf("removed node: %s", node_to_str(&s->node));
+	sd_info("removed node: %s", node_to_str(&s->node));
 
 	unregister_event(s->fd);
 	close(s->fd);
@@ -259,20 +184,11 @@ del:
 	list_del(&s->join_wait_list);
 	free(s);
 
-	event_force_refresh();
-
-	if (--val)
+	if (--nr_removed)
 		goto remove;
 
 end:
-	if (election) {
-		sd_dprintf("master is removed, electing new master");
-		failed = master_election();
-
-		assert(nr_joined_sheep ? !!master_sheep : true);
-	}
-
-	sd_dprintf("nodes which failed during remove_handler(): %d", failed);
+	sd_debug("nodes which failed during remove_handler(): %d", failed);
 }
 
 static LIST_HEAD(join_wait_queue);
@@ -298,10 +214,10 @@ retry:
 
 	wbytes = xwrite(waiting->fd, &snd, sizeof(snd));
 	if (sizeof(snd) != wbytes) {
-		sd_printf(SDOG_ERR, "xwrite() failed: %m");
+		sd_err("xwrite() failed: %m");
 		remove_sheep(waiting);
 
-		sd_iprintf("node %s is failed to join",
+		sd_info("node %s is failed to join",
 			node_to_str(&waiting->node));
 		nr_failed++;
 
@@ -325,47 +241,42 @@ static void sph_handle_join(struct sph_msg *msg, struct sheep *sheep)
 		buf = xzalloc(msg->body_len);
 		rbytes = xread(fd, buf, msg->body_len);
 		if (rbytes != msg->body_len) {
-			sd_eprintf("xread() failed: %m");
+			sd_err("xread() failed: %m");
 			goto purge_current_sheep;
 		}
 		free(buf);
 
 		list_add(&sheep->join_wait_list, &join_wait_queue);
 
-		sd_dprintf("there is already a joining sheep");
+		sd_debug("there is already a joining sheep");
 		return;
 	}
 
 	join = xzalloc(msg->body_len);
 	rbytes = xread(fd, join, msg->body_len);
 	if (msg->body_len != rbytes) {
-		sd_eprintf("xread() failed: %m");
+		sd_err("xread() failed: %m");
 		free(join);
 		goto purge_current_sheep;
 	}
 
-	sheep->node = join->node;
+	sheep->node = join->new_node;
+	join->nr_nodes = build_node_array(join->nodes);
 
 	snd.type = SPH_SRV_MSG_NEW_NODE;
 	snd.body_len = msg->body_len;
 
-	if (!nr_joined_sheep) {
-		/* this sheep is a new master */
-		/* FIXME: is this master_elected need? */
-		join->master_elected = true;
+	/* elect one node from the already joined nodes */
+	if (join->nr_nodes > 0) {
+		struct sd_node *n = join->nodes + rand() % join->nr_nodes;
+		fd = find_sheep_by_nid(&n->nid)->fd;
 	}
 
-	assert(nr_joined_sheep ? !!master_sheep : true);
-
-	wbytes = writev2(!nr_joined_sheep ? fd : master_sheep->fd,
-			&snd, join, msg->body_len);
+	wbytes = writev2(fd, &snd, join, msg->body_len);
 	free(join);
 
 	if (sizeof(snd) + msg->body_len != wbytes) {
-		sd_eprintf("writev2() failed: %m");
-
-		if (nr_joined_sheep)
-			remove_sheep(master_sheep);
+		sd_err("writev2() failed: %m");
 
 		goto purge_current_sheep;
 	}
@@ -377,7 +288,7 @@ purge_current_sheep:
 	remove_sheep(sheep);
 }
 
-static void sph_handle_new_node_reply(struct sph_msg *msg, struct sheep *sheep)
+static void sph_handle_accept(struct sph_msg *msg, struct sheep *sheep)
 {
 	int fd = sheep->fd, removed = 0;
 	ssize_t rbytes, wbytes;
@@ -391,46 +302,27 @@ static void sph_handle_new_node_reply(struct sph_msg *msg, struct sheep *sheep)
 	struct sph_msg_join_reply *join_reply_body;
 	struct sph_msg_join_node_finish *join_node_finish;
 
-	enum cluster_join_result join_result;
-
-	if (nr_joined_sheep && sheep != master_sheep) {
-		sd_eprintf("sheep which is not a master replied "
-			"SPH_CLI_MSG_NEW_NODE_REPLY");
-		goto purge_current_sheep;
-	}
-
-	sd_dprintf("new node reply from %s", node_to_str(&sheep->node));
+	sd_debug("new node reply from %s", node_to_str(&sheep->node));
 
 	join = xzalloc(msg->body_len);
 	rbytes = xread(fd, join, msg->body_len);
 	if (msg->body_len != rbytes) {
-		sd_eprintf("xread() failed: %m");
+		sd_err("xread() failed: %m");
 		free(join);
 
 		goto purge_current_sheep;
 	}
 
-	join_result = join->res;
+	sd_debug("joining node is %s", node_to_str(&join->new_node));
 
-	sd_dprintf("joining node is %s", node_to_str(&join->node));
-
-	joining_sheep = find_sheep_by_nid(&join->node.nid);
-	if (!joining_sheep) {
-		/* master is broken */
-		sd_eprintf("invalid nid is required, %s",
-			node_to_str(&join->node));
-		sd_eprintf("purging master sheep: %s and joining one",
-			node_to_str(&master_sheep->node));
-
-		remove_sheep(master_sheep);
-		goto purge_current_sheep;
-	}
+	joining_sheep = find_sheep_by_nid(&join->new_node.nid);
+	assert(joining_sheep != NULL);
 
 	opaque_len = msg->body_len - sizeof(struct sph_msg_join);
 	opaque = xzalloc(opaque_len);
 	memcpy(opaque, join->opaque, opaque_len);
 
-	sd_printf(SDOG_DEBUG, "length of opaque: %d", opaque_len);
+	sd_debug("length of opaque: %d", opaque_len);
 	memset(&snd, 0, sizeof(snd));
 	snd.type = SPH_SRV_MSG_JOIN_REPLY;
 	snd.body_len = sizeof(struct sph_msg_join_reply) + opaque_len;
@@ -445,7 +337,6 @@ static void sph_handle_new_node_reply(struct sph_msg *msg, struct sheep *sheep)
 	join_reply_body->nodes[join_reply_body->nr_nodes++] =
 		joining_sheep->node;
 	memcpy(join_reply_body->opaque, opaque, opaque_len);
-	join_reply_body->res = join_result;
 
 	wbytes = writev2(joining_sheep->fd, &snd,
 			join_reply_body, snd.body_len);
@@ -453,9 +344,8 @@ static void sph_handle_new_node_reply(struct sph_msg *msg, struct sheep *sheep)
 	free(join);
 
 	if (sizeof(snd) + snd.body_len != wbytes) {
-		sd_eprintf("writev2() to master failed: %m");
+		sd_err("writev2() to master failed: %m");
 
-		remove_sheep(master_sheep);
 		goto purge_current_sheep;
 	}
 
@@ -468,7 +358,6 @@ static void sph_handle_new_node_reply(struct sph_msg *msg, struct sheep *sheep)
 	join_node_finish->nr_nodes = build_node_array(join_node_finish->nodes);
 	join_node_finish->nodes[join_node_finish->nr_nodes++] =
 		joining_sheep->node;
-	join_node_finish->res = join_result;
 
 	list_for_each_entry(s, &sheep_list_head, sheep_list) {
 		if (s->state != SHEEP_STATE_JOINED)
@@ -480,7 +369,7 @@ static void sph_handle_new_node_reply(struct sph_msg *msg, struct sheep *sheep)
 		wbytes = writev2(s->fd, &snd, join_node_finish, snd.body_len);
 
 		if (sizeof(snd) + snd.body_len != wbytes) {
-			sd_eprintf("writev2() failed: %m");
+			sd_err("writev2() failed: %m");
 			remove_sheep(s);
 			removed++;
 		}
@@ -490,17 +379,7 @@ static void sph_handle_new_node_reply(struct sph_msg *msg, struct sheep *sheep)
 	free(opaque);
 
 	joining_sheep->state = SHEEP_STATE_JOINED;
-	nr_joined_sheep++;
 
-	if (nr_joined_sheep == 1) {
-		assert(!master_sheep);
-		assert(joining_sheep == sheep);
-
-		master_sheep = sheep;
-
-		sd_iprintf("new master elected: %s",
-			node_to_str(&sheep->node));
-	}
 	state = SPH_STATE_DEFAULT;
 
 	removed += release_joining_sheep();
@@ -526,7 +405,7 @@ static void sph_handle_notify(struct sph_msg *msg, struct sheep *sheep)
 	notify = xzalloc(msg->body_len);
 	rbytes = xread(fd, notify, msg->body_len);
 	if (rbytes != msg->body_len) {
-		sd_eprintf("xread() failed: %m");
+		sd_err("xread() failed: %m");
 		goto purge_current_sheep;
 	}
 
@@ -549,7 +428,7 @@ static void sph_handle_notify(struct sph_msg *msg, struct sheep *sheep)
 
 		wbytes = writev2(s->fd, &snd, notify_forward, snd.body_len);
 		if (sizeof(snd) + snd.body_len != wbytes) {
-			sd_eprintf("writev2() failed: %m");
+			sd_err("writev2() failed: %m");
 			goto notify_failed;
 		}
 
@@ -586,7 +465,7 @@ static void sph_handle_block(struct sph_msg *msg, struct sheep *sheep)
 		wbytes = writev2(s->fd, &snd,
 				&sheep->node, sizeof(struct sd_node));
 		if (sizeof(snd) + sizeof(struct sd_node) != wbytes) {
-			sd_eprintf("writev2() failed: %m");
+			sd_err("writev2() failed: %m");
 			goto block_failed;
 		}
 
@@ -605,7 +484,7 @@ static void sph_handle_leave(struct sph_msg *msg, struct sheep *sheep)
 	struct sheep *s;
 	struct sph_msg snd;
 
-	sd_iprintf("%s is leaving", node_to_str(&sheep->node));
+	sd_info("%s is leaving", node_to_str(&sheep->node));
 
 	memset(&snd, 0, sizeof(snd));
 	snd.type = SPH_SRV_MSG_LEAVE_FORWARD;
@@ -620,7 +499,7 @@ static void sph_handle_leave(struct sph_msg *msg, struct sheep *sheep)
 		wbytes = writev2(s->fd, &snd,
 				&sheep->node, sizeof(struct sd_node));
 		if (sizeof(snd) + sizeof(struct sd_node) != wbytes) {
-			sd_eprintf("writev2() failed: %m");
+			sd_err("writev2() failed: %m");
 			goto fwd_leave_failed;
 		}
 
@@ -633,7 +512,7 @@ fwd_leave_failed:
 
 static void (*msg_handlers[])(struct sph_msg*, struct sheep *) = {
 	[SPH_CLI_MSG_JOIN] = sph_handle_join,
-	[SPH_CLI_MSG_NEW_NODE_REPLY] = sph_handle_new_node_reply,
+	[SPH_CLI_MSG_ACCEPT] = sph_handle_accept,
 	[SPH_CLI_MSG_NOTIFY] = sph_handle_notify,
 	[SPH_CLI_MSG_BLOCK] = sph_handle_block,
 	[SPH_CLI_MSG_LEAVE] = sph_handle_leave,
@@ -648,26 +527,25 @@ static void read_msg_from_sheep(struct sheep *sheep)
 	ret = xread(sheep->fd, &rcv, sizeof(rcv));
 
 	if (ret != sizeof(rcv)) {
-		sd_eprintf("xread() failed: %m, ");
+		sd_err("xread() failed: %m, ");
 		goto remove;
 	}
 
 	if (!(0 <= rcv.type && rcv.type < ARRAY_SIZE(msg_handlers))) {
-		sd_eprintf("invalid message type: %d, ", rcv.type);
-		sd_eprintf("from node: %s", node_to_str(&sheep->node));
-		sd_eprintf("from node (sockaddr): %s",
-			sockaddr_in_to_str(&sheep->addr));
-		sd_eprintf("read bytes: %d, body length: %d",
-			ret, rcv.body_len);
+		sd_err("invalid message type: %d, ", rcv.type);
+		sd_err("from node: %s", node_to_str(&sheep->node));
+		sd_err("from node (sockaddr): %s",
+		       sockaddr_in_to_str(&sheep->addr));
+		sd_err("read bytes: %d, body length: %d", ret, rcv.body_len);
 		goto remove;
 	}
 
-	sd_dprintf("received op: %s", sph_cli_msg_to_str(rcv.type));
+	sd_debug("received op: %s", sph_cli_msg_to_str(rcv.type));
 
 	return msg_handlers[rcv.type](&rcv, sheep);
 
 remove:
-	sd_eprintf("removing node: %s", node_to_str(&sheep->node));
+	sd_err("removing node: %s", node_to_str(&sheep->node));
 	remove_sheep(sheep);
 }
 
@@ -676,8 +554,8 @@ static void sheep_comm_handler(int fd, int events, void *data)
 	if (events & EPOLLIN)
 		read_msg_from_sheep(data);
 	else if (events & EPOLLHUP || events & EPOLLERR) {
-		sd_eprintf("epoll() error: %s",
-			node_to_str(&((struct sheep *)data)->node));
+		sd_err("epoll() error: %s",
+		       node_to_str(&((struct sheep *)data)->node));
 		remove_sheep(data);
 	}
 }
@@ -694,18 +572,18 @@ static void sheep_accept_handler(int fd, int events, void *data)
 	len = sizeof(struct sockaddr_in);
 	new_sheep->fd = accept(fd, (struct sockaddr *)&new_sheep->addr, &len);
 	if (new_sheep->fd < 0) {
-		sd_eprintf("accept() failed: %m");
+		sd_err("accept() failed: %m");
 		goto clean;
 	}
 
 	if (-1 == set_keepalive(new_sheep->fd)) {
-		sd_eprintf("set_keepalive() failed: %m");
+		sd_err("set_keepalive() failed: %m");
 		goto clean;
 	}
 
 	ret = register_event(new_sheep->fd, sheep_comm_handler, new_sheep);
 	if (ret < 0) {
-		sd_eprintf("register_event() failed: %m");
+		sd_err("register_event() failed: %m");
 		goto clean;
 	}
 
@@ -714,7 +592,7 @@ static void sheep_accept_handler(int fd, int events, void *data)
 
 	INIT_LIST_HEAD(&new_sheep->join_wait_list);
 
-	sd_iprintf("accepted new sheep connection");
+	sd_info("accepted new sheep connection");
 	return;
 
 clean:
@@ -750,7 +628,7 @@ static void usage(void)
 
 static void exit_handler(void)
 {
-	sd_printf(SDOG_INFO, "exiting...");
+	sd_info("exiting...");
 }
 
 static int set_listen_fd_cb(int fd, void *data)
@@ -766,8 +644,7 @@ static int set_listen_fd_cb(int fd, void *data)
 
 static void crash_handler(int signo)
 {
-	sd_printf(SDOG_EMERG, "shepherd exits unexpectedly (%s).",
-		  strsignal(signo));
+	sd_emerg("shepherd exits unexpectedly (%s).", strsignal(signo));
 
 	sd_backtrace();
 
@@ -781,7 +658,7 @@ int main(int argc, char **argv)
 	bool daemonize = true;
 	int log_level = SDOG_INFO;
 	const char *log_file = "/var/log/shepherd.log";
-	const char *log_format = "default";
+	const char *log_format = "server";
 	struct logger_user_info shepherd_info;
 
 	int port = SHEPHERD_PORT;
@@ -825,12 +702,12 @@ int main(int argc, char **argv)
 		case 'p':
 			port = strtol(optarg, &p, 10);
 			if (p == optarg) {
-				fprintf(stderr, "invalid port: %s\n", optarg);
+				sd_err("invalid port: %s", optarg);
 				exit(1);
 			}
 			break;
 		default:
-			fprintf(stderr, "unknown option: %c\n", ch);
+			sd_err("unknown option: %c", ch);
 			usage();
 			exit(1);
 			break;
@@ -841,20 +718,15 @@ int main(int argc, char **argv)
 		ret = daemon(0, 0);
 
 		if (-1 == ret) {
-			fprintf(stderr, "daemon() failed: %m\n");
+			sd_err("daemon() failed: %m");
 			exit(1);
 		}
 	}
 
-	/*
-	 * early_log_init() must be called before any calling of
-	 * sd_printf() series
-	 */
 	shepherd_info.port = port;
 	early_log_init(log_format, &shepherd_info);
 
-	ret = log_init(progname, LOG_SPACE_SIZE, !daemonize,
-		log_level, (char *)log_file);
+	ret = log_init(progname, !daemonize, log_level, (char *)log_file);
 	if (ret)
 		panic("initialize logger failed: %m");
 
